Type: Package
Package: iotarelr
Title: Iota Inter Coder Reliability for Content Analysis
Version: 0.1.9
Authors@R: c(
    person("Berding", "Florian", , "florian.berding@uni-hamburg.de", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-3593-1695")),
    person("Pargmann", "Julia", , "julia.pargmann@uni-hamburg.de", role = "ctb",
           comment = c(ORCID = "0000-0003-3616-0172"))
  )
Description: Routines and tools for assessing the quality of content
    analysis on the basis of the Iota Reliability Concept. The concept is
    inspired by item response theory and can be applied to any kind of
    content analysis which uses a standardized coding scheme and discrete
    categories.  It is also applicable for content analysis conducted by
    artificial intelligence.  The package provides reliability measures
    for a complete scale as well as for every single category. Analysis of
    subgroup-invariance and error corrections are implemented. This
    information can support the development process of a coding scheme and
    allows a detailed inspection of the quality of the generated data.
    Equations and formulas working in this package are part of Berding et
    al. (2022)<doi:10.3389/feduc.2022.818365> and Berding and Pargmann
    (2022) <doi:10.30819/5581>.
License: GPL-3
URL: https://fberding.github.io/iotarelr/
BugReports: https://github.com/FBerding/iotarelr/issues
Depends: R (>= 3.5.0)
Imports: ggalluvial, ggplot2, gridExtra, methods, Rcpp, rlang, stats
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
LinkingTo: Rcpp
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-05-07 08:36:33 UTC; WissMit
Author: Berding Florian [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3593-1695>),
  Pargmann Julia [ctb] (ORCID: <https://orcid.org/0000-0003-3616-0172>)
Maintainer: Berding Florian <florian.berding@uni-hamburg.de>
Repository: CRAN
Date/Publication: 2026-05-07 12:40:34 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-07 14:20:51 UTC; unix
Archs: iotarelr.so.dSYM

# Generated by roxygen2: do not edit by hand

export(EM_algo_c)
export(check_conformity_c)
export(check_dgf)
export(check_new_rater)
export(compute_iota1)
export(compute_iota2)
export(est_con_multinominal_c)
export(est_expected_categories)
export(get_consequences)
export(get_iota2_measures)
export(get_patterns)
export(get_summary)
export(plot_iota)
export(plot_iota2_alluvial)
import(ggalluvial)
import(ggplot2)
importFrom(Rcpp,sourceCpp)
importFrom(methods,is)
importFrom(rlang,.data)
useDynLib(iotarelr)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# iotarelr - Iota Inter-Coder-Reliability for Content Analysis <img src="man/figures/logo.png" align="right" height="120" alt="" />

*iotarelr* provides routines and tools for assessing the quality of
content analysis on the basis of the Iota Reliability Concept. The
concept is inspired by item response theory and can be applied to any
kind of content analysis which uses a standardized coding scheme and
discrete categories. It is also applicable for content analysis
conducted by artificial intelligence. The package provides reliability
measures for the complete scale as well as for every single category.
Analysis of subgroup-invariance and error corrections are implemented.
This information can support the development process of a coding scheme
and allows a detailed inspection of the quality of the generated data.

The tools are able to provide answers to the following questions:

- How reliable is the scale of a coding scheme?
- How reliable is a single category within a scale?
- Does the coding scheme work similarly for different groups of
  materials or different groups of people?
- How do coding errors bias the generated data?
- How can the generated data be corrected for errors?

<!-- badges: start -->

[![](https://www.r-pkg.org/badges/version/iotarelr?color=green)](https://cran.r-project.org/package=iotarelr)
[![Downloads](http://cranlogs.r-pkg.org/badges/grand-total/iotarelr)](https://cran.r-project.org/package=iotarelr)
[![Downloads](http://cranlogs.r-pkg.org/badges/last-month/iotarelr?color=blue)](https://cran.r-project.org/package=iotarelr)
[![](https://img.shields.io/badge/devel%20version-0.1.9-green.svg)](https://github.com/fberding/iotarelr)
[![Project Status: Active - The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![Codecov test
coverage](https://codecov.io/gh/FBerding/iotarelr/graph/badge.svg)](https://app.codecov.io/gh/FBerding/iotarelr)
[![R-CMD-check](https://github.com/FBerding/iotarelr/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/FBerding/iotarelr/actions/workflows/R-CMD-check.yaml)
[![DOI](https://img.shields.io/badge/doi-10.32614/CRAN.package.iotarelr-green.svg)](https://doi.org/10.32614/CRAN.package.iotarelr)
<!-- badges: end -->

## Introduction

A brief introduction on how to use the package can be found via Get
started. Articles describing how to conduct advanced analysis can be
found via Articles.

## Installation

You can install the package from CRAN with:

``` r
install.packages("iotarelr")
```

You can install the development version of *iotarelr* from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("FBerding/iotarelr")
```

## Components

*iotarelr* calculates the following components of the *Iota Reliability
Concept*.

**On the level of every single category:**

- ***Assignment Error Matrix:*** Matrix containing the probabilities to
  assign a coding unit truly belonging to category i to category j.
- ***Alpha Reliability:*** Probability to assign a coding unit of
  category i to category i.
- ***Beta Reliability:*** Probability to assign a coding unit of
  category j to category i.
- ***Iota Reliability:***
  - ***Iota:*** Value ranging between 0 and 1, reflecting how well the
    generated data of category i really reflects category i.
  - ***Iota Error I:*** Value describing how many coding units are
    missing in the data for category i.
  - ***Iota Error II:*** Value describing how many coding units from
    *other* categories are part of the data for category i.

**On the scale level:**

- ***Iota Index:*** Measure for describing the reliability of a scale.
  Zero indicates the absence of reliability. One indicates perfect
  reliability.
- ***Dynamic Iota Index:*** Transformation of the original *Iota Index*
  to account for the uncertainty of reliability estimation. Zero
  indicates the absence of reliability. One indicates perfect
  reliability.

## Accuracy of Reliability Estimates

The parameter estimation of the components makes use of Maximum
Likelihood Estimation (Expectation Maximization Algorithm), which
comprises an additional conditioning stage. The following figure shows
the extent to which the estimated parameters correspond to their true
values based on a simulation study (Berding & Pargmann 2022).

<figure>
<img src="man/figures/README-accuracy.png"
alt="Figure 1 - Accuracy of Parameter Estimation" />
<figcaption aria-hidden="true">Figure 1 - Accuracy of Parameter
Estimation</figcaption>
</figure>

In general, the parameter (Primary Parameters, Alpha and Beta
Reliability) estimates do not deviate from their true values by more
than 5 percentage points (see median in the figure). In most cases,
*Iota Index* deviates no more than .043 from its true value. The
estimates are more accurate, the greater the sample size and the more
raters are involved in coding. Furthermore, the estimates are more
accurate for higher values of true reliability.

Please note that the high deviation for Beta Reliability under the
condition of very high reliability results from rare cases with perfect
Alpha Reliability. For more details please refer to Berding & Pargmann
(2022).

## Predictive Power

Studies investigated the power of the Iota Concept for predicting the
quality of data generated by content analysis and compared the Iota
measures with other existing measures of inter-coder reliability. The
figure shows *R²* for both nominal and ordinal data.

<figure>
<img src="man/figures/README-pred_power_nominal.png"
alt="Figure 2 - Predictive Power for Nominal Data" />
<figcaption aria-hidden="true">Figure 2 - Predictive Power for Nominal
Data</figcaption>
</figure>

For the case of nominal data, the *Dynamic Iota Index* performs
similarly or even better to *Krippendorff’s Alpha* when predicting the
deviation between the estimated sample association and the true sample
association. Alike applies for estimating the risk of Type I errors and
the chance for correctly classifying the effect size into categories
proposed by Cohen (1988).

<figure>
<img src="man/figures/README-pred_power_ordinal.png"
alt="Figure 3 - Predictive Power for Ordinal Data" />
<figcaption aria-hidden="true">Figure 3 - Predictive Power for Ordinal
Data</figcaption>
</figure>

In the case of ordinal data, the *Dynamic Iota Index* shows a slightly
inferior performance compared to *Krippendorff’s Alpha*, but the
predictive power of both measures remains very high.

## Cut-Off Values

The Iota Concept provides cut-off values for several measures on the
scale level. The following table reports the values that are currently
recommended.

| measure            | minimum | satisfactory | good  | excellent |
|:-------------------|:--------|:-------------|:------|:----------|
| Dynamic Iota Index | 0.829   | 0.961        | 0.985 | 1\*       |
| Static Iota Index  | 0.686   | 0.853        | 0.898 | 1\*       |
| Average Iota       | 0.693   | 0.847        | 0.875 | 1\*       |
| Minimum Iota       | 0.623   | 0.785        | 0.812 | 1\*       |

The *minimal* values imply the expectation that the estimated and true
association/correlation do not deviate more than .3. Furthermore, these
values justify the expectation that the risk of Type I errors is less
than 10%.

The cut-off value for *satisfactory* justifies the expectation that the
estimated and true association/correlation do not deviate more than .1
and that the risk of Type I errors is less than 5%.

The values for *good* imply that the estimated and true
association/correlation do not deviate more than .3 with a certainty of
95%. Furthermore, these values guarantee that the risk of Type I errors
is less than 10 % with a certainty of 95%.

Values of the category *excellent* ensure with a certainty of 95% that
the risk of Type I errors is less than 5% and the deviation between
estimated and true sample association/correlation exceeds not more than
.1. Please note that this degree of certainty is not completely reached
for all measures.

The presented cut-off values are only rules of thumb. They are derived
from the situations demanding the most reliability. More
situation-specific cut-off values can be calculated with the function
`get_consequences()`. Please refer to the vignette “Calculating
consequences and cut-off values”.

## Further Readings

- Florian Berding and Julia Pargmann (2022). Iota Reliability Concept of
  the Second Generation. Measures for Content Analysis Done by Humans or
  Artificial Intelligence. Berlin: Logos.
  <https://doi.org/10.30819/5581>

- Florian Berding, Elisabeth Riebenbauer, Simone Stuetz, Heike Jahncke,
  Andreas Slopinski, and Karin Rebmann (2022). Performance and
  Configuration of Artificial Intelligence in Educational Settings.
  Introducing a New Reliability Concept Based on Content Analysis.
  Frontiers in Education. <https://doi.org/10.3389/feduc.2022.818365>

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
library(iotarelr)
head(iotarelr_written_exams)

## -----------------------------------------------------------------------------
res_iota2<-compute_iota2(
  data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")],
  random_starts = 10,
  trace = FALSE)
expected_categories<-est_expected_categories(
  data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")],
  aem=res_iota2$categorical_level$raw_estimates$assignment_error_matrix)
head(expected_categories)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(iotarelr)
res_iota2<-compute_iota2(data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")],
                         random_starts = 10,
                         trace = FALSE)
expected_categories<-est_expected_categories(
  data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")],
  aem=res_iota2$categorical_level$raw_estimates$assignment_error_matrix)
true_values<-expected_categories$expected_category

## -----------------------------------------------------------------------------
res_new_rater<-check_new_rater(
  true_values = true_values,
  assigned_values = iotarelr_new_rater)

## -----------------------------------------------------------------------------
get_summary(res_new_rater)

## ----fig.height = 3, fig.width = 7.2, fig.align = "center", fig.cap="Figure 1: Plot of Iota for a New Rater"----
plot_iota(res_new_rater)


## ----fig.height = 5, fig.width = 7.2, fig.align = "center", fig.cap="Figure 2: Stream Plot"----
plot_alluvial_new_rater<-plot_iota2_alluvial(res_new_rater)
plot(plot_alluvial_new_rater)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
library(iotarelr)
head(iotarelr_written_exams)

## -----------------------------------------------------------------------------
dgf_exam<-check_dgf(data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")],
                    splitcr = iotarelr_written_exams$Sex,
                    random_starts = 300)

## -----------------------------------------------------------------------------
get_summary(dgf_exam$group_female)

## -----------------------------------------------------------------------------
get_summary(dgf_exam$group_male)

## ----fig.height = 6, fig.width = 7.2, fig.align = "center", fig.cap = "Figure 1: Plot of Iota for different Groups"----
plot_iota(dgf_exam,
          ylab = "Groups")

## ----fig.height = 5, fig.width = 7.2, fig.align = "center", fig.cap="Figure 2: Stream Plot for Females"----
plot_alluvial_females<-plot_iota2_alluvial(
  dgf_exam$group_female,
  label_titel = "Coding Stream from True to Assigned Categories (Females)")
plot(plot_alluvial_females)

## ----fig.height = 5, fig.width = 7.2, fig.align = "center", fig.cap="Figure 3: Stream Plot for Males"----
plot_alluvial_males<-plot_iota2_alluvial(
  dgf_exam$group_male,
  label_titel = "Coding Stream from True to Assigned Categories (Males)")
plot(plot_alluvial_males)

# This file is part of the R package "iotarelr".
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as published by
# the Free Software Foundation.
#
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>



#' Computes Iota and its elements in version 1
#'
#' Computes all elements of the Iota Reliability Concept
#'
#' @param data Data for which the elements should be estimated. Data must be
#' an object of type \code{data.frame} or \code{matrix} with  cases in the rows and
#' raters in the columns.
#' @return A list with the following components
#' \item{alpha}{A vector containing the chance-corrected Alpha Reliabilities for
#' every category.}
#' \item{beta}{A vector containing the chance-corrected Beta Reliabilities for
#' every category.}
#' \item{iota}{A vector containing the Iota values for every category.}
#' \item{assignment_error_matrix}{A matrix with the conditional probabilities
#' for every category. The rows refer to the true categories and the columns
#' refer to the assigned categories. The elements on the diagonal represent the
#' alpha errors of that category. The other elements in each row represent the
#' conditioned probabilities that a coding unit is wrongly assigned to another
#' category.}
#' \item{average_iota}{A numeric value ranging between 0 and 1, representing the
#' Average Iota values on a categorical level. It describes the reliability of
#' the whole scale.}
#' @references -   Berding, Florian, Elisabeth Riebenbauer, Simone Stuetz, Heike
#'Jahncke, Andreas Slopinski, and Karin Rebmann. 2022. Performance and
#'Configuration of Artificial Intelligence in Educational
#'Settings.Introducing a New Reliability Concept Based on Content
#'Analysis. Frontiers in Education.
#'https://doi.org/10.3389/feduc.2022.818365
#' @export

compute_iota1<-function(data){
  data<-as.data.frame(data)
  categorical_levels<-names(table(data[1]))
  for(i in 2:ncol(data)){
    tmp<-names(table(data[i]))
    for(j in tmp)
      if(j %in% categorical_levels==FALSE){
        categorical_levels<-c(categorical_levels,j)
      }
  }

  for (i in 1:ncol(data)){
    data[[i]]<-factor(x=lapply(data[[i]],as.character),levels=categorical_levels)
  }
  categorical_levels<-sort(categorical_levels,decreasing = FALSE)
  n_rater=ncol(data)
  n_categories<-length(categorical_levels)
  N=nrow(data)
  n_rater=ncol(data)
  p_cell<-1/(n_categories^n_rater)

  N_Aggrement_Table<-matrix(nrow=1,ncol=n_categories,data=NA)
  colnames(N_Aggrement_Table)<-categorical_levels
  for (k in categorical_levels){
    aggreement<-data[,1:n_rater]==k
    aggreement<-replace(x=aggreement,aggreement==TRUE,1)
    aggreement<-rowSums(aggreement)
    aggreement<-subset(aggreement,aggreement==n_rater)
    aggreement<-length(aggreement)
    N_Aggrement_Table[1,k]<-aggreement
  }

  P_Condition<-matrix(nrow=1,ncol=n_categories,data=NA)
  colnames(P_Condition)<-categorical_levels

  N_Condition<-matrix(nrow=1,ncol=n_categories,data=0)
  colnames(N_Condition)<-categorical_levels

  P_Marginal<-matrix(nrow=n_categories,ncol = n_rater,data=NA)
  rownames(P_Marginal)<-categorical_levels

  for (k in categorical_levels){
    for (r in 1:n_rater){
      #Count the frequencies of k for every rater and sum
      N_Condition_Category<-nrow(subset(data,data[r]==k))
      N_Condition[1,k]<-N_Condition[1,k]+N_Condition_Category
      P_Marginal[k,r]<-nrow(subset(data[r],data[r]==k))/N
    }
    #Correction for over-counting the diagonal cells
    N_Condition[1,k]<-N_Condition[1,k]-(n_rater-1)*N_Aggrement_Table[1,k]
  }
  P_Condition<-N_Condition/N
  P_Alpha_Reliability<-(N_Aggrement_Table/N)/P_Condition
  P_Alpha_Error<-1-P_Alpha_Reliability

  #Estimation of beta-Error
  P_Condition_Beta<-matrix(nrow=1,ncol=n_categories,data=NA)
  colnames(P_Condition_Beta)<-categorical_levels

  N_Condition_Beta<-matrix(nrow=1,ncol=n_categories,data=NA)
  colnames(N_Condition_Beta)<-categorical_levels

  N_Target_Beta<-matrix(nrow=1,ncol=n_categories,data=NA)
  colnames(N_Target_Beta)<-categorical_levels

  for (k in categorical_levels){
    categories<-categorical_levels
    categories<-subset(categories,categories!=k)
    N_Condition_Beta[1,k]<-N-sum(N_Aggrement_Table[1,])

    N_Combination<-data[,1:n_rater]!=k
    N_Combination<-replace(x=N_Combination,N_Combination==TRUE,1)
    N_Combination<-rowSums(N_Combination)
    N_Combination<-subset(N_Combination,N_Combination==n_rater)
    N_Combination<-length(N_Combination)

    N_Target_Beta[1,k]<-N-N_Combination-N_Aggrement_Table[1,k]
  }
  P_Condition_Beta<-N_Condition_Beta/N
  P_Target_Beta<-N_Target_Beta/N
  P_Beta_Error<-P_Target_Beta/P_Condition_Beta
  P_Beta_Reliability<-1-P_Beta_Error

  #Estimating the Assignment-Error-Matrix
  assignment_error_matrix<-matrix(nrow=n_categories,ncol=n_categories,data=NA)
  rownames(assignment_error_matrix)<-categorical_levels
  colnames(assignment_error_matrix)<-categorical_levels

  for(k in categorical_levels){
    assignment_error_matrix[k,k]<-P_Alpha_Error[1,k]
    categories<-categorical_levels
    categories<-subset(categories,categories!=k)
    for(k2 in categories){
      assignment_error_matrix[k,k2]<-(P_Alpha_Error[1,k]*P_Beta_Error[1,k2])/(P_Alpha_Error[1,k]*sum(P_Beta_Error[1,categories]))
    }
  }


  #Estimating normalized and chance-corrected alpha values for iota
  P_No_K<-p_cell*(n_categories-1)^n_rater
  P_Diagonal<-n_categories*p_cell

  Alpha_Rel_Chance<-p_cell/(1-P_No_K)
  Alpha_Error_Chance<-1-Alpha_Rel_Chance
  alpha<-abs((P_Alpha_Reliability-Alpha_Rel_Chance)/(1-Alpha_Rel_Chance))

  #Estimating normalized and chance-corrected beta values for iota
  P_Beta_Error_Chance_Target<-1-P_No_K-p_cell
  Beta_Error_Chance<-P_Beta_Error_Chance_Target/(1-P_Diagonal)
  Beta_Rel_Chance<-1-Beta_Error_Chance

  Realized_Beta_Error<-P_Condition_Beta*P_Beta_Error
  Realized_Beta_Error_Chance<-(1-P_Diagonal)*Beta_Error_Chance

  Realized_Beta_Reliability<-1-Realized_Beta_Error
  Realized_Beta_Reliability_Chance<-1-Realized_Beta_Error_Chance


  beta<-abs((Realized_Beta_Reliability-Realized_Beta_Reliability_Chance)/(1-Realized_Beta_Reliability_Chance))

  beta<-replace(x=beta,beta=="NaN",values = 1)
  beta<-replace(x=beta,beta==NaN,values=1)
  beta<-replace(x=beta,beta=="Inf",values=1)
  beta<-replace(x=beta,beta==Inf,values=1)
  beta<-replace(x=beta,beta=="-Inf",values=1)
  beta<-replace(x=beta,beta==-Inf,values=1)
  beta<-replace(x=beta,beta=="NA",values=1)
  beta<-replace(x=beta,beta==NA,values=1)
  #test_abc<-((1-n_categories*p_cell)*Beta_Error_Chance-P_Condition_Beta*P_Beta_Error)/((1-n_categories*p_cell)*Beta_Error_Chance)
  #print(round(test_abc,3)==round(Beta_Rel,3))



  reliability<-(alpha+beta)/2

  results<-NULL
  results["alpha"]<-list(alpha)
  results["beta"]<-list(beta)
  results["iota"]<-list(reliability)
  results["assignment_error_matrix"]<-list(assignment_error_matrix)
  results["average_iota"]<-mean(reliability)

  class(results)<-"iotarelr_iota1"

  return(results)
}

# This file is part of the R package "iotarelr".
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as published by
# the Free Software Foundation.
#
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>


#'Plot Iota2
#'
#'Function for creating a plot object that can be plotted via 'ggplot2'.
#'
#'@param object Estimates of Iota 2 created with \code{compute_iota2()},
#'\code{check_dgf()} or \code{check_new_rater()}.
#'@param xlab \code{Character} passed to xlab() from scale_fill_manual(). Label
#'of the x-axis.
#'@param ylab \code{Character} passed to ylab() from scale_fill_manual(). Label
#'of the y-axis.
#'@param liota \code{Character} passed to labels() from scale_fill_manual().
#'Label for Iota.Amount of cases that are assigned to the correct category.
#'@param lcase2 \code{Character} passed to labels() from scale_fill_manual().
#'Label for the amount of cases that are assigned to a false category.
#'@param lcase3 \code{Character} passed to labels() from scale_fill_manual().
#'Label for the amount of cases that are assigned from a false category.
#'@param lscale_quality \code{character} passed to scale_fill_manual() determining
#'the title for the quality of a scale. Only used in conjunction with
#'\code{scale}.
#'@param lscale_cat Vector of strings with length 5. This vector contains the
#'labels for each category of quality for the scale.
#'@param number_size \code{Double} passed to geom_text() determining the size
#'of the numbers within the plot.
#'@param key_size \code{Double} passed to theme() determining the size of the
#'legend keys.
#'@param text_size \code{Double} passed to theme() determining the size of the
#'text within the legend.
#'@param scale \code{String} for requesting an additional plot of reliability
#'on the scale level. If \code{scale="dynamic_iota_index"} Dynamic Iota Index
#'is used.  If \code{scale="static_iota_index"} Static Iota Index
#'is used. If \code{scale="none"} no additional plot is created.
#'@param legend_position \code{string} Position of the legend. Possible values
#'are "bottom","right","left", "top" and "none".
#'@param legend_direction \code{string} Layout of the items in the legend. Possible
#'values are "horizontal" and "vertical".
#'@return Function returns an object of class \code{gg, ggplot} illustrating how
#'the data of the different categories influence each other.
#'@importFrom rlang .data
#'@importFrom methods is
#'@references  Florian Berding and Julia Pargmann (2022).Iota Reliability Concept
#'of the Second Generation. Measures for Content Analysis Done by
#'Humans or Artificial Intelligences. Berlin: Logos.
#'https://doi.org/10.30819/5581
#'@export
plot_iota<-function(object,
                    xlab="Amount on all cases",
                    ylab="Categories",
                    liota="Assignment of the true category (Iota)",
                    lcase2="Assignment to the false category",
                    lcase3="Assignment from the false true category",
                    lscale_quality="Scale Quality",
                    lscale_cat=c("insufficent",
                                 "minimum",
                                 "satisfactory",
                                 "good",
                                 "excellent"),
                    number_size=6,
                    key_size=0.5,
                    text_size=10,
                    legend_position="bottom",
                    legend_direction="vertical",
                    scale="none"){

  if((methods::is(object,"iotarelr_iota2") | methods::is(object,"iotarelr_iota2_dgf"))==FALSE){
    stop("Class of object for iota is not supported by this function.")
  }
  if(legend_direction%in%c("horizontal","vertical")==FALSE){
    stop("legend_direction must be 'horizontal' or 'vertical'.")
  }
  if(legend_position%in%c("bottom","top","right","left","none")==FALSE){
    stop("legend_direction must be 'bottom', 'top', 'left', 'right' or 'none'.")
  }

  image_data<-NULL
  if(methods::is(object,"iotarelr_iota2")){
    n_group=1
  } else if(methods::is(object,"iotarelr_iota2_dgf")){
    n_group<-length(object)
  }

  for(g in 1:n_group){
    if(methods::is(object,"iotarelr_iota2")){
      tmp_object<-object
    } else if(methods::is(object,"iotarelr_iota2_dgf")){
      tmp_object<-object[[g]]
    }

    p_classes=tmp_object$information$est_true_cat_sizes
    p_alpha_reliability=tmp_object$categorical_level$raw_estimates$alpha_reliability
    p_alpha_error=1-p_alpha_reliability
    n_categories=length(p_classes)
    p_beta_error<-1-tmp_object$categorical_level$raw_estimates$beta_reliability

    beta_error_sum<-vector(length = n_categories)
    for(i in 1:n_categories){
      beta_error_sum[i]<-sum(p_classes*p_alpha_error)-
        p_classes[i]*p_alpha_error[i]
    }
    iota_numerator<-p_classes*p_alpha_reliability
    iota_denominator<-p_classes*p_alpha_reliability+
      p_classes*p_alpha_error+
      p_beta_error*beta_error_sum

    iota<-iota_numerator/iota_denominator
    case_2<-p_alpha_error*p_classes/iota_denominator
    case_3<-p_beta_error*beta_error_sum/iota_denominator

    names(iota)<-names(p_classes)
    names(case_2)<-names(p_classes)
    names(case_3)<-names(p_classes)

    tmp<-t(rbind(iota,case_2,case_3))

    tmp_image_data<-NULL
    for(i in 1:nrow(tmp)){
      for (j in 1:3){
        if(methods::is(object,"iotarelr_iota2")){
          tmp_category=rownames(tmp)[i]
          tmp_group<-NA
        } else if(methods::is(object,"iotarelr_iota2_dgf")) {
          tmp_category=rownames(tmp)[i]
          tmp_group<-names(object)[g]
        }

        tmp_image_data<-rbind(tmp_image_data,c(tmp_category,
                                               tmp[i,j],
                                               tmp[i,j]/sum(tmp[i,]),
                                               sum(tmp[i,0:(j-1)])/sum(tmp[i,]),
                                               sum(tmp[i,1:j])/sum(tmp[i,]),
                                               colnames(tmp)[j],
                                               tmp_group)
        )
      }
    }
    tmp_image_data<-as.data.frame(tmp_image_data)
    colnames(tmp_image_data)<-c("category","value", "amount","position_l","position_r","case","group")
    tmp_image_data$amount<-as.numeric(tmp_image_data$amount)
    tmp_image_data$value<-as.numeric(tmp_image_data$value)
    tmp_image_data$position_l<-as.numeric(tmp_image_data$position_l)
    tmp_image_data$position_r<-as.numeric(tmp_image_data$position_r)
    tmp_image_data$case<-factor(as.character(tmp_image_data$case),
                                levels=c("case_3",
                                         "case_2",
                                         "iota"))
    image_data<-rbind(image_data,tmp_image_data)
  }


  image_plot<-ggplot2::ggplot(data=image_data)
  if(methods::is(object,"iotarelr_iota2")){
    image_plot<-image_plot+
      ggplot2::geom_col(position="stack",
                        ggplot2::aes(y=.data$category,
                                     x=.data$amount,
                                     fill=.data$case))+
      ggplot2::geom_text(ggplot2::aes(x=(.data$position_l+.data$position_r)*0.5,
                                      y=.data$category,
                                      label = round(.data$amount,digits=3)),
                         size = number_size, hjust = 0.5, vjust = 0, check_overlap=TRUE)+
      ggplot2::ylab(ylab)
  } else if(methods::is(object,"iotarelr_iota2_dgf")){
    image_plot<-image_plot+
      ggplot2::geom_col(position="stack",
                        ggplot2::aes(y=.data$group,
                                     x=.data$amount,
                                     fill=.data$case))+
      ggplot2::facet_grid(category ~ .)+
      ggplot2::geom_text(ggplot2::aes(x=(.data$position_l+.data$position_r)*0.5,
                                      y=.data$group,
                                      label = round(.data$amount,digits=3)),
                         size = number_size, hjust = 0.5, vjust = 0, check_overlap=TRUE)+
      ggplot2::ylab(ylab)
  }
  image_plot<-image_plot+
    ggplot2::scale_fill_manual(
      values=c("iota"="darkgreen",
               "case_2"="orange",
               "case_3"="red"),
      labels=c("iota"=liota,
               "case_2"=lcase2,
               "case_3"=lcase3),
      name="")+
    ggplot2::xlab(xlab)+
    ggplot2::theme_classic()+
    ggplot2::theme(legend.position="bottom",
                   legend.justification = "left",
                   legend.key.size = ggplot2::unit(key_size, "cm"),
                   legend.text = ggplot2::element_text(size=text_size),
                   legend.direction="vertical")

  ##-------------------------------Image for the scale level
  cut_off_values<-matrix(c(0.829, 0.961, 0.985, 1,
                           0.686, 0.853, 0.898, 1),
                         byrow=TRUE,
                         nrow = 2)
  rownames(cut_off_values)<-c("dynamic_iota_index",
                              "static_iota_index")

  if(scale!="none"){
    scale_cat_colors<-c("red",
                        "orange",
                        "yellow",
                        "green",
                        "darkgreen")
    names(scale_cat_colors)<-lscale_cat
    image_scale<-ggplot2::ggplot()+
      ggplot2::geom_rect(ggplot2::aes(xmin = 0,
                                      xmax = cut_off_values[scale,1],
                                      ymin = 0,
                                      ymax = 1,
                                      fill=lscale_cat[1]),
                         color="black")+
      ggplot2::geom_rect(ggplot2::aes(xmin = cut_off_values[scale,1],
                                      xmax = cut_off_values[scale,2],
                                      ymin = 0,
                                      ymax = 1,
                                      fill=lscale_cat[2]),
                         color="black")+
      ggplot2::geom_rect(ggplot2::aes(xmin = cut_off_values[scale,2],
                                      xmax = cut_off_values[scale,3],
                                      ymin = 0,
                                      ymax = 1,
                                      fill=lscale_cat[3]),
                         color="black")+
      ggplot2::geom_rect(ggplot2::aes(xmin = cut_off_values[scale,3],
                                      xmax = cut_off_values[scale,4],
                                      ymin = 0,
                                      ymax = 1,
                                      fill=lscale_cat[4]),
                         color="black")+
      ggplot2::geom_rect(ggplot2::aes(xmin = cut_off_values[scale,4],
                                      xmax = 1,
                                      ymin = 0,
                                      ymax = 1,
                                      fill=lscale_cat[5]),
                         color="black")+
      ggplot2::scale_fill_manual(values=scale_cat_colors,
                                 name=lscale_quality)+
      ggplot2::theme_classic()+
      ggplot2::theme(axis.line.y = ggplot2::element_blank(),
                     axis.ticks.y = ggplot2::element_blank(),
                     axis.text.y = ggplot2::element_blank())
    if(methods::is(object,"iotarelr_iota2")){
      if(scale=="dynamic_iota_index"){
        image_scale=image_scale+
          ggplot2::geom_vline(xintercept = object$scale_level$iota_index_dyn2,
                              linewidth=2)+
          ggplot2::labs(x="Dynamic Iota Index",
                        y="")
      } else if(scale=="static_iota_index") {
        image_scale=image_scale+
          ggplot2::geom_vline(xintercept = object$scale_level$iota_index_d4,
                              linewidth=2)+
          ggplot2::labs(x="Static Iota Index",
                        y="")
      }
    } else if(methods::is(object,"iotarelr_iota2_dgf")){
      for(g in 1:n_group){
        if(scale=="dynamic_iota_index"){
          image_scale=image_scale+
            ggplot2::geom_vline(xintercept = object[[g]]$scale_level$iota_index_dyn2,
                                linewidth=1.5)+
            ggplot2::annotate(geom = "text",
                              x = object[[g]]$scale_level$iota_index_dyn2-.02,
                              y = 0.5,
                              label = names(object)[g],
                              angle = 90,
                              check_overlap = TRUE)+
            ggplot2::labs(x="Dynamic Iota Index",
                          y="")
        } else if(scale=="static_iota_index") {
          image_scale=image_scale+
            ggplot2::geom_vline(xintercept = object[[g]]$scale_level$iota_index_d4,
                                linewidth=1.5)+
            ggplot2::annotate(geom = "text",
                              x = object[[g]]$scale_level$iota_index_dyn2-.02,
                              y = 0.5,
                              label = names(object)[g],
                              angle = 90,
                              check_overlap = TRUE)+
            ggplot2::labs(x="Static Iota Index",
                          y="")
        }
      }
    }

  }

  image_plot=image_plot+ggplot2::theme(legend.position = legend_position)

  if(scale=="none"){
    return(image_plot)
  } else {
    image_complete<-gridExtra::arrangeGrob(image_plot,
                                           image_scale,
                                           layout_matrix=matrix(c(1,1,1,2),
                                                                ncol = 1))
    return(image_complete)
  }
}

#'Plot of the Coding Stream
#'
#'Function for creating an alluvial plot that can be plotted via 'ggplot2'.
#'
#'@param object Estimates of Iota 2 created with \code{compute_iota2()},
#'\code{check_new_rater()} or with \code{check_dgf()}. Please note that the object
#'created by \code{check_dgf()} cannot be passed directly. Only the elements of
#'the corresponding list are compatible.
#'@param label_titel \code{Character} containing the title of the plot.
#'@param label_prefix_true \code{Character} representing the prefix for tagging
#'the true categories. Character is applied to every category.
#'@param label_prefix_assigned \code{Character} representing the prefix for tagging
#'the assigned categories. Character is applied to every category.
#'@param label_legend_title \code{Character} containing the title of the legend.
#'@param label_true_category \code{Character} describing the stratum of true
#'categories.
#'@param label_assigned_category \code{Character} describing the stratum of
#'assigned categories.
#'@param label_y_axis \code{Character}. Label of the y-axis.
#'@param label_categories_size \code{double} determining the size of the
#'label for each true and assigned category within the plot.
#'@param key_size \code{double} determining the size of the legend.
#'@param text_size \code{double} determining the size of the text within the legend.
#'@param legend_position \code{string} Position of the legend. Possible values
#'are "bottom","right","left", "top" and "none".
#'@param legend_direction \code{string} Layout of the items in the legend. Possible
#'values are "horizontal" and "vertical".
#'@return Returns an object of class \code{gg} and \code{ggplot} which can be
#'shown with \code{plot()}.
#'@note An example for interpreting the plot can be found in the vignette
#'\href{../doc/iotarelr.html}{Get started} or via
#'\code{vignette("iotarelr", package = "iotarelr")}.
#'@importFrom rlang .data
#'@import ggplot2
#'@import ggalluvial
#'@export
plot_iota2_alluvial<-function(object,
                              label_titel="Coding Stream from True to Assigned Categories",
                              label_prefix_true="true",
                              label_prefix_assigned="labeled as",
                              label_legend_title="True Categories",
                              label_true_category="True Category",
                              label_assigned_category="Assigned Category",
                              label_y_axis="Relative Frequencies",
                              label_categories_size=3,
                              key_size=0.5,
                              text_size=10,
                              legend_position="right",
                              legend_direction="vertical"){

  if(methods::is(object,"iotarelr_iota2")==FALSE){
    stop("Class of object for iota is not supported by this function.
         Object must be of class iotarelr_iota2.")
  }

  if(legend_direction%in%c("horizontal","vertical")==FALSE){
    stop("legend_direction must be 'horizontal' or 'vertical'.")
  }
  if(legend_position%in%c("bottom","top","right","left","none")==FALSE){
    stop("legend_direction must be 'bottom', 'top', 'left', 'right' or 'none'.")
  }

  n_categories=ncol(object$categorical_level$raw_estimates$assignment_error_matrix)
  categorical_levels<-colnames(object$categorical_level$raw_estimates$assignment_error_matrix)
  alluvial_data<-matrix(data=NA,
                         nrow=n_categories*n_categories,
                         ncol=3)
  colnames(alluvial_data)<-c("true_category","assigned_category","freq")
  index=1
  for(i in 1:n_categories){
    for(j in 1:n_categories){
      alluvial_data[index,1]<-paste(label_prefix_true,categorical_levels[i])
      alluvial_data[index,2]<-paste(label_prefix_assigned,categorical_levels[j])
      alluvial_data[index,3]<-object$information$est_true_cat_sizes[i]*
        object$categorical_level$raw_estimates$assignment_error_matrix[i,j]
      index=index+1
    }
  }
  alluvial_data<-as.data.frame(alluvial_data)
  alluvial_data$freq<-as.numeric(alluvial_data$freq)
  gg_alluvival_plot<-ggplot2::ggplot(
    alluvial_data,
    ggplot2::aes(
      axis1 = .data$true_category,
      axis2 = .data$assigned_category,
      y = .data$freq)) +
    ggalluvial::geom_alluvium(ggplot2::aes(fill=.data$true_category),
                              width = 0,
                              knot.pos = 0,
                              curve_type = "sigmoid",
                              reverse = FALSE)+
    ggplot2::scale_fill_discrete(name=label_legend_title)+
    ggalluvial::geom_stratum(width = 1/8,
                             reverse = FALSE)+
    ggplot2::geom_text(stat = ggalluvial::StatStratum,
                       ggplot2::aes(label = ggplot2::after_stat(.data$stratum)),
                       reverse = FALSE,
                       size = label_categories_size) +
    ggplot2::scale_x_continuous(breaks = 1:2,
                                labels = c(label_true_category, label_assigned_category))+
    ggplot2::theme_classic()+
    ggtitle(label_titel)+
    ggplot2::ylab(label_y_axis)+
    ggplot2::theme(legend.position=legend_position,
                   legend.justification = "left",
                   legend.key.size = ggplot2::unit(key_size, "cm"),
                   legend.text = ggplot2::element_text(size=text_size),
                   legend.direction=legend_direction)
  return(gg_alluvival_plot)
}

test_iota1<-compute_iota1(
  data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")])
test_iota2<-compute_iota2(
  data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")],
  random_starts = 2,
  trace = FALSE)
test_iota2_dgf<-check_dgf(
  data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")],
  splitcr = iotarelr_written_exams$Sex,
  random_starts = 1,
  trace = FALSE)
test_iota2_new_rater<-check_new_rater(
  true_values = iotarelr_written_exams$`Coder A`,
  assigned_values = iotarelr_new_rater)

test_that("plots", {
  testthat::expect_s3_class(plot_iota(test_iota2),
                            c("gg","ggplot")
  )
  testthat::expect_s3_class(plot_iota(test_iota2_new_rater),
                            c("gg","ggplot")
  )
  testthat::expect_s3_class(plot_iota(test_iota2_dgf),
                            c("gg","ggplot")
  )

  testthat::expect_s3_class(plot_iota(test_iota2,
                                      scale="dynamic_iota_index"),
                            c("gtable","gTree","grob", "gDesc")
  )
  testthat::expect_s3_class(plot_iota(test_iota2_new_rater,
                                      scale="dynamic_iota_index"),
                            c("gtable","gTree","grob", "gDesc")
  )
  testthat::expect_s3_class(plot_iota(test_iota2_dgf,
                                      scale="dynamic_iota_index"),
                            c("gtable","gTree","grob", "gDesc")
  )
  testthat::expect_s3_class(plot_iota(test_iota2,
                                      scale="static_iota_index"),
                            c("gtable","gTree","grob", "gDesc")
  )
  testthat::expect_s3_class(plot_iota(test_iota2_new_rater,
                                      scale="static_iota_index"),
                            c("gtable","gTree","grob", "gDesc")
  )
  testthat::expect_s3_class(plot_iota(test_iota2_dgf,
                                      scale="static_iota_index"),
                            c("gtable","gTree","grob", "gDesc")
  )
  testthat::expect_error(plot_iota(test_iota1)
  )
  testthat::expect_s3_class(plot_iota2_alluvial(test_iota2_new_rater),
                            c("gg","ggplot")

  )
  testthat::expect_s3_class(plot_iota2_alluvial(test_iota2),
                            c("gg","ggplot")
  )
  testthat::expect_error(plot_iota2_alluvial(test_iota1))
  testthat::expect_error(plot_iota2_alluvial(test_iota2_dgf))
})

# This file is part of the R package "iotarelr".
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as published by
# the Free Software Foundation.
#
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>

.onUnload<-function(libpath){
  library.dynam.unload("iotarelr",libpath)
}

#'Get patterns
#'
#'Auxiliary function written in \code{R} for providing the necessary information
#'about the patterns generated by raters. This function produces the
#'input for the EM-algorithm.
#'
#'@param data \code{Matrix} or \code{data.frame} containing the ratings. The
#'cases are in the rows and the raters are in the columns. Characters in the cells
#'are supported. At least two raters are necessary.
#'@param categorical_levels \code{Vector} containing all possible categories of
#'the content analysis.
#'@return Function returns a \code{list} with the following components:
#'\item{n}{Integer representing the number of different patterns in the data.}
#'\item{shape}{\code{Matrix} containing all unique patterns in in the data.
#'Cells of the matrix are characters.}
#'\item{frq}{\code{Vector} containing the frequencies of the patterns.}
#'\item{count}{\code{Matrix} containing the relative frequencies of the
#'categories within each pattern. The number of rows equals the number of
#'patterns. The number of columns equals the number of categories.}
#'@export
get_patterns<-function(data,categorical_levels){

  n_rater<-ncol(data)
  pattern_shape<-unique(data)
  pattern_shape<-matrix(sapply(pattern_shape,as.character),ncol=n_rater)
  pattern_id<-vector(length = nrow(data))
  for(i in 1:nrow(data)){
    pattern_id[i]<-paste0(sapply(data[i,],as.character),collapse = "")
  }

  pattern_frq<-vector(length = nrow(pattern_shape))
  for(i in 1:nrow(pattern_shape)){
    tmp<-pattern_id
    tmp<-pattern_id==paste0(pattern_shape[i,],collapse = "")
    pattern_frq[i]<-sum(tmp)
  }

  internal_count_frq<-matrix(data=NA,nrow = nrow(pattern_shape),ncol=length(categorical_levels))
  colnames(internal_count_frq)<-categorical_levels
  for (k in categorical_levels){
    tmp<-pattern_shape
    tmp<-tmp==k
    internal_count_frq[,k]<-rowSums(tmp)
  }
  internal_count_frq<-internal_count_frq/n_rater

  pattern<-NULL
  pattern["n"]<-list(nrow(pattern_shape))
  pattern["shape"]<-list(pattern_shape)
  pattern["frq"]<-list(pattern_frq)
  pattern["count"]<-list(internal_count_frq)
  return(pattern)
}

#' Computes Iota and its elements in version 2
#'
#' Fits a model of Iota2 to the data
#'
#' @param data Data for which the elements should be estimated. Data must be
#' an object of type \code{data.frame} or \code{matrix} with cases in the rows and
#' raters in the columns.
#' @param random_starts An integer for the number of random starts for the
#' EM algorithm.
#' @param max_iterations An integer for the maximum number of iterations within
#' the EM algorithm.
#' @param cr_rel_change Positive numeric value for defining the convergence of the
#' EM algorithm.
#' @param con_step_size \code{Double} for specifying the size for increasing or
#' decreasing the probabilities during the conditioning stage of estimation.
#' This value should not be less than 1e-3.
#' @param con_random_starts \code{Integer} for the number of random starts
#' within the conditioning stage.
#' @param con_max_iterations \code{Integer} for the maximum number of iterations
#' during the conditioning stage.
#' @param con_rel_convergence \code{Double} for determining the convergence
#' criterion during the conditioning stage. The algorithm stops if the relative change
#' is smaller than this criterion.
#' @param fast \code{Bool} If \code{TRUE} a fast estimation is applied during the
#' condition stage. This option ignores all parameters beginning with "con_".
#' If \code{FALSE} the estimation described in Berding and
#' Pargmann (2022) is used. Default is \code{TRUE}.
#'@param trace \code{TRUE} for printing progress information on the console.
#'\code{FALSE} if this information is not to be printed.
#'@param con_trace \code{TRUE} for printing progress information on the console
#'during estimations in the conditioning stage. \code{FALSE} if this information
#'is not to be printed.
#' @param b_min Value ranging between 0 and 1, determining the minimal size of
#' the categories for checking if boundary values occurred. The algorithm tries
#' to select solutions that are not considered to be boundary values.
#' @return Returns a \code{list} with the following three components:
#' The first component \code{estimates_categorical_level} comprises all
#' elements that describe the ratings on a categorical level. The elements are
#' sub-divided into raw estimates and chance-corrected estimates.
#' \describe{
#' \item{\code{raw_estimates}}{
#' \describe{
#' \item{\code{alpha_reliability:}}{A vector containing the Alpha
#' Reliabilities for each category. These values represent probabilities.}
#' \item{\code{beta_reliability:}}{A vector containing the Beta Reliabilities for each
#' category. These values represent probabilities.}
#' \item{\code{assignment_error_matrix:}}{Assignment Error Matrix containing the conditional
#' probabilities for assigning a unit of category i to categories 1 to n.}
#'  \item{\code{iota:}}{A vector containing the Iota values for each category.}
#'  \item{\code{iota_error_1:}}{A vector containing the Iota Error Type I values for each category.}
#'  \item{\code{iota_error_2:}}{A vector containing the Iota Error Type II values for each category.}
#' }}
#'
#' \item{\code{elements_chance_corrected}}{
#' \describe{
#' \item{\code{alpha_reliability:}}{A vector containing the chance-corrected Alpha Reliabilities for each category.}
#' \item{\code{beta_reliability:}}{A vector containing the chance-corrected Beta Reliabilities for each category.}
#' }}
#'
#' The second component \code{estimates_scale_level} contains elements for
#' describing the quality of the ratings on a scale level. It comprises the
#' following elements:
#' \describe{
#' \item{\code{iota_index:}}{The Iota Index, representing the reliability on a scale level.}
#' \item{\code{iota_index_d4:}}{The Static Iota Index, which is a transformation of the original Iota Index,
#' in order to consider the uncertainty of estimation.}
#' \item{\code{iota_index_dyn2:}}{The Dynamic Iota Index, which is a transformation of the original Iota Index,
#' in order to consider the uncertainty of estimation.}
#' }
#'
#' The third component \code{information} contains important information
#' regarding the parameter estimation. It comprises the following elements:
#'\describe{
#' \item{\code{log_likelihood:}}{Log-likelihood of the best solution.}
#' \item{\code{convergence:}}{If estimation converged 0, otherwise 1.}
#' \item{\code{est_true_cat_sizes:}}{Estimated categorical sizes. This is the estimated amount of the categories.}
#' \item{\code{conformity:}}{\code{0} if the solution is in line with assumptions of weak superiority.
#'  A number greater 0 indicates the number of violations of the assumption
#'  of weak superiority.}
#'  \item{\code{random_starts:}}{Numer of random starts for the EM algorithm.}
#' \item{\code{boundaries:}}{\code{False} if the best solution does not contain boundary values.
#' \code{True} if the best solution does contain boundary values}
#' \item{\code{p_boundaries:}}{Percentage of solutions with boundary values during the estimation.}
#' \item{\code{call:}}{Name of the function that created the object.}
#' \item{\code{n_rater:}}{Number of raters.}
#' \item{\code{n_cunits:}}{Number of coding units.}
#' }}
#'@references  Florian Berding and Julia Pargmann (2022).Iota Reliability Concept
#'of the Second Generation. Measures for Content Analysis Done by
#'Humans or Artificial Intelligences. Berlin: Logos.
#'https://doi.org/10.30819/5581
#' @export
compute_iota2<-function(data,
                        random_starts=10,
                        max_iterations=5000,
                        cr_rel_change=1e-12,
                        con_step_size = 1e-4,
                        con_rel_convergence=1e-12,
                        con_max_iterations=5000,
                        con_random_starts=5,
                        b_min=.01,
                        fast=TRUE,
                        trace=TRUE,
                        con_trace=FALSE){

  #---------------------------------------------------------------------------
  #Checking input data
  if((is.data.frame(data)==TRUE |
     is.matrix(data==TRUE))==FALSE){
    stop("data must be a matrix or data.frame")
  }

  #Converting input data
  data<-as.data.frame(data)
  categorical_levels<-names(table(data[1]))
  for(i in 2:ncol(data)){
    tmp<-names(table(data[i]))
    for(j in tmp)
      if(j %in% categorical_levels==FALSE){
        categorical_levels<-c(categorical_levels,j)
      }
  }
  categorical_levels<-sort(categorical_levels,decreasing = FALSE)

  for (i in 1:ncol(data)){
    data[[i]]<-factor(x=lapply(data[[i]],as.character),levels=categorical_levels)
  }

  n_rater=ncol(data)
  n_categories<-length(categorical_levels)
  N=nrow(data)

  #-------------------------------------------------------------------------
  #Identifying patterns
  patterns<-get_patterns(data,categorical_levels)

  #-------------------------------------------------------------------------
  #Performing parameter estimations
  cons_matrix<-matrix(data=FALSE,nrow=n_categories,ncol=n_categories)
  diag(cons_matrix)<-TRUE
  estimates_list<-EM_algo_c(obs_pattern_shape=patterns$shape,
                            obs_pattern_frq=patterns$frq,
                            obs_internal_count=patterns$count,
                            categorical_levels=categorical_levels,
                            random_starts=random_starts,
                            max_iterations=max_iterations,
                            rel_convergence=cr_rel_change,
                            con_step_size = con_step_size,
                            con_random_starts=con_random_starts,
                            con_max_iterations = con_max_iterations,
                            con_rel_convergence = con_rel_convergence,
                            fast=fast,
                            trace=trace,
                            con_trace = con_trace)
  #-------------------------------------------------------------------------
  #Summarizing estimates
  summary<-matrix(data=NA,nrow=random_starts,ncol=6)
  for(i in 1:random_starts){
    summary[i,1]<-estimates_list[[i]]$log_likelihood
    summary[i,2]<-estimates_list[[i]]$convergence
    summary[i,3]<-i
  }

  #Check for correct solutions
  estimates_list_tmp<-estimates_list
  compare_sum<-0
  for(i in 1:length(estimates_list_tmp)){
    estimates_list_tmp[[i]]["con_violations"]<-check_conformity_c(estimates_list_tmp[[i]]$aem)
    summary[i,4]<-estimates_list_tmp[[i]]$con_violations
    summary[i,5]<-max(estimates_list_tmp[[i]]$categorial_sizes)
    summary[i,6]<-min(estimates_list_tmp[[i]]$categorial_sizes)
  }

  #Selection of the best estimates
  #Reducing the estimates to parameters that are as compliant as possible
  summary_selected_I<-subset(summary,summary[,6]>b_min)
  if(nrow(summary_selected_I)!=0){
  index_min<-summary_selected_I[match(x=min(summary_selected_I[,1],na.rm = TRUE),
                                      table=summary_selected_I[,1]),3]
  boundaries=FALSE
  } else {
    index_min<-summary[match(x=min(summary[,1],na.rm = TRUE),
                                        table=summary[,1]),3]
  boundaries=TRUE
  }
  estimates<-estimates_list_tmp[[index_min]]
  p_boundaries<-(length(estimates_list_tmp)-nrow(summary_selected_I))/length(estimates_list_tmp)

  #---------------------------------------------------------------------------
  #Assigning the best solution

  #Assignment-Error-Matrix
  aem=estimates$aem
  colnames(aem)=categorical_levels
  rownames(aem)=categorical_levels

  #Class Sizes
  #p_classes<-estimates$categorial_sizes
  #names(p_classes)<-categorical_levels

  #-------------------------------------------------------------------------
  #Summary
  #Request Reliability Estimates
  results<-NULL
  results<-get_iota2_measures(aem = aem,
                              categorical_sizes = estimates$categorial_sizes,
                              categorical_levels = categorical_levels
  )
  #Summarizing central information
  Esimtates_Information<-NULL
  Esimtates_Information["log_likelihood"]<-list(estimates$log_likelihood)
  Esimtates_Information["random_starts"]<-list(random_starts)
  Esimtates_Information["iteration"]<-list(estimates$iteration)
  Esimtates_Information["convergence"]<-list(estimates$convergence)
  Esimtates_Information["est_true_cat_sizes"]<-list(estimates$categorial_sizes)
  Esimtates_Information["conformity"]<-list(estimates$con_violations)
  Esimtates_Information["boundaries"]<-list(boundaries)
  Esimtates_Information["p_boundaries"]<-list(p_boundaries)
  Esimtates_Information["call"]<-list("compute_iota2")
  Esimtates_Information["n_rater"]<-list(ncol(data))
  Esimtates_Information["n_cunits"]<-list(N)
  Esimtates_Information["estimates_list"]<-list(estimates_list_tmp)
  results["information"]<-list(Esimtates_Information)

  #Assigning class
  class(results)<-"iotarelr_iota2"
  return(results)
}

#' Get Iota2 Measures
#'
#' Function for calculating the elements of the Iota Concept 2
#'
#' @param aem Assignment Error Matrix.
#' @param categorical_sizes Probabilities for the different categories to occur.
#' @param categorical_levels \code{Vector} containing all possible categories of
#' the content analysis.
#' @return Returns a \code{list} of all measures belonging to the Iota Concept
#' of the second generation.
#' The first component \code{estimates_categorical_level} comprises all
#' elements that describe the ratings on a categorical level. The elements are
#' sub-divided into raw estimates and chance-corrected estimates.
#' \describe{
#' \item{\code{raw_estimates}}{
#' \describe{
#' \item{\code{iota:}}{A vector containing the Iota values for each category.}
#'  \item{\code{iota_error_1:}}{A vector containing the Iota Error Type I values for each category.}
#'  \item{\code{iota_error_2:}}{A vector containing the Iota Error Type II values for each category.}
#' \item{\code{alpha_reliability:}}{A vector containing the Alpha
#' Reliabilities for each category. These values represent probabilities.}
#' \item{\code{beta_reliability:}}{A vector containing the Beta Reliabilities for each
#' category. These values represent probabilities.}
#' \item{\code{assignment_error_matrix:}}{Assignment Error Matrix containing the conditional
#' probabilities for assigning a unit of category i to categories 1 to n.}
#' }}
#' \item{\code{elements_chance_corrected}}{
#' \describe{
#' \item{\code{alpha_reliability:}}{A vector containing the chance-corrected Alpha Reliabilities for each category.}
#' \item{\code{beta_reliability:}}{A vector containing the chance-corrected Beta Reliabilities for each category.}
#' }}
#' }
#' The second component \code{estimates_scale_level} contains elements for
#' describing the quality of the ratings on a scale level. It comprises the
#' following elements:
#' \describe{
#' \item{\code{iota_index:}}{The Iota Index, representing the reliability on a scale level.}
#' \item{\code{iota_index_d4:}}{The Static Iota Index, which is a transformation of the original Iota Index,
#' in order to consider the uncertainty of estimation.}
#' \item{\code{iota_index_dyn2:}}{The Dynamic Iota Index, which is a transformation of the original Iota Index,
#' in order to consider the uncertainty of estimation.}
#' }
 #'@references  Florian Berding and Julia Pargmann (2022).Iota Reliability Concept
#'of the Second Generation. Measures for Content Analysis Done by
#'Humans or Artificial Intelligences. Berlin: Logos.
#'https://doi.org/10.30819/5581
#' @export
get_iota2_measures<-function(aem,
                             categorical_sizes,
                             categorical_levels){
  n_categories=length(categorical_levels)
  categorical_levels<-as.character(categorical_levels)

  #alpha-reliability
  p_alpha_reliability<-as.vector(t(as.matrix(diag(aem))))
  names(p_alpha_reliability)<-categorical_levels
  p_alpha_error<-1-p_alpha_reliability

  #Estimation of beta-error
  p_beta_error<-vector(length = n_categories)
  p_beta_error[]<-1
  names(p_beta_error)<-categorical_levels

  for (k in categorical_levels){
    other_k<-subset(categorical_levels,categorical_levels!=k)
    p_beta_error_condition<-0
    p_beta_error_target<-0
    for(k2 in other_k){
      p_beta_error_condition<-p_beta_error_condition+categorical_sizes[k2]*(1-aem[k2,k2])
      p_beta_error_target<-p_beta_error_target+categorical_sizes[k2]*aem[k2,k]
    }
    if(p_beta_error_condition>0){
      p_beta_error[k]<-p_beta_error_target/p_beta_error_condition
    }
    if(p_beta_error_condition<=0){
      p_beta_error[k]<-0
    }
  }
  p_beta_reliability<-1-p_beta_error

  #Estimating normalized and chance-corrected alpha values for Iota
  alpha<-(p_alpha_reliability-1/n_categories)/(1-1/n_categories)

  #Estimating normalized and chance-corrected beta values for Iota
  beta<-vector(length = n_categories)
  names(beta)<-categorical_levels
  for (k in categorical_levels){
    other_k<-subset(categorical_levels,categorical_levels!=k)
    p_beta_error_condition_Chance<-0
    p_beta_error_target_chance<-0
    for(k2 in other_k){
      p_beta_error_condition_Chance<-p_beta_error_condition_Chance+categorical_sizes[k2]*(1-1/n_categories)
      p_beta_error_target_chance<-p_beta_error_target_chance+categorical_sizes[k2]*1/n_categories
    }
    tmp<-1-p_beta_error_target_chance/p_beta_error_condition_Chance
    beta[k]<-(p_beta_reliability[k]-tmp)/(1-tmp)
    #beta[k]<-abs((p_beta_reliability[k]-tmp)/(1-tmp))
  }

  #Estimating Iota
  beta_error_sum<-vector(length = n_categories)
  for(i in 1:n_categories){
    beta_error_sum[i]<-sum(categorical_sizes*p_alpha_error)-
      categorical_sizes[i]*p_alpha_error[i]
  }
  iota_numerator<-categorical_sizes*p_alpha_reliability
  iota_denominator<-categorical_sizes*p_alpha_reliability+
    categorical_sizes*p_alpha_error+
    p_beta_error*beta_error_sum

  iota<-iota_numerator/iota_denominator
  names(iota)<-categorical_levels

  iota_e1_numerator<-categorical_sizes*(1-p_alpha_reliability)
  iota_e1<-iota_e1_numerator/iota_denominator
  names(iota_e1)<-categorical_levels

  iota_e2<-1-iota-iota_e1
  names(iota_e2)<-categorical_levels

  #Estimation of Iota Index
  iota_index<-categorical_sizes%*%rowSums(abs(
    aem-matrix(data=1/n_categories,
               nrow = n_categories,
               ncol=n_categories)))/(2*(n_categories-1)/n_categories)

  #Estimation of Iota Index dyn=2
  iota_index_dyn2<-iota_index^(1+iota_index^2)

  #Estimation of Iota Index d=4
  d=4
  distances<-matrix(ncol=n_categories,
                    nrow=n_categories,
                    data=NA)
  for(i in 1:n_categories){
    for(j in 1:n_categories){
      distances[i,j]<-abs(aem[i,j]-1/n_categories)^d
    }
  }
  iota_index_d4<-categorical_sizes%*%rowSums(distances)
  max_distance<-1/((1-1/n_categories)^d+(n_categories-1)*(abs(1/n_categories)^d))
  iota_index_d4<-max_distance*iota_index_d4

  #Summary of all Estimates
  Elements_Chance_Corrected<-NULL
  Elements_Chance_Corrected["alpha_reliability"]<-list(alpha)
  Elements_Chance_Corrected["beta_reliability"]<-list(beta)

  Elements_Raw_Estimates<-NULL
  Elements_Raw_Estimates["iota"]<-list(iota)
  Elements_Raw_Estimates["iota_error_1"]<-list(iota_e1)
  Elements_Raw_Estimates["iota_error_2"]<-list(iota_e2)
  Elements_Raw_Estimates["alpha_reliability"]<-list(p_alpha_reliability)
  Elements_Raw_Estimates["beta_reliability"]<-list(p_beta_reliability)
  Elements_Raw_Estimates["assignment_error_matrix"]<-list(aem)

  Estimates_Categorical_Level<-NULL
  Estimates_Categorical_Level["raw_estimates"]<-list(Elements_Raw_Estimates)
  Estimates_Categorical_Level["chance_corrected"]<-list(Elements_Chance_Corrected)

  Estimates_Scale_Level<-NULL
  Estimates_Scale_Level["iota_index"]<-list(iota_index)
  Estimates_Scale_Level["iota_index_d4"]<-list(iota_index_d4)
  Estimates_Scale_Level["iota_index_dyn2"]<-list(iota_index_dyn2)

  results<-NULL
  results["categorical_level"]<-list(Estimates_Categorical_Level)
  results["scale_level"]<-list(Estimates_Scale_Level)

  return(results)
}

skip("Comparisaton of fast and slow estimation")
test_that("Comparisaton of fast and slow estimation", {

  expect_equal(
    check_new_rater(
      true_values = iotarelr_written_exams$`Coder A`,
      assigned_values = iotarelr_new_rater,
      con_random_starts = 200,
      fast = FALSE)$categorical_level$raw_estimates$assignment_error_matrix,
    check_new_rater(
      true_values = iotarelr_written_exams$`Coder A`,
      assigned_values = as.factor(iotarelr_new_rater),
      con_random_starts = 20,
      fast = TRUE)$categorical_level$raw_estimates$assignment_error_matrix,
    tolerance=1e-1
  )

  expect_equal(
    check_new_rater(
      true_values = iotarelr_written_exams$`Coder A`,
      assigned_values = iotarelr_new_rater,
      con_random_starts = 200,
      fast = FALSE)$information$est_true_cat_sizes,
    check_new_rater(
      true_values = iotarelr_written_exams$`Coder A`,
      assigned_values = as.factor(iotarelr_new_rater),
      con_random_starts = 20,
      fast = TRUE)$information$est_true_cat_sizes,
    tolerance=1e-1
  )

  expect_equal(
    compute_iota2(
      data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")],
      random_starts = 200,
      trace = FALSE,
      fast = FALSE)$categorical_level$raw_estimates$assignment_error_matrix,
    compute_iota2(
      data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")],
      random_starts = 20,
      trace = FALSE,
      fast=TRUE)$categorical_level$raw_estimates$assignment_error_matrix,
    tolerance=1e-1
  )

  expect_equal(
    compute_iota2(
      data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")],
      random_starts = 200,
      trace = FALSE,
      fast = FALSE)$information$est_true_cat_sizes,
    compute_iota2(
      data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")],
      random_starts = 20,
      trace = FALSE,
      fast=TRUE)$information$est_true_cat_sizes,
    tolerance=1e-1
  )

})

# This file is part of the R package "iotarelr".
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as published by
# the Free Software Foundation.
#
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>


#'Check new rater
#'
#'Function for estimating the reliability of codings for a new rater based on
#'Iota 2
#'
#'@param true_values \code{Vector} containing the true categories of the coding
#'units. Vector must have the same length as \code{assigned_values}.
#'@param assigned_values \code{Vector} containing the assigned
#'categories of the coding units. Missing values are currently not supported and
#'have to be omitted from the vector. Vector must have the same length as
#'\code{true_values}.
#' @param con_step_size \code{Double} for specifying the size for increasing or
#' decreasing the probabilities during the conditioning stage of estimation.
#' This value should not be less than 1e-3.
#' @param con_random_starts \code{Integer} for the number of random starts
#' within the condition stage.
#' @param con_max_iterations \code{Integer} for the maximum number of iterations
#' during the conditioning stage.
#' @param con_rel_convergence \code{Double} for determining the convergence
#' criterion during the conditioning stage. The algorithm stops if the relative change
#' is smaller than this criterion.
#'@param con_trace \code{TRUE} for printing progress information on the console
#'during estimations in the conditioning stage. \code{FALSE} if you do not want to have
#'this information printed.
#' @param fast \code{Bool} If \code{TRUE} a fast estimation is applied during the
#' condition stage. This option ignores all parameters beginning with "con_".
#' If \code{FALSE} the estimation described in Berding and
#' Pargmann (2022) is used. Default is \code{TRUE}.
#' @param free_aem \code{Bool} If \code{TRUE} the Assignment Error Matrix is
#' estimated in a way ensuring conformity with the assumption of weak superiority.
#' if \code{FALSE} the Assignment Error Matrix is freely estimated. \code{TRUE}
#' is default.
#' @return Returns a \code{list} with the following three components:
#' The first component \code{estimates_categorical_level} comprises all
#' elements that describe the ratings on a categorical level. The elements are
#' sub-divided into raw estimates and chance-corrected estimates.
#'
#' \code{raw_estimates}
#' \describe{
#' \item{\code{alpha_reliability:}}{A vector containing the Alpha
#' Reliabilities for each category. These values represent probabilities.}
#' \item{\code{beta_reliability:}}{A vector containing the Beta Reliabilities for each
#' category. These values represent probabilities.}
#' \item{\code{assignment_error_matrix:}}{An Assignment Error Matrix containing the conditional
#' probabilities for assigning a unit of category i to categories 1 to n.}
#' \item{\code{iota:}}{A vector containing the Iota values for each category.}
#' }
#' \code{elements_chance_corrected}
#' \describe{
#' \item{\code{alpha_reliability:}}{A vector containing the chance-corrected Alpha Reliabilities for each category.}
#' \item{\code{beta_reliability:}}{A vector containing the chance-corrected Beta Reliabilities for each category.}
#' }
#'
#' The second component \code{estimates_scale_level} contains elements to
#' describe the quality of the ratings on a scale level. It contains the
#' following elements:
#' \describe{
#' \item{\code{iota_index:}}{The Iota Index representing the reliability on a scale level.}
#' \item{\code{iota_index_d4:}}{The Static Iota Index, which is a transformation of the original Iota Index,
#' in order to consider the uncertainty of estimation.}
#' \item{\code{iota_index_dyn2:}}{The Dynamic Iota Index, which is a transformation of the original Iota Index,
#' in order to consider the uncertainty of estimation.}
#' }
#' The third component \code{information} contains important information
#' regarding the parameter estimation. It comprises the following elements:
#'\describe{
#' \item{\code{log_likelihood:}}{Log-likelihood of the best solution.}
#' \item{\code{convergence:}}{If estimation converged 0, otherwise 1.}
#' \item{\code{est_true_cat_sizes:}}{Estimated categorical sizes. This is the estimated amount of the categories.}
#' \item{\code{conformity:}}{\code{0} if the solution is in line with assumptions of weak superiority.
#'  A number greater 0 indicates the number of violations of the assumption
#'  of weak superiority.}
#'  \item{\code{random_starts:}}{Numer of random starts for the EM algorithm.}
#' \item{\code{boundaries:}}{\code{False} if the best solution does not contain boundary values.
#' \code{True} if the best solution does contain boundary values}
#' \item{\code{p_boundaries:}}{Percentage of solutions with boundary values during estimation.}
#' \item{\code{call:}}{Name of the function that created the object.}
#' \item{\code{n_rater:}}{Number of raters.}
#' \item{\code{n_cunits:}}{Number of coding units.}
#' }
#'@note The returned object contains further slots since the returned object is
#'of class \code{iotarelr_iota2}. These slots are empty because they are not part of the
#'estimation within this function.
#'@note Please do not use the measures on the scale level if the Assignment Error Matrix
#'was freely estimated since this kind of matrix is not conceptualized for comparing
#'the coding process with random guessing.
#'@references  Florian Berding and Julia Pargmann (2022). Iota Reliability Concept
#'of the Second Generation. Measures for Content Analysis Done by
#'Humans or Artificial Intelligences. Berlin:Logos.
#'https://doi.org/10.30819/5581
#'@export
check_new_rater<-function(true_values,
                          assigned_values,
                          con_step_size=1e-4,
                          con_random_starts=5,
                          con_max_iterations=5000,
                          con_rel_convergence=1e-12,
                          con_trace=FALSE,
                          fast=TRUE,
                          free_aem=FALSE){

  #Checking format of true values and assigned values
  if((is.data.frame(true_values)==TRUE |
     is.factor(true_values)==TRUE |
     is.vector(true_values)==TRUE)==FALSE){
    stop("true_values must be a vector, factor or data.frame")
  }
  if((is.data.frame(assigned_values)==TRUE |
      is.factor(assigned_values)==TRUE |
      is.vector(assigned_values)==TRUE)==FALSE){
    stop("assigned_values must be a vector, factor or data.frame")
  }

  #Standardizing input data of the true values
  true_values<-unname(true_values)
  if(is.data.frame(true_values)==TRUE){
    true_values<-as.matrix(true_values)
  }
  true_values<-as.character(true_values)

  #Standardizing input data of the assigned values
  if(is.data.frame(assigned_values)==TRUE){
    assigned_values<-as.matrix(assigned_values)
  }
  assigned_values<-as.character(assigned_values)
  assigned_values<-unname(assigned_values)

  #Gathering information on the categorical levels
  categorical_levels<-names(table(true_values))
  categorical_levels<-sort(categorical_levels,decreasing = FALSE)

  categorical_levels_assigned_data<-names(table(assigned_values))
  categorical_levels_assigned_data<-sort(categorical_levels_assigned_data,decreasing = FALSE)

  #Checking if the categorical levels are compatible
  if(sum(categorical_levels_assigned_data%in%categorical_levels)!=length(categorical_levels_assigned_data)){
    stop("The categorical levels of the assigned values exceed the range of
         possible categorical levels of the true values. Please check your
         assigned data for levels that do not occur in the true data.")
  }

  #check if the range of true values is without gaps
  min_freq=(table(true_values)<=0)
  if(sum(min_freq)>=1){
    cat_without_codings=categorical_levels[which(min_freq)]
    stop("There are no codings for some true categories. These are",
         paste(cat_without_codings,collapse = " ")
    )
  }

  #Start of the calculations
  obs<-cbind(true_values,assigned_values)
  colnames(obs)<-c("true_values","assigned_values")
  obs<-as.data.frame(obs)

  n_categories=length(categorical_levels)
  freq_matrix<-matrix(data=NA,
                      nrow = n_categories,
                      ncol = n_categories)

  colnames(freq_matrix)<-categorical_levels
  rownames(freq_matrix)<-categorical_levels
  for(i in categorical_levels){
    for(j in categorical_levels){
      tmp_subset<-subset(obs,obs$true==i)
      freq_matrix[i,j]<-sum(tmp_subset$assigned_values==j)
    }
  }

  categorical_sizes=rowSums(freq_matrix)/sum(freq_matrix)
  aem<-NULL
  if(free_aem==FALSE){
    for(i in 1:n_categories){
      tmp<-est_con_multinominal_c(observations=freq_matrix[i,],
                             anchor = i,
                             max_iter = con_max_iterations,
                             step_size = con_step_size,
                             cr_rel_change = con_rel_convergence,
                             n_random_starts = con_random_starts,
                             fast=fast,
                             trace = con_trace)
      aem<-rbind(aem,tmp)
    }
  } else {
      aem<-freq_matrix/rowSums(freq_matrix)
  }
  colnames(aem)<-categorical_levels
  rownames(aem)<-categorical_levels

  measures<-get_iota2_measures(aem=aem,
                               categorical_sizes = categorical_sizes,
                               categorical_levels= categorical_levels)

  Esimtates_Information<-NULL
  Esimtates_Information["log_likelihood"]<-list(NA)
  Esimtates_Information["iteration"]<-list(NA)
  Esimtates_Information["convergence"]<-list(NA)
  Esimtates_Information["est_true_cat_sizes"]<-list(categorical_sizes)
  Esimtates_Information["conformity"]<-list(check_conformity_c(aem=aem))
  Esimtates_Information["boundaries"]<-list(NA)
  Esimtates_Information["p_boundaries"]<-list(NA)

  Esimtates_Information["n_rater"]<-list(1)
  Esimtates_Information["n_cunits"]<-list(length(true_values))
  Esimtates_Information["call"]<-list("check_new_rater")

  Esimtates_Information["random_starts"]<-list(NA)
  Esimtates_Information["estimates_list"]<-list(NA)

  results<-NULL
  results["categorical_level"]<-list(measures$categorical_level)
  results["scale_level"]<-list(measures$scale_level)
  results["information"]<-list(Esimtates_Information)

  class(results)<-"iotarelr_iota2"

  return(results)
}



#'Check for Different Guidance Functioning (DGF)
#'
#'Function for checking if the coding scheme is the same for different sub-groups.
#'
#' @param data Data for which the elements should be estimated. Data must be
#' an object of type \code{data.frame} or \code{matrix} with cases in the rows and
#' raters in the columns. Please note that no additional variables are allowed
#' in this object.
#' @param splitcr \code{Vector} containing the assignments of coding units to
#' groups. The vector must have the same length as the number of rows of object
#' \code{data}.
#' @param random_starts An integer for the number of random starts for the
#' EM algorithm.
#' @param max_iterations An integer for the maximum number of iterations within
#' the EM algorithm.
#' @param cr_rel_change Positive numeric value for defining the convergence of the
#' EM algorithm.
#' @param con_step_size \code{Double} for specifying the size for increasing or
#' decreasing the probabilities during the conditioning stage of estimation.
#' This value should not be less than 1e-3.
#' @param con_random_starts \code{Integer} for the number of random starts
#' within the condition stage.
#' @param con_max_iterations \code{Integer} for the maximum number of iterations
#' during the condition stage.
#' @param con_rel_convergence \code{Double} for determining the convergence
#' criterion during condition stage. The algorithm stops if the relative change
#' is smaller than this criterion.
#'@param trace \code{TRUE} for printing progress information on the console.
#'\code{FALSE} if this information is not to be printed.
#'@param con_trace \code{TRUE} for printing progress information on the console
#'during estimations in the condition stage. \code{FALSE} if this information
#'is not to be printed.
#' @param fast \code{Bool} If \code{TRUE} a fast estimation is applied during the
#' condition stage. This option ignores all parameters beginning with "con_".
#' If \code{FALSE} the estimation described in Berding and
#' Pargmann (2022) is used. Default is \code{TRUE}.
#'@param b_min Value ranging between 0 and 1 determining the minimal size of
#'the categories for checking if boundary values occurred. The algorithm tries
#'to select solutions that are not considered to be boundary values.
#'@return Returns an object of class \code{iotarelr_iota2_dif}. For each group,
#'the results of the estimation are saved separately. The structure within each
#'group is similar to the results from \code{compute_iota2()}. Please check
#'that documentation.
#'@references  Florian Berding and Julia Pargmann (2022).Iota Reliability Concept
#'of the Second Generation. Measures for Content Analysis Done by
#'Humans or Artificial Intelligences. Berlin:Logos.
#'https://doi.org/10.30819/5581
#'@export

check_dgf<-function(data,
                    splitcr,
                    random_starts = 300,
                    max_iterations = 5000,
                    cr_rel_change = 1e-12,
                    con_step_size = 1e-4,
                    con_random_starts = 10,
                    con_max_iterations = 5000,
                    con_rel_convergence = 1e-12,
                    b_min = 0.01,
                    trace = FALSE,
                    con_trace = FALSE,
                    fast=TRUE){

  tmp<-table(splitcr)
  n_group=length(tmp)
  group_names<-names(tmp)

  splitcr<-factor(x=lapply(splitcr,FUN=as.character),levels = group_names)

  sub_group_data<-NULL
  results<-NULL
  for(i in 1:n_group){
    sub_group_data<-subset(data,splitcr==group_names[i])
    if (trace==TRUE){
      print(paste(date(),"Estimating Parameters for subgroup",group_names[i]))
    }
    results[paste0("group_",group_names[i])]<-list(
      compute_iota2(data=sub_group_data,
                    random_starts = random_starts,
                    max_iterations = max_iterations,
                    cr_rel_change = cr_rel_change,
                    con_step_size = con_step_size,
                    con_rel_convergence = con_rel_convergence,
                    con_max_iterations = con_max_iterations,
                    con_random_starts = con_random_starts,
                    b_min = b_min,
                    trace = trace,
                    con_trace = con_trace,
                    fast=fast))
      }
  class(results)<-"iotarelr_iota2_dgf"
  return(results)
}

#'Estimate Expected Categories
#'
#'Function for estimating the expected category of coding units.
#'
#' @param data \code{Matrix} which contains the codings for every coding unit. The
#' coding units must be in the rows and the raters must be in the columns. At
#' least two raters are necessary.
#' @param aem Assignment Error Matrix based on the second generation of the Iota Concept (Iota2).
#' @return Returns a \code{matrix} with the original data, the conditioned
#' probability of each true category, and the expected category for every coding unit.
#'@references  Florian Berding and Julia Pargmann (2022).Iota Reliability Concept
#'of the Second Generation. Measures for Content Analysis Done by
#'Humans or Artificial Intelligences. Berlin:Logos.
#'https://doi.org/10.30819/5581
#' @export

est_expected_categories<-function(data,
                                  aem){
  if(ncol(data)<2){
    stop("Data table must have at least two columns.")
  }

  prob_matrix<-matrix(data=NA,
                      nrow=nrow(data),
                      ncol=ncol(aem))
  colnames(prob_matrix)<-colnames(aem)

  exp_categories<-vector(length = nrow(data))

  for(i in 1:nrow(data)){
    for(j in 1:nrow(aem)){
      tmp_p<-1
      for(r in 1:ncol(data)){
        assigned_category<-data[i,r]
        index_category<-match(x=assigned_category,
                              table=colnames(aem))
        tmp_p=tmp_p*aem[j,index_category]
      }
      prob_matrix[i,j]<-tmp_p
    }
    prob_matrix[i,]<-prob_matrix[i,]/sum(prob_matrix[i,])
    index_max<-match(x=max(prob_matrix[i,]),
                     table=prob_matrix[i,])
    exp_categories[i]=colnames(aem)[index_max]
  }
  results<-cbind(data,prob_matrix,exp_categories)
  colnames(results)<-c(
    colnames(data),
    paste0("prob_",colnames(prob_matrix)),
    "expected_category")
  return(results)
}

#'Get Summary
#'
#'Function for creating a short summary of the estimated Iota components.
#'
#' @param object An object of class \code{iotarelr_iota2} created by
#' \code{\link{compute_iota2}}, \code{\link{check_new_rater}}, or
#' \code{\link{check_dgf}}.
#' @return Prints central statistics of the estimated model.
#' @importFrom methods is
#' @export
get_summary<-function(object){
  if(methods::is(object,"iotarelr_iota2")==FALSE){
    stop("Class of object is not supported by this function.")
  }

  call=object$information$call

  n_starts<-object$information$random_starts
  n_categories<-length(object$information$est_true_cat_sizes)
  n_cunits<-object$information$n_cunits
  category_levels<-names(object$information$est_true_cat_sizes)

  log_likelihood<-object$information$log_likelihood
  convergence<-object$information$convergence
  n_rater<-object$information$n_rater

  if(call!="check_new_rater"){
    est_log_likelihood<-vector(length = n_starts)
    for(i in 1:n_starts){
      est_log_likelihood[i]<-object$information$estimates_list[[i]]$log_likelihood
    }
    replications<-sum(round(est_log_likelihood,digits = 4)==round(log_likelihood,digits = 4))
    if(replications>1){
      string_rep="The best log-likelihood has been replicated."
    } else {
      string_rep="The best log-likelihood has not been replicated. Increase the
      number of random stars and/or inspect the Assignment Error Matrices and
      categorical sizes."
    }
  } else {
    string_rep=""
  }

  dimensions<-c("Alpha",
                "Beta",
                "Iota",
                "Iota Error Type I",
                "Iota Error Type II")
  summary_table<-NULL
  summary_table<-rbind(summary_table,object$categorical_level$raw_estimates$alpha_reliability)
  summary_table<-rbind(summary_table,object$categorical_level$raw_estimates$beta_reliability)
  summary_table<-rbind(summary_table,object$categorical_level$raw_estimates$iota)
  summary_table<-rbind(summary_table,object$categorical_level$raw_estimates$iota_error_1)
  summary_table<-rbind(summary_table,object$categorical_level$raw_estimates$iota_error_2)
  summary_table<-round(summary_table,digits=4)
  summary_table<-cbind(dimensions,summary_table)
  summary_table<-as.data.frame(summary_table)

  iota_index<-round(object$scale_level$iota_index,digits = 3)
  iota_index_d4<-round(object$scale_level$iota_index_d4,digits = 3)
  iota_index_dyn2<-round(object$scale_level$iota_index_dyn2, digits = 3)

  cat(sep="\n",
      "Summary\n",
      paste("Call:",call),
      "",
      paste("Number of Raters:",n_rater),
      paste("Number of Categories:",n_categories),
      paste("Categories:",paste(category_levels,collapse = ",")),
      paste("Number of Coding Units:",n_cunits),
      "",
      paste("Random Start:",n_starts),
      paste("Log-Likelihood:",log_likelihood),
      string_rep,
      "",
      "Primary Parameters",
      "Assignment Error Matrix")
  print(format(as.data.frame(round(object$categorical_level$raw_estimates$assignment_error_matrix,
                             digits=3)),
               scientific=FALSE))
  cat(sep="\n",
      "",
      "Categorical Sizes")
  print(round(object$information$est_true_cat_sizes,digits=3))
  cat(sep="\n",
      "",
      "Categorical Level")
  print(format(summary_table,digits=3))
  cat(sep="\n",
      "",
      "Scale Level",
      paste("Iota Index:",iota_index),
      paste("Static Iota Index:",iota_index_d4),
      paste("Dynamic Iota Index:",iota_index_dyn2)
      )
}
summary.iotarelr_iota2<-get_summary

#'Get Consequences
#'
#'Function estimating the consequences of reliability for subsequent analysis.
#'
#' @param measure_typ Type of measure used for estimation. Set "iota_index" for
#' the original Iota Index, "static_iota_index" for the static transformation
#' of the Iota Index with d=4 or "dynamic_iota_index" for the dynamic transformation
#' of the Iota Index with d=2.
#' @param measure_1_val Reliability value for the independent variable.
#' @param measure_2_val Reliability value for the dependent variable. If not
#' set, the function uses the same value as for the independent variable.
#' @param level Level of certainty for calculating the prediction intervals.
#' @param strength True strength of the relationship between the independent and
#' dependent variable. Possible values are "no", "weak", "medium" and "strong". If
#' no value is supplied, a strong relationship is assumed for deviation and a weak
#' relationship for all others. They represent the most demanding situations for the
#' reliability.
#' @param data_type Type of data. Possible values are "nominal" or "ordinal".
#' @param sample_size Size of the sample in the study.
#' @return Returns a \code{data.frame} which contains the prediction intervals
#' for the deviation between  true and estimated sample association/correlation,
#' risk of Type I errors and chance to correctly classify the effect size.
#' Additionally, the probability is estimated so that the statistics of the sample
#' deviate from an error free sample with no or only a weak effect .
#' @note The classification of effect sizes uses the work of Cohen (1988),
#' who differentiates effect sizes by their relevance for practice.
#'
#' For nominal data, all statistics refer to Cramer's V. For ordinal data, all
#' statistics refer to Kendall's Tau.
#'
#' The models for calculating the consequences are taken from Berding and
#' Pargmann (2022).
#' @references Cohen, J. (1988). Statistical Power Analysis for the Behavioral
#' Sciences (2nd Ed.). Taylor & Francis.
#' @references  Berding, Florian, and Pargmann, Julia (2022).Iota Reliability Concept
#'of the Second Generation.Measures for Content Analysis Done by
#'Humans or Artificial Intelligences. Berlin:Logos.
#'https://doi.org/10.30819/5581
#' @export
get_consequences<-function(measure_typ="dynamic_iota_index",
                           measure_1_val,
                           measure_2_val=NULL,
                           level=0.95,
                           strength=NULL,
                           data_type,
                           sample_size){

  if(measure_typ=="iota_index"){
    measure_index="Index"
  } else if (measure_typ=="static_iota_index"){
    measure_index="Iota_Index_400"
  } else if (measure_typ=="dynamic_iota_index"){
    measure_index="Index_dyn_200"
  } else {
    stop("Measure Type not supported.")
  }

  if(is.null(strength)==TRUE){
    strength_deviation="strong"
    model_deviation=pred_models[["deviation"]][[data_type]][[measure_index]][[strength_deviation]]
    strength_type_i_err="weak"
    model_type_i=pred_models[["type_i_errors"]][[data_type]][[measure_index]][[strength_type_i_err]]
    strength_effect_size="weak"
    model_effect_size=pred_models[["effect_sizes"]][[data_type]][[measure_index]][[strength_effect_size]]
  } else {
    strength_deviation=strength
    model_deviation=pred_models[["deviation"]][[data_type]][[measure_index]][[strength_deviation]]
    strength_type_i_err=strength
    model_type_i=pred_models[["type_i_errors"]][[data_type]][[measure_index]][[strength_type_i_err]]
    strength_effect_size=strength
    model_effect_size=pred_models[["effect_sizes"]][[data_type]][[measure_index]][[strength_effect_size]]
  }

  if(is.null(measure_2_val)==TRUE){
    measure_2_val=measure_1_val
  }
  values<-c(1,measure_1_val,measure_2_val,sample_size)
  values_probit<-c(-1,measure_1_val,measure_2_val,sample_size)

  expected_deviation=values%*%model_deviation$parameters
  dev_n=model_deviation$n
  dev_p=model_deviation$p
  dev_se=model_deviation$standard_error
  dev_cov=model_deviation$covmatrix
  dev_inner_sqrt=1+(values%*%dev_cov)%*%((values))
  dev_lwr<-expected_deviation-abs(stats::qt(p=(1-level)/2,df=(dev_n-dev_p)))*dev_se*sqrt(dev_inner_sqrt)
  dev_upr<-expected_deviation+abs(stats::qt(p=(1-level)/2,df=(dev_n-dev_p)))*dev_se*sqrt(dev_inner_sqrt)

  expected_error_rate_raw=values_probit%*%model_type_i$parameters
  i_rate_n=model_type_i$n
  i_rate_p=model_type_i$p
  i_rate_se=model_type_i$standard_error
  i_rate_cov=model_type_i$covmatrix
  i_rate_inner_sqrt=1+(values_probit%*%i_rate_cov)%*%((values_probit))
  i_rate_lwr_raw<-expected_error_rate_raw-abs(stats::qt(p=(1-level)/2,df=(i_rate_n-i_rate_p)))*i_rate_se*sqrt(i_rate_inner_sqrt)
  i_rate_upr_raw<-expected_error_rate_raw+abs(stats::qt(p=(1-level)/2,df=(i_rate_n-i_rate_p)))*i_rate_se*sqrt(i_rate_inner_sqrt)

  expected_error_rate=stats::pnorm(q=expected_error_rate_raw)
  i_rate_lwr=stats::pnorm(q=i_rate_lwr_raw)
  i_rate_upr=stats::pnorm(q=i_rate_upr_raw)

  expected_cassification_rate_raw=values_probit%*%model_effect_size$parameters
  c_rate_n=model_effect_size$n
  c_rate_p=model_effect_size$p
  c_rate_se=model_effect_size$standard_error
  c_rate_cov=model_effect_size$covmatrix
  c_rate_inner_sqrt=1+(values_probit%*%c_rate_cov)%*%((values_probit))
  c_rate_lwr_raw<-expected_cassification_rate_raw-abs(stats::qt(p=(1-level)/2,df=(c_rate_n-c_rate_p)))*c_rate_se*sqrt(c_rate_inner_sqrt)
  c_rate_upr_raw<-expected_cassification_rate_raw+abs(stats::qt(p=(1-level)/2,df=(c_rate_n-c_rate_p)))*c_rate_se*sqrt(c_rate_inner_sqrt)

  expected_cassification_rate=stats::pnorm(q=expected_cassification_rate_raw)
  c_rate_lwr=stats::pnorm(q=c_rate_lwr_raw)
  c_rate_upr=stats::pnorm(q=c_rate_upr_raw)

  #Estimating chances for specific values
  #Deviation
  steps=seq(from=0.5, to=1, by=0.001)
  dev_tmp_results<-matrix(data=NA,
                          nrow=length(steps),
                          ncol = 4)
  for(i in 1:length(steps)){
    tmp_dev_lwr<-expected_deviation-abs(stats::qt(p=(1-steps[i]),df=(dev_n-dev_p)))*dev_se*sqrt(dev_inner_sqrt)
    tmp_dev_upr<-expected_deviation+abs(stats::qt(p=(1-steps[i]),df=(dev_n-dev_p)))*dev_se*sqrt(dev_inner_sqrt)

    target_val_no=0.10
    target_val_weak=0.30

    if(target_val_no>=expected_deviation){
      tmp_val<-tmp_dev_upr
      dev_tmp_results[i,1]<-steps[i]
      dev_tmp_results[i,2]<-abs(target_val_no-tmp_val)
    } else {
      tmp_val<-tmp_dev_lwr
      dev_tmp_results[i,1]<-1-(steps[i])
      dev_tmp_results[i,2]<-abs(target_val_no-tmp_val)
    }

    if(target_val_weak>=expected_deviation){
      tmp_val<-tmp_dev_upr
      dev_tmp_results[i,3]<-steps[i]
      dev_tmp_results[i,4]<-abs(target_val_weak-tmp_val)
    } else {
      tmp_val<-tmp_dev_lwr
      dev_tmp_results[i,3]<-1-(steps[i])
      dev_tmp_results[i,4]<-abs(target_val_weak-tmp_val)
    }
  }
  dev_p_01<-dev_tmp_results[match(x=min(dev_tmp_results[,2]),
                                  table=dev_tmp_results[,2])
                            ,1]
  dev_p_03<-dev_tmp_results[match(x=min(dev_tmp_results[,4]),
                                  table=dev_tmp_results[,4])
                            ,3]

  #Type I error rates
  i_rate_tmp_results<-matrix(data=NA,
                          nrow=length(steps),
                          ncol = 4)
  for(i in 1:length(steps)){
    tmp_i_rate_lwr<-stats::pnorm(q=expected_error_rate_raw-abs(stats::qt(p=(1-steps[i]),df=(i_rate_n-i_rate_p)))*i_rate_se*sqrt(i_rate_inner_sqrt))
    tmp_i_rate_upr<-stats::pnorm(q=expected_error_rate_raw+abs(stats::qt(p=(1-steps[i]),df=(i_rate_n-i_rate_p)))*i_rate_se*sqrt(i_rate_inner_sqrt))

    target_val_no=0.05
    target_val_weak=0.10

    if(target_val_no>=expected_error_rate){
      tmp_val<-tmp_i_rate_upr
      i_rate_tmp_results[i,1]<-steps[i]
      i_rate_tmp_results[i,2]<-abs(target_val_no-tmp_val)
    } else {
      tmp_val<-tmp_i_rate_lwr
      i_rate_tmp_results[i,1]<-1-(steps[i])
      i_rate_tmp_results[i,2]<-abs(target_val_no-tmp_val)
    }
    if(target_val_weak>=expected_error_rate){
      tmp_val<-tmp_i_rate_upr
      i_rate_tmp_results[i,3]<-steps[i]
      i_rate_tmp_results[i,4]<-abs(target_val_weak-tmp_val)
    } else {
      tmp_val<-tmp_i_rate_lwr
      i_rate_tmp_results[i,3]<-1-(steps[i])
      i_rate_tmp_results[i,4]<-abs(target_val_weak-tmp_val)
    }
  }
  i_rate_p_05<-i_rate_tmp_results[match(x=min(i_rate_tmp_results[,2]),
                                  table=i_rate_tmp_results[,2])
                            ,1]
  i_rate_p_10<-i_rate_tmp_results[match(x=min(i_rate_tmp_results[,4]),
                                  table=i_rate_tmp_results[,4])
                            ,3]

  c_rate_tmp_results<-matrix(data=NA,
                             nrow=length(steps),
                             ncol = 4)
  for(i in 1:length(steps)){
    tmp_c_rate_lwr<-stats::pnorm(q=expected_cassification_rate_raw-abs(stats::qt(p=(1-steps[i]),df=(c_rate_n-c_rate_p)))*c_rate_se*sqrt(c_rate_inner_sqrt))
    tmp_c_rate_upr<-stats::pnorm(q=expected_cassification_rate_raw+abs(stats::qt(p=(1-steps[i]),df=(c_rate_n-c_rate_p)))*c_rate_se*sqrt(c_rate_inner_sqrt))

    target_val_no=0.95
    target_val_weak=0.90

    if(target_val_no>=expected_cassification_rate){
      tmp_val<-tmp_c_rate_upr
      c_rate_tmp_results[i,1]<-steps[i]
      c_rate_tmp_results[i,2]<-abs(target_val_no-tmp_val)
    } else {
      tmp_val<-tmp_c_rate_lwr
      c_rate_tmp_results[i,1]<-1-(steps[i])
      c_rate_tmp_results[i,2]<-abs(target_val_no-tmp_val)
    }
    if(target_val_weak>=expected_cassification_rate){
      tmp_val<-tmp_c_rate_upr
      c_rate_tmp_results[i,3]<-steps[i]
      c_rate_tmp_results[i,4]<-abs(target_val_weak-tmp_val)
    } else {
      tmp_val<-tmp_c_rate_lwr
      c_rate_tmp_results[i,3]<-1-(steps[i])
      c_rate_tmp_results[i,4]<-abs(target_val_weak-tmp_val)
    }
  }


    c_rate_p_95<-1-c_rate_tmp_results[match(x=min(c_rate_tmp_results[,2]),
                                          table=c_rate_tmp_results[,2])
                                    ,1]
    c_rate_p_90<-1-c_rate_tmp_results[match(x=min(c_rate_tmp_results[,4]),
                                          table=c_rate_tmp_results[,4])
                                    ,3]

    res_dev<-c(dev_lwr,expected_deviation,dev_upr,dev_p_01,dev_p_03)
    res_i_rate<-c(i_rate_lwr,expected_error_rate,i_rate_upr,i_rate_p_05,i_rate_p_10)
    res_c_rate<-c(c_rate_lwr,expected_cassification_rate,c_rate_upr,c_rate_p_95,c_rate_p_90)
    results<-rbind(res_dev,res_c_rate,res_i_rate)
    rownames(results)<-c("deviation","classification rate","risk of Type I errors")
    colnames(results)<-c(paste("lower",level,"%"),
                         "mean",
                         paste("upper",level,"%"),
                         "practically no effect",
                         "practically weak effect")
    results<-as.data.frame(results)
    results<-round(results,digits=3)
    format(results, scientific =FALSE)
    return(results)
}

#------------------------------------------------------------------------------
test_that("consequences nominal", {
  test_con_nominal_weak_dyn<-get_consequences(
    measure_typ = "dynamic_iota_index",
    measure_1_val = 1.00,
    level = .95,
    strength = "weak",
    data_type = "nominal",
    sample_size = 500)
  testthat::expect_equal(test_con_nominal_weak_dyn["deviation","practically no effect"],
                         .933)
  testthat::expect_equal(test_con_nominal_weak_dyn["deviation","practically weak effect"],
                         .999)
  testthat::expect_equal(test_con_nominal_weak_dyn["classification rate","practically no effect"],
                         .817)
  testthat::expect_equal(test_con_nominal_weak_dyn["classification rate","practically weak effect"],
                         .958)
  testthat::expect_equal(test_con_nominal_weak_dyn["risk of Type I errors","practically no effect"],
                         .833)
  testthat::expect_equal(test_con_nominal_weak_dyn["risk of Type I errors","practically weak effect"],
                         .964)

  test_con_nominal_medium_dyn<-get_consequences(
    measure_typ = "dynamic_iota_index",
    measure_1_val = 1.00,
    level = .95,
    strength = "medium",
    data_type = "nominal",
    sample_size = 500)
  testthat::expect_equal(test_con_nominal_medium_dyn["deviation","practically no effect"],
                         .849)
  testthat::expect_equal(test_con_nominal_medium_dyn["deviation","practically weak effect"],
                         .999)
  testthat::expect_equal(test_con_nominal_medium_dyn["classification rate","practically no effect"],
                         .002)
  testthat::expect_equal(test_con_nominal_medium_dyn["classification rate","practically weak effect"],
                         .064)
  testthat::expect_equal(test_con_nominal_medium_dyn["risk of Type I errors","practically no effect"],
                         .999)
  testthat::expect_equal(test_con_nominal_medium_dyn["risk of Type I errors","practically weak effect"],
                         .999)

  test_con_nominal_strong_dyn<-get_consequences(
    measure_typ = "dynamic_iota_index",
    measure_1_val = 1.00,
    level = .95,
    strength = "strong",
    data_type = "nominal",
    sample_size = 500)
  testthat::expect_equal(test_con_nominal_strong_dyn["deviation","practically no effect"],
                         .789)
  testthat::expect_equal(test_con_nominal_strong_dyn["deviation","practically weak effect"],
                         .998)
  testthat::expect_equal(test_con_nominal_strong_dyn["classification rate","practically no effect"],
                         .999)
  testthat::expect_equal(test_con_nominal_strong_dyn["classification rate","practically weak effect"],
                         .999)
  testthat::expect_equal(test_con_nominal_strong_dyn["risk of Type I errors","practically no effect"],
                         .999)
  testthat::expect_equal(test_con_nominal_strong_dyn["risk of Type I errors","practically weak effect"],
                         .999)


})
#------------------------------------------------------------------------------
test_that("consequences ordinal", {
  test_con_ordinal_weak_dyn<-get_consequences(
    measure_typ = "dynamic_iota_index",
    measure_1_val = 1.00,
    level = .95,
    strength = "weak",
    data_type = "ordinal",
    sample_size = 500)
  testthat::expect_equal(test_con_ordinal_weak_dyn["deviation","practically no effect"],
                         .877)
  testthat::expect_equal(test_con_ordinal_weak_dyn["deviation","practically weak effect"],
                         .999)
  testthat::expect_equal(test_con_ordinal_weak_dyn["classification rate","practically no effect"],
                         .584)
  testthat::expect_equal(test_con_ordinal_weak_dyn["classification rate","practically weak effect"],
                         .874)
  testthat::expect_equal(test_con_ordinal_weak_dyn["risk of Type I errors","practically no effect"],
                         .997)
  testthat::expect_equal(test_con_ordinal_weak_dyn["risk of Type I errors","practically weak effect"],
                         .999)

  test_con_ordinal_medium_dyn<-get_consequences(
    measure_typ = "dynamic_iota_index",
    measure_1_val = 1.00,
    level = .95,
    strength = "medium",
    data_type = "ordinal",
    sample_size = 500)
  testthat::expect_equal(test_con_ordinal_medium_dyn["deviation","practically no effect"],
                         .724)
  testthat::expect_equal(test_con_ordinal_medium_dyn["deviation","practically weak effect"],
                         .999)
  testthat::expect_equal(test_con_ordinal_medium_dyn["classification rate","practically no effect"],
                         .007)
  testthat::expect_equal(test_con_ordinal_medium_dyn["classification rate","practically weak effect"],
                         .197)
  testthat::expect_equal(test_con_ordinal_medium_dyn["risk of Type I errors","practically no effect"],
                         .999)
  testthat::expect_equal(test_con_ordinal_medium_dyn["risk of Type I errors","practically weak effect"],
                         .999)

  test_con_ordinal_strong_dyn<-get_consequences(
    measure_typ = "dynamic_iota_index",
    measure_1_val = 1.00,
    level = .95,
    strength = "strong",
    data_type = "ordinal",
    sample_size = 500)
  testthat::expect_equal(test_con_ordinal_strong_dyn["deviation","practically no effect"],
                         .617)
  testthat::expect_equal(test_con_ordinal_strong_dyn["deviation","practically weak effect"],
                         .993)
  testthat::expect_equal(test_con_ordinal_strong_dyn["classification rate","practically no effect"],
                         .645)
  testthat::expect_equal(test_con_ordinal_strong_dyn["classification rate","practically weak effect"],
                         .998)
  testthat::expect_equal(test_con_ordinal_strong_dyn["risk of Type I errors","practically no effect"],
                         .999)
  testthat::expect_equal(test_con_ordinal_strong_dyn["risk of Type I errors","practically weak effect"],
                         .999)


})

test_that("Input types for check new rater", {

  #assigned values as factor
  expect_equal(
    check_new_rater(
      true_values = iotarelr_written_exams$`Coder A`,
      assigned_values = iotarelr_new_rater,
      con_random_starts = 20,
      fast = FALSE),
    check_new_rater(
      true_values = iotarelr_written_exams$`Coder A`,
      assigned_values = as.factor(iotarelr_new_rater),
      con_random_starts = 20,
      fast = FALSE),
    tolerance=1e-1
  )
  expect_equal(
    check_new_rater(
      true_values = iotarelr_written_exams$`Coder A`,
      assigned_values = iotarelr_new_rater,
      con_random_starts = 20,
      fast = TRUE),
    check_new_rater(
      true_values = iotarelr_written_exams$`Coder A`,
      assigned_values = as.factor(iotarelr_new_rater),
      con_random_starts = 20,
      fast = TRUE)
  )

  #assigned values as data.frame
  expect_equal(
    check_new_rater(
      true_values = iotarelr_written_exams$`Coder A`,
      assigned_values = iotarelr_new_rater,
      con_random_starts = 20,
      fast = FALSE),
    check_new_rater(
      true_values = iotarelr_written_exams$`Coder A`,
      assigned_values = as.data.frame(iotarelr_new_rater),
      con_random_starts = 20,
      fast = FALSE),
    tolerance=1e-1
  )
  expect_equal(
    check_new_rater(
      true_values = iotarelr_written_exams$`Coder A`,
      assigned_values = iotarelr_new_rater,
      con_random_starts = 20,
      fast=TRUE),
    check_new_rater(
      true_values = iotarelr_written_exams$`Coder A`,
      assigned_values = as.data.frame(iotarelr_new_rater),
      con_random_starts = 20,
      fast=TRUE)
  )

  #true values as factor
  expect_equal(
    check_new_rater(
      true_values = as.factor(iotarelr_written_exams$`Coder A`),
      assigned_values = iotarelr_new_rater,
      con_random_starts = 20,
      fast = FALSE),
    check_new_rater(
      true_values = iotarelr_written_exams$`Coder A`,
      assigned_values = iotarelr_new_rater,
      con_random_starts = 20,
      fast = FALSE),
    tolerance=1e-1
  )

  expect_equal(
    check_new_rater(
      true_values = as.factor(iotarelr_written_exams$`Coder A`),
      assigned_values = iotarelr_new_rater,
      con_random_starts = 20,
      fast=TRUE),
    check_new_rater(
      true_values = iotarelr_written_exams$`Coder A`,
      assigned_values = iotarelr_new_rater,
      con_random_starts = 20,
      fast = TRUE)
  )

  #true values as data.frame
  expect_equal(
    check_new_rater(
      true_values = as.data.frame(iotarelr_written_exams$`Coder A`),
      assigned_values = iotarelr_new_rater,
      con_random_starts = 20,
      fast = FALSE),
    check_new_rater(
      true_values = iotarelr_written_exams$`Coder A`,
      assigned_values = iotarelr_new_rater,
      con_random_starts = 20,
      fast=FALSE),
    tolerance=1e-2
  )
  expect_equal(
    check_new_rater(
      true_values = as.data.frame(iotarelr_written_exams$`Coder A`),
      assigned_values = iotarelr_new_rater,
      con_random_starts = 20,
      fast=TRUE),
    check_new_rater(
      true_values = iotarelr_written_exams$`Coder A`,
      assigned_values = iotarelr_new_rater,
      con_random_starts = 20,
      fast=TRUE)
  )
})

test_iota1<-compute_iota1(
  data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")])
test_iota2<-compute_iota2(
  data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")],
  random_starts = 2,
  trace = FALSE)
test_iota2_dgf<-check_dgf(
  data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")],
  splitcr = iotarelr_written_exams$Sex,
  random_starts = 1,
  trace = FALSE)
test_iota2_new_rater<-check_new_rater(
  true_values = iotarelr_written_exams$`Coder A`,
  assigned_values = iotarelr_new_rater)
#------------------------------------------------------------------------------
test_that("object classes", {
  testthat::expect_s3_class(test_iota2,
                         "iotarelr_iota2"
                        )
  testthat::expect_s3_class(test_iota2_new_rater,
                         "iotarelr_iota2"
  )
  testthat::expect_s3_class(test_iota2_dgf,
                         "iotarelr_iota2_dgf"
  )
  testthat::expect_s3_class(test_iota1,
                         "iotarelr_iota1"
  )
})

#------------------------------------------------------------------------------
test_that("summary", {
  testthat::expect_output(get_summary(test_iota2))

  testthat::expect_output(get_summary(test_iota2_new_rater))

  testthat::expect_error(get_summary(test_iota2_dgf))
  testthat::expect_output(get_summary(test_iota2_dgf$group_female))
  testthat::expect_output(get_summary(test_iota2_dgf$group_male))

  testthat::expect_error(get_summary(test_iota1))
})
#------------------------------------------------------------------------------
test_that("expected categories", {
  test_exp_cat<-est_expected_categories(data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")],
                                        aem=test_iota2$categorical_level$raw_estimates$assignment_error_matrix)
  testthat::expect_s3_class(test_exp_cat,"data.frame")
  testthat::expect_equal(rowSums(test_exp_cat[c("prob_average",
                                                 "prob_good",
                                                 "prob_poor")]),
                         rep(x=1,times=nrow(test_exp_cat)))

})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#'Generating randomly chosen probabilities for Assignment Error Matrix
#'
#'Function written in \code{C++} for generating a set of randomly chosen
#'probabilities for the Assignment Error Matrix.
#'@param n_categories Integer for the number of categories in the data. Must be
#'at least 2.
#'@return Returns a matrix for Assignment Error Matrix (AEM) with randomly
#'generated probabilities. The generated probabilities are in line with
#'the assumption of weak superiority.
#'@importFrom Rcpp sourceCpp
#'@useDynLib iotarelr
get_random_start_values_p <- function(n_categories) {
    .Call('_iotarelr_get_random_start_values_p', PACKAGE = 'iotarelr', n_categories)
}

#'Generating randomly chosen probabilities for categorical sizes
#'
#'Function written in \code{C++} for generating a set of randomly chosen
#'probabilities describing the size of the different classes. The
#'probabilities describe the relative frequencies of the categories in the data.
#'@param n_categories Integer for the number of categories in the data. Must be
#'at least 2.
#'@return Returns a vector of randomly chosen categorical sizes.
#'@importFrom Rcpp sourceCpp
#'@useDynLib iotarelr
get_random_start_values_class_sizes <- function(n_categories) {
    .Call('_iotarelr_get_random_start_values_class_sizes', PACKAGE = 'iotarelr', n_categories)
}

#' Estimating log-likelihood
#'
#' Function written in \code{C++} estimating the log likelihood of a given
#' parameter set.
#'
#' @param categorial_sizes \code{Vector} containing the sizes of the different
#' categories. That is amount of a category on all cases.
#' @param aem \code{Matrix} in aem form. This matrix reports the true category
#' in the rows and the assigned categories in the columns. The cells represent
#' the probabilities that a coding unit of category i is assigned to category j.
#' @param obs_pattern_shape \code{Matrix} containing the unique patterns found
#' in the data. Ideally this matrix is generated by the function
#' \code{get_patterns()}.
#' @param obs_pattern_frq \code{Vector} containing the frequencies of the
#' patterns. Ideally it is generated by the the function
#' \code{get_patterns()}.
#' @param categorical_levels \code{Vector} containing all possible categories of
#' the content analysis.
#' @return Returns the log likelihood as a single numeric value.
#'@references  Berding, Florian, and Pargmann, Julia (2022).Iota Reliability Concept
#'of the Second Generation.Measures for Content Analysis Done by
#'Humans or Artificial Intelligences. Berlin: Logos.
#'https://doi.org/10.30819/5581
#'@importFrom Rcpp sourceCpp
#'@useDynLib iotarelr
fct_log_likelihood_c <- function(categorial_sizes, aem, obs_pattern_shape, obs_pattern_frq, categorical_levels) {
    .Call('_iotarelr_fct_log_likelihood_c', PACKAGE = 'iotarelr', categorial_sizes, aem, obs_pattern_shape, obs_pattern_frq, categorical_levels)
}

#' Estimating log-likelihood in Condition Stage
#'
#' Function written in \code{C++} estimating the log likelihood of a given
#' parameter set during the condition stage.
#'
#' @param probabilities \code{NumericVector} containing the probabilities of
#' a multinominal distribution. In the context of Iota Reliability this refers
#' to a specific row of the Assignment Error Matrix.
#' @param observations \code{NumericVector} containing the number of
#' observations for each category of the multinominal distribution.
#' @return Returns the log likelihood as a single numeric value.
#'@importFrom Rcpp sourceCpp
#'@useDynLib iotarelr
#'@references  Berding, Florian, and Pargmann, Julia (2022).Iota Reliability Concept
#'of the Second Generation.Measures for Content Analysis Done by
#'Humans or Artificial Intelligences. Berlin: Logos.
#'https://doi.org/10.30819/5581
log_likelihood_multi_c <- function(probabilities, observations) {
    .Call('_iotarelr_log_likelihood_multi_c', PACKAGE = 'iotarelr', probabilities, observations)
}

#' Gradient for Log Likelihood in Condition Stage
#'
#' Function written in \code{C++} estimating the gradient of the log likelihood
#' function for a given parameter set and given observations.
#'
#' @param param_values \code{NumericVector} containing the probabilities of
#' a multinominal distribution. The length of this factor is the number of
#' categories - 1 since it contains only the parameters to be estimated.
#' @param observations \code{NumericVector} containing the number of
#' observations for each category of the multinominal distribution. The length
#' of this vector equals the number of categories.
#' @return Returns the gradient as a \code{NumericVector}.
#'@importFrom Rcpp sourceCpp
#'@useDynLib iotarelr
#'@references  Berding, Florian, and Pargmann, Julia (2022).Iota Reliability Concept
#'of the Second Generation.Measures for Content Analysis Done by
#'Humans or Artificial Intelligences. Berlin: Logos.
#'https://doi.org/10.30819/5581
grad_ll <- function(param_values, observations) {
    .Call('_iotarelr_grad_ll', PACKAGE = 'iotarelr', param_values, observations)
}

#' Estimating log likelihood in Condition Stage
#'
#' Function written in \code{C++} estimating the log likelihood of a given
#' parameter set during the condition stage.
#'
#' @param observations \code{NumericVector} containing the frequency of the
#' categories.
#' @param anchor \code{Integer} ranging between 1 and the number of categories.
#' Anchor defines the reference category. That is the category with the highest
#' probability according to the assumption of weak superiority.
#' @param max_iter \code{Integer} specifying the maximal number of iterations
#' for each random start.
#' @param n_random_starts \code{Integer} for the number of random start.
#' @param step_size \code{Double} for specifying the size for increasing or
#' decreasing the probabilities during the estimation. This value should not
#' be less than 1e-3.
#' @param cr_rel_change \code{Double} for defining when the estimation should
#' stop. That is, if the change in log-likelihood is smaller as this value the
#' estimation stops.
#' @param fast \code{Bool} If \code{TRUE} a fast estimation is applied. This
#' option ignored all other parameters. If
#' \code{FALSE} the estimation described in Berding and Pargmann (2022) is used.
#' Default is \code{TRUE}.
#' @param trace \code{Bool} \code{TRUE} if information about the progress of
#' estimation should be printed to the console. \code{FALSE} if not desired.
#' @return Returns the log likelihood as a single numeric value.
#' @importFrom Rcpp sourceCpp
#' @useDynLib iotarelr
#'@references  Berding, Florian, and Pargmann, Julia (2022).Iota Reliability Concept
#'of the Second Generation.Measures for Content Analysis Done by
#'Humans or Artificial Intelligences. Berlin: Logos.
#'https://doi.org/10.30819/5581
#' @export
est_con_multinominal_c <- function(observations, anchor, max_iter = 500000L, step_size = 1e-4, cr_rel_change = 1e-12, n_random_starts = 10L, fast = TRUE, trace = FALSE) {
    .Call('_iotarelr_est_con_multinominal_c', PACKAGE = 'iotarelr', observations, anchor, max_iter, step_size, cr_rel_change, n_random_starts, fast, trace)
}

#'Check assumptions of weak superiority
#'
#'This function tests if the probabilities within the Assignment Error Matrix
#'are in line with the assumption of weak superiority.
#'
#'@param aem matrix of probabilities
#'@return Returns the number of violations of the assumption of weak superiority.
#'0 if the assumptions are fulfilled.
#'@importFrom Rcpp sourceCpp
#'@useDynLib iotarelr
#'@references  Berding, Florian, and Pargmann, Julia (2022).Iota Reliability Concept
#'of the Second Generation.Measures for Content Analysis Done by
#'Humans or Artificial Intelligences. Berlin: Logos.
#'https://doi.org/10.30819/5581
#'@export
check_conformity_c <- function(aem) {
    .Call('_iotarelr_check_conformity_c', PACKAGE = 'iotarelr', aem)
}

#'Parameter estimation via EM Algorithm with Condition Stage
#'
#'Function written in \code{C++} for estimating the parameters of the model
#'via Expectation Maximization (EM Algorithm).
#'
#' @param obs_pattern_shape \code{Matrix} containing the unique patterns found
#' in the data. Ideally this matrix is generated by the function
#' \code{get_patterns()}.
#' @param obs_pattern_frq \code{Vector} containing the frequencies of the
#' patterns. Ideally it is generated by the the function
#' \code{get_patterns()}.
#' @param obs_internal_count \code{Matrix} containing the relative frequencies
#' of each category within each pattern. Ideally this matrix is generated by
#' the function \code{get_patterns()}.
#' @param categorical_levels \code{Vector} containing all possible categories of
#' the content analysis.
#'@param random_starts \code{Integer} for determining how often the algorithm
#'should restart with randomly chosen values for the Assignment Error Matrix
#'and the categorical sizes.
#'@param max_iterations \code{Integer} for determining the maximum number of iterations
#'for each random start.
#'@param rel_convergence \code{Double} for determining the convergence criterion. The
#'algorithm stops if the relative change is smaller than this criterion.
#' @param con_step_size \code{Double} for specifying the size for increasing or
#' decreasing the probabilities during the condition stage of estimation.
#' This value should not be less than 1e-3.
#' @param con_random_starts \code{Integer} for the number of random starts
#' within the condition stage.
#' @param con_max_iterations \code{Integer} for the maximum number of iterations
#' during the condition stage.
#' @param con_rel_convergence \code{Double} for determining the convergence
#' criterion during condition stage. The algorithm stops if the relative change
#' is smaller than this criterion.
#' @param fast \code{Bool} If \code{TRUE} a fast estimation is applied during the
#' condition stage. This option ignores all parameters beginning with "con_".
#' If \code{FALSE} the estimation described in Berding and
#' Pargmann (2022) is used. Default is \code{TRUE}.
#'@param trace \code{TRUE} for printing progress information on the console.
#'\code{FALSE} if this information should not be printed.
#'@param con_trace \code{TRUE} for printing progress information on the console
#'during estimations in the condition stage. \code{FALSE} if this information
#'should not be printed.
#'@return Function returns a \code{list} with the estimated parameter sets for
#'every random start. Every parameter set contains the following components:
#'\item{log_likelihood}{Log likelihood of the estimated solution.}
#'\item{aem}{Estimated Assignment Error Matrix (aem). The rows represent the
#'true categories while the columns stand for the assigned categories. The cells
#'describe the probability that a coding unit of category i is assigned to
#'category j.}
#'\item{categorial_sizes}{\code{Vector} of estimated sizes for each
#'category.}
#'\item{convergence}{If the algorithm converged within the iteration limit
#'\code{TRUE}. \code{FALSE} in every other case.}
#'\item{iteration}{Number of iterations when the algorithm was terminated.}
#'@export
#'@importFrom Rcpp sourceCpp
#'@useDynLib iotarelr
#'@references  Berding, Florian, and Pargmann, Julia (2022).Iota Reliability Concept
#'of the Second Generation.Measures for Content Analysis Done by
#'Humans or Artificial Intelligences. Berlin: Logos.
#'https://doi.org/10.30819/5581
EM_algo_c <- function(obs_pattern_shape, obs_pattern_frq, obs_internal_count, categorical_levels, random_starts, max_iterations, rel_convergence, con_step_size, con_random_starts, con_max_iterations, con_rel_convergence, fast, trace, con_trace) {
    .Call('_iotarelr_EM_algo_c', PACKAGE = 'iotarelr', obs_pattern_shape, obs_pattern_frq, obs_internal_count, categorical_levels, random_starts, max_iterations, rel_convergence, con_step_size, con_random_starts, con_max_iterations, con_rel_convergence, fast, trace, con_trace)
}

# This file is part of the R package "iotarelr".
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as published by
# the Free Software Foundation.
#
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>

#' Sample Vector
#'
#' A vector containing the ratings of a new rater.
#' The data is not real and is only created for illustration purposes.
#' @docType data
#' @format A vector with the length of 318.
"iotarelr_new_rater"

# This file is part of the R package "iotarelr".
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as published by
# the Free Software Foundation.
#
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>

#' Example Data Set
#'
#' A data set containing the ratings of three coders for written exams.
#' It also contains the gender of the people who took the exam. The data
#' is not real and is only created for illustrating purposes.
#' @docType data
#' @format A data frame with 318 rows and 4 variables:
#' \describe{
#'   \item{Coder A}{Ratings of coder A.}
#'   \item{Coder B}{Ratings of coder B.}
#'   \item{Coder C}{Ratings of coder C.}
#'   \item{Sex}{Referring to the biological aspects of an individual.}
#' }
"iotarelr_written_exams"

---
title: "Get started"
author: "Florian Berding and Julia Pargmann"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{01_get_started}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```
# 1 Introduction
*iotarelr* is a package for estimating the components of the Iota Reliability Concept.
The first version of this concept is described by Berding et al. (2022). The current
version is Iota2 (Berding & Pargmann 2022).

In this vignette, we describe how to use the package on the basis of the current 
version. We will show how to calculate and to interpret the basic
components. Advanced analyses are described with separate vignettes which you can
find via articles.

In order to use *iotarelr*, you need to load the package at the beginning of your
analysis.
```{r setup}
library(iotarelr)
```

In this introduction, we use a sample data set which is called "iotarelr_written_exams". 
The data set does not contain real data from written exams. It is a data set which 
was created for illustration purposes only. Let us have a look at it.
```{r}
head(iotarelr_written_exams)
```
The data set contains four columns. The first three columns contain the ratings from
three coders. That is, every written essay is scored by three people. In terms 
of content analysis, three raters judged every essay, representing the coding units. 
An essay can be rated as "poor", "average" or "good". Thus, three categories are available.
The fourth column contains the gender of the person who wrote the essay. This column, however,
is not important at the moment. Such kinds of data can be used to analyze if the
codings work similarly across different groups of people or different groups of 
materials. This kind of analysis is described in the vignette [Different Guidance Functioning](V_02_Estimating_Consequences_for_Subsequent_Analyses.html).
Keep in mind that at the moment, we are ignoring this column.

The *Iota Concept* provides measures which allow to inspect the reliability of single
categories as well as a complete scale. This is different from traditional measures
of reliability for content analysis, such as *Percentage Agreement*, *Cohen's Kappa* or
*Krippendorff's Alpha* which focus on the scale level only. Since in the literature of
content analysis, the terms "category", "scale" and "dimension" are used differently, it
is important to understand how the *Iota Concept* uses these terms. Figure 1 illustrates the understanding.

![Figure 1: Scale and Categories in Iota Concept](iotarelr-terms.png){width=100%}

Regarding our example, the performance in the exams is the scale which is made up
of three categories: "good", "average" and "poor". Since the categories can be
sorted by degree or performance, they form an ordinal scale. An example for
a nominal scale are colors. Here, the different colors are the categories. They
make up the scale "color".

It is very important to note that the *Iota Concept* differentiates between
two layers of data, which are shown in Figure 2.

![Figure 2: Layers of Data within Iota Concept](iotarelr-introduction.png){width=100%}

The coding units are the units to which a coding scheme is applied. In our example,
these are the exams. Each exam has a **true category**.
That is, an exam is good, average or poor. These categories make up the 
**true data**. However, the true categories are not inherently known. Thus, raters use a
coding scheme to decide about the category of a coding unit. In the example,
the raters have to judge if an exam is good, average or poor and assign the 
corresponding category to that coding unit/exam. All assignments make up the **labeled data**. 

However, it is not ensured that the assigned category is the same as the true category.
For example, in Figure 2, exam 1 is assigned to the category "good", which is also
the true category. Exam 2 is also assigned to the category "good", although in truth,
the exam represents an average exam. Alike applies to exam 3. The assigned category 
and true category should match, but this is not certain.

Since the assigned category and the true category may differ, the labeled data can vary
in the degree of how well it reflects the true data. That is, the degree of
overlapping between the true data and labeled data can vary. In Figure 2, three exams
are assigned to be good exams. Thus, the data set contains three cases which are 
labeled as good exams. Although the cases
are labeled as good, only one case really is a good exam. Thus, the true data
and the labeled data for the category "good" overlap by 33%. For the category
"average", true data and labeled data overlap by 0% since there is one exam
that in truth is an average exam. However, there is no labeled data representing
this. Something similar applies for the category "poor".

The aim of reliability is to ensure that the **labeled data** matches the **true data**
as well as possible. To achieve this aim, the Iota Concept models the coding process
and provides measures for quantifying the degree of reliability as shown in the
following sections. 

# 2 Example for using the Iota Concept of the second generation
## 2.1 Estimation
The calculation of all components of Iota2 can be requested by using 
`compute_iota2()`. At this stage, it is crucial only to use the part of 
the data set which contains the codings. In every case, the coding units have to
be in the rows and the raters in the columns. We first save the codings in a new
object, request the estimation of Iota2 and save the results.
```{r}
codings<-iotarelr_written_exams[c("Coder A","Coder B","Coder C")]
res_iota2<-compute_iota2(data=codings,
                         random_starts = 10,
                         trace = FALSE)
```
The estimation of all components of Iota2 can take a moment. If you would like to 
see the progression, you can set `trace = TRUE`. After the estimation has finished,
we can request a summary with the function `get_summary()`.
```{r}
get_summary(res_iota2)
```
The summary first shows basic information such as the number of raters, the 
number of categories, the concrete categories and the number of coding units. 
Next, the summary presents the
log-likelihood which to some extent describes  the quality of the model. Lower
values imply a better fitting model than higher values. Additionally, the summary
prints whether or not the the best log-likelihood could be replicated. If this is not the case,
we recommend to increase the number of random starts and/or to investigate the
solutions to ensure that the subsequent analyses are based on the best model possible
.

## 2.2 Analysing the reliability on the categorical level
### 2.2.1 Interpretation of the central components
Next, the summary shows the results for the primary parameters. These are the
*Assignment Error Matrix* and the *categorical sizes*, which are the basis for all
other measures of the *Iota Concept*. 

The *Assignment Error Matrix* describes the rating process. The true category is
always in the rows and the assigned categories are always in the columns. In
the current example, the first row describes how exams representing an average
performance are assigned to the three different categories. An exam
representing an average performance is assigned to be such an exam in about 91.7%
of cases. In about 8.3% of cases, an average exam is assigned to be a good
exam. Finally, an average exam is never assigned to be a poor exam. Here, the corresponding
probability is about 0%.

When concentrating on the exams which reflect a good performance, the second row of
the *Assignment Error Matrix* is important. In about 49.3% of cases, a good
exam is assigned as a good exam. In other words: The probability to recognize a 
good exam as a good exam equals flipping a coin. In about 32.6% of cases, a good exam is assigned
to the category "average". That is, this exam is judged as showing an average performance, although it
is truly a good exam. Finally, a good exam is rated as a "poor" exam in about 18.1%
of cases. Thus, the raters make more mistakes in evaluating a good exam than an average exam (49.3% for good exams compared to 91.7% for average
exams). 

The *categorical sizes* represent an estimate for the distribution of the **true** categories
in the population. In this example, we expect about 11.9% of all exams to be
average, 39.6% to be good and 48.5% to be poor.

Based on the *Assignment Error Matrix* and the *categorical sizes*, additional measures
of reliability can be derived. These are shown in the next lines of the summary.
The first is the *Alpha Reliability*, which corresponds to the diagonal of 
the *Assignment Error Matrix*. This determines the probability that a coding unit of a specific 
category is correctly assigned to that category. In this example, a coding unit truly
belonging to the category "average" is assigned to the category "average" in about 91.71%
of cases. In other words: An exam truly representing an average performance is
rated as an average exam in about 91.71% of cases. This value is quite high. 
In contrast, the *Alpha Reliability* of the category "good" is only .4933. This means that a coding unit truly belonging to the category "good" is assigned "good" only in
about 50% of cases. This is quite a bad result.
Alike applies for the category "poor".

The next row presents the *Beta Reliability*, representing
the probability that errors on **other** categories do **not** influence the category under 
investigation. In this example, the *Beta Reliability* of the category "average" 
is about .3718. This means that if a rater makes a mistake on the categories "good"
or "poor", the mistake is **not** assigned to the category "average" with a probability
of 37.18%. In other words: If a rater has to judge an exam which truly represents 
a good or poor performance and if the rater does not recognize the good or poor 
performance, they do **not** assign the exam to the category "average" with a probability of 37.18% .
Focusing on
the category "poor", the *Beta Reliability* is about .6596. This means: If a rater
has to judge an exam which truly reflects an average or a good performance and if
the rater does not recognize the good or average performance, they do **not** assign that
exam as poor in about 65.96 % of the cases. Thus, the *Beta Reliability* can be 
used to show how errors on the other categories influence the data generated 
for a specific category. The higher the values, the less influenced the data 
for that category. In this example, the category "average" is more influenced by
errors on both of the other categories than the categories "good" and "poor".

Finally, *Iota Reliability* combines the *Alpha* and the *Beta Reliability* by taking the
sizes of the categories into account. *Iota Reliability* is the core of the *Iota Concept*.
*Iota Reliability* is a measure that describes how well the data labeled as a specific category 
really reflects that category. It considers
three cases:

- all cases which are assigned to the correct category (*Iota*)
- all cases of that category which are not recovered (*Iota Error I*)
- all cases that are assigned to that category although they truly belong to 
another category (*Iota Error II*). 

Before we start to interpret these values for the case of the example, some additional explanations
are necessary. Figure 3 illustrates all three cases.

![Figure 3: Illustrating Iota](iotarelr-iota.png){width=100%}

Referring to the first row in Figure 3, the box represents all coding units that
**truly** belong to category *i*. In case of perfect reliability, the **labeled data**
corresponds to these coding units. 

Row two in Figure 3 splits the coding units of that category. The green part of the box
represents the coding units which in truth belong to category *i* and which are part
of the labeled data. The yellow part of the box represents the number of coding
units which in truth belong to category *i*, but which are not assigned to category *i*.
These coding units are missing in the labeled data. In the situation presented 
in row two, the total number of coding units representing category *i*
is underestimated in the labeled data. 

Let us now turn to the third row to provide a complete view. The 
**labeled data** of category *i* can also include coding units that **truly** belong
to **other** categories. These coding units are represented by the red box.
They are results from coding errors made in other categories and
increase the number of units labeled as category *i*, although in truth, they do not belong
to category *i*. Thus, these units contribute to an overestimation of 
the number of coding units truly belonging to category *i*.

*Iota* refers to the green box of row 3, *Iota Error I* refers to the yellow box
and *Iota Error II* refers to the red box. All three measures are expressed as a
ratio on $N+n_{Iota Error II}$. 

With this in mind, we can now interpret the *Iota Reliability*. The easiest way is by
visualizing the values. This can be done with the function `plot_iota()`.

```{r, fig.height = 3, fig.width = 7.2, fig.align = "center", fig.cap="Figure 4: Example Plot of Iota"}
plot_iota(res_iota2)
```

***Category average:*** Let us start with the category "average". For this category, *Iota* is about .268.
This means that about 26.8% of the relevant labeled data really are average exams. 
The remaining 73.2% represent average exams that are missing in the results (*Iota Error I*) or
represent exams that truly belong to other categories (*Iota Error II*). With the help 
of the plot, both of these cases can be characterized in more detail. The yellow rectangle 
represents the average exams which are not part of the labeled data. The number 
.024 indicates that only a few exams
are "forgotten". The red rectangle describes the amount of exams that are part of 
the labeled data for the average exams, but which are in truth good or poor exams.
They make up about 70% of the labeled data. To sum up, the data representing the average
exams 

- recovers most of the exams that really are average exams (*Iota*).
- ignores only a small amount of exams which really are average exams (*Iota Error I*).
- is extremely biased by exams which in truth are good or poor (*Iota Error II*). Their
amount is greater than *Iota* and *Iota Error I* combined, indicating that the number of
coding units for average exams is extremely overestimated by the labeled data compared 
to the true data. 

Thus, although the category "average" is quite reliable, the low 
reliability of both of the other categories destroys the high quality of the part of 
the data that should represent the average exams. 

***Category good:*** Turning to the category "good", *Iota* is about .387, implying that about 38.7%
of the data labeled as "good exams" really are good exams. However, nearly the 
same number of good exams are not represented in the data, as the yellow rectangle
in the plot shows. Thus, half of the good exams are missing in the labeled data.
$\frac{.398}{.387+.398}=.507$.
With a value of .215, only about 21.5% of the data is made up by
exams that truly represent an average or a poor performance. Thus, the data 
labeled as "good exams"

- only consists of exams which are in truth good exams to about 39% (*Iota*).
- ignores about 50 % of the exams which are in truth good exams (*Iota Error I*).
- is weakly biased by exams that are in truth average or poor exams (*Iota Error II*).

In contrast to the data representing the average exams, the data representing the
good exams is not extremely biased by those from other categories, but ignores 
a high number of relevant ones. Thus, the labeled data underestimates the number
of truly good exams compared to the true data.

***Category poor:*** Finally, let's have a look at the category "poor". Here,
*Iota* is about .408, implying that the relevant data consist to 40.8% of the 
correct coding units/exams. However, *Iota Error I* is about .463, implying that
a high number of truly poor exams is not represented within the labeled data for
that category. *Iota Error II* is about .129 which is quite low, implying that about
12.9% of the relevant data is made up of coding units truly representing good or
average exams. Thus, the data representing the poor exams

- consists only to about 40.8%  of exams which are in truth poor exams (*Iota*).
- ignores about 53.2% of the exams which are in truth poor exams (*Iota Error I*).
- is weakly biased by exams that are in truth good or average exams (*Iota Error II*).

The number of poor exams is underestimated by the labeled data compared to the true data.

***Summary:*** To sum up, the sample coding scheme guiding the ratings of the exams requires a revision.
The data generated by the coding scheme is heavily biased in favor of 
average exams. The number of average exams is overestimated since a lot of good
and poor exams are treated as average exams. As a consequence, the number of good 
and poor exams is underestimated. Since the *Alpha Reliability* of the category
"average" is quite high, a revision should focus on the categories "good" 
and "poor". This revision should aim to provide more precise definitions and clearer
or more suitable examples that support raters in perceiving the true categories of the exams.

### 2.2.2 Coding Stream Analysis
Since version 0.1.3, *ioatrelr* provides an additional analysis possibility via
the function `plot_iota2_alluvial()`. This function takes an an estimated object
of Iota2 and returns a plot representing the coding process guided by a coding scheme.
For the current example, the plot is shown in Figure 5.
```{r, fig.height = 5, fig.width = 7.2, fig.align = "center", fig.cap="Figure 5: Alluvial Plot of Iota2"}
plot_alluvial<-plot_iota2_alluvial(res_iota2)
plot(plot_alluvial)
```
The boxes on the left side represent how the true categories are distributed in 
the population. Their height directly reflects the estimated true categorical sizes. 
The boxes on the right side show the amounts in the labeled data.
The curves between the two pairs of boxes illustrate the coding process. 

**Average exams:** Concentrating on the true average exams, the plot illustrates that most of the
true average exams are part of the labeled data. This can be seen by the big red
curve ranging from the true average exams to the labeled average exams. Only
a very small number of true average exams is assigned as good exams. This can be
seen on the very small red curve ranging from the true average exams to the good exams.
Thus, most of the true average exams are correctly assigned. 

However, the box for the labeled average exams is several times larger than the box 
for the true average exams. This indicates that the coding scheme tends to 
overestimate the number of average exams. Figure 5 illustrates that the source for
this overestimation is in the truly good and the truly poor exams. This can be seen
by the big green curve ranging from the truly good exams to the labeled average exams
and by the big blue curve ranging from the truly poor exams to the labeled average
exams. 

**Good exams:** A similar analysis is possible for the good exams. The three green curves
ranging from the truly good exams to the other categories illustrate how the truly
good exams are assigned to the different possible categories. The two big curves
show that most of the truly good exams are assigned to the categories "good" and
"average". The smaller green curve indicates that a smaller number of truly good
exams are assigned to the category of poor exams. The box for the truly good exams 
is smaller than the box for the labeled good exams indicating that the labeled
data underestimates the number of good exams. The coding units behind the labeled
data for the good exams is made up mostly by truly good exams (big green curve) followed by a high
number of truly poor exams (big blue curve) and only a very small number of truly average exams (red curve). 

**Poor exams:** Finally, the box for the labeled poor exams is about 0.5 times 
smaller than the box for the truly poor exams, indicating that the total
number of poor exams is underestimated by the coding scheme. The three blue curves
illustrate that most of the truly poor exams are assigned to the category "poor"
in the labeled data. A similar number of truly poor exams is assigned to the
category "average" and only a smaller number is assigned to the category "good".

Summing up, the alluvial plot for Iota2 provides additional insights into 
how a coding scheme works by illustrating the coding stream and splitting up
*Iota*, *Iota Error I* and *Iota Error II* into more detail.This can be very helpful
for judging the quality of data and for reviewing a coding scheme. 

## 2.3 Analysing the reliability on the scale level
In many cases, not only the reliability of singles categories is important but
also the reliability of the complete scale. The *Iota Concept* provides several
measures for characterizing this kind of reliability. Currently, we recommend
to use the *Dynamic Iota Index*. This measure ranges from 0 to 1, 0 indicating
the absence of reliability, 1 indicating a perfect reliability. Referring
to the summary above, the *Dynamic Iota Index* is about .267. This is a low value
which replicates the analysis on the categorical level.

According to the the rules of thumb developed by Berding and Pargmann (2022), this
value is not sufficient to generate reliable data. The value should be at
least .829 to allow subsequent statistical analysis such as correlation analysis 
or significance testing. This is due to the fact that this degree of reliability leads to
high deviations from the true sample correlation and suffers from a high risk of
drawing wrong conclusions from statistical analyses.

# 3 Further readings
In this vignette, we presented the very basic usage of the *Iota Concept*. If you would
like to lean more about the consequences of reliability on the scale level, we would
like to recommend the vignette 
[Estimating Consequences for Subsequent Analysis](V_02_Estimating_Consequences_for_Subsequent_Analyses.html). 
If you are interested in how the *Iota Concept* can be used to analyze if a coding
scheme performs similarly for different group of materials, we recommend the
vignette [Different Guidance Functioning](V_03_Different_Guidance_Functioning.html).

As we are permanently working on improving *Iota Concept*, please check the
package's website for news.

For more details, we would like to refer to the following literature:

-   Florian Berding and Julia Pargmann (2022). Iota Reliability Concept
    of the Second Generation. Measures for Content Analysis Done by
    Humans or Artificial Intelligence. Berlin: Logos.
    <https://doi.org/10.30819/5581>
    
-   Florian Berding, Elisabeth Riebenbauer, Simone Stuetz, Heike
    Jahncke, Andreas Slopinski, and Karin Rebmann (2022). Performance and
    Configuration of Artificial Intelligence in Educational
    Settings. Introducing a New Reliability Concept Based on Content
    Analysis. Frontiers in Education.
    <https://doi.org/10.3389/feduc.2022.818365>

---
title: "Old 1) How to use Iota1"
author: "Florian Berding and Julia Pargmann"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Old_01_How_to_use_Iota1}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
# Important Note
**Please note that Iota1 is outdated. Please use the new version Iota2.**
**Central definitions of *Iota* have changed in the new version.**
**Please refer to [Get started](iotarelr.html)**

**This vignette is not updated to future developments.**

# 1 Introduction
Reliability is a central characteristic of any assessment instrument, and describes the extent to which the instrument produces error-free data (Schreier, 2012). In terms of content analysis, Krippendorff (2019) suggests replicability as a fundamental reliability concept, which is also referred to as inter-coder reliability. This describes the degree to which "a process can be reproduced by different analysts, working under varying conditions, at different locations, or using different but functionally equivalent measuring instruments" (Krippendorff, 2019).

The package *iotarelr* provides an environment for estimating the degree of inter-coder reliability
based on the Iota Reliability Concept developed by Berding et al. (2022). The concept
provides one of the first measures for characterizing the degree of reliability for a complete scale and
for *every single category*. Most of the older measures are limited to information on the complete scale.

The suggested concept is applicable to any kind of content analysis that uses a coding scheme with nominal or ordinal data regardless the kind of coders (human or artificial intelligence), the number of coders, and the number of categories. 
The following introduction shows how to use *Iota1*.

# 2 Example for using iotarelr in practice
## 2.1 Estimating the values

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```
At the beginning, data generated by at least two coders is needed. Let us assume that four coders analyzed 20 textual fragments with a coding scheme consisting of three categories A, B, and C.

```{r setup}
library(iotarelr)
coder_1<-c("A","A","C","C","B","A","A","B","C","A","B","C","A","B","A","B","C","A","A","C")
coder_2<-c("A","A","C","B","B","A","A","B","C","A","A","C","A","C","A","B","A","A","A","C")
coder_3<-c("A","A","C","C","B","A","A","B","C","A","B","C","A","B","A","B","C","A","A","C")
coder_4<-c("A","C","C","C","B","A","A","B","C","A","B","A","A","B","A","C","B","A","A","C")

coded_data<-cbind(coder_1,coder_2,coder_3,coder_4)
```
In this example, the characteristics are saved as characters. The package also supports that the categories are stored as integers. The only important aspect is that the rows must contain the coding units (e.g., the textual fragments), and that the columns represent the different coders. In the next step, the estimation of iota and its elements starts via the function `compute_iota1()`.
```{r}
results<-compute_iota1(data=coded_data)
```

## 2.2 The alpha values
```{r}
results$alpha
```
The component `$alpha` saves the values for the *chance-corrected* alpha reliabilities. These values describe the extend in which the true characteristic of a coding unit is discovered. It ranges from 0 to 1. 0 indicates the absence of reliability. That is, the assignment of the true category equals a random selection between the categories. 1 indicates that the true value is always recovered.

In the current example, the chance-corrected alpha value for category A is relatively high. This means that the coding scheme leads coders  to assign characteristic A to a coding unit if the true characteristic of the coding unit is A with a high probability. In contrast, the values for the other two categories are very low. This indicates that a coding unit with the true characteristic B or C is often assigned to the other categories. That is, the coding scheme does not ensure that coders discover the true category if the true category is B or C.

## 2.3 The beta values
To provide a more detailed insight of the coding scheme, the beta values account for the errors which occur in the case that the true category is not discovered. That is, the beta values describe the extent to which a category is influenced by errors made in other categories. For example, if the true category of a coding unit is A and a coder does not assign A to that coding unit, the data for category B and C are biased. The data representing categories B and C comprises a coding unit that should not be part of the data.

The *chance-corrected* values for the beta reliabilities are stored in `$beta` and range between 0 and 1. 0 indicates that the beta reliability equals the beta reliability in the case of complete guessing. 1 indicate the absence of any beta errors.
```{r}
results$beta
```
In the current example, the chance-corrected beta reliabilities are relatively high. This means that the different categories are not strongly influenced by errors in the other categories.

## 2.4 The iota values
Iota values summarize the different types of errors for each category by averaging the chance-corrected reliabilities. They are stored in `$iota`.
```{r}
results$iota
```
Iota can range between 0 and 1. 0 indicates that the quality of the coding of a category equals random guessing. Codings of this category are not reliable. 1 indicates a perfect reliability of a category. That is, the true value of a coding unit is recovered if the true category is the category under investigation and errors made by coding coding units of other categories do note influence the data of the category under investigation. In the current example category A is quite reliable while the other categories are not.

## 2.5 Assignment-Error Matrix (AEM)
The assignment-error matrix combines the alpha and beta values and provides the most detailed description of a coding scheme. It is based on the raw estimates without any chance-correction.
```{r}
results$assignment_error_matrix
```
The AEM has to be read row by row because the rows represent the true category of a coding unit and the columns represent the assigned categories. The values on the diagonal represent the alpha-error of the categories. That is the probability *not* to assign the true category to a coding unit. The other cells describe the probability to assign the category to the other categories. That is, they inform about the probability to choose a category under the condition that the true category is *not* recovered.

In the example, the alpha-error of category A is about .43 meaning that a coding unit of category A is coded as another category in about 43 %. Or in other words: The probability to assign the true category to a coding unit of category A is about 57%. Thus, the second and third cell in the first row mean: If the true category of a coding unit belonging to category A is *not* recovered, about 45% of the codings are assigned as category B and 55% of the codings are assigned to category C. Thus, category C suffers more from errors made with coding units truly belonging to category A than category B.

The alpha-error of category B is about .85. Thus in about 85% of cases the coding units truly belonging to category B are assigned to another category. In other words: The probability to recover the true category is about 15 % if the true category of the coding unit is B. In the case that this error occurs, 40% of the cases are assigned as category A and 60 % are assigned to category C. Thus, the data of category C suffers more from errors made on coding units belonging truly to category B than category A.

The alpha-error of category C is about 79%. Thus, in about 79% of the cases a coding unit truly belonging to category C is assigned to category A or B. If this error occurs 44 % of the codings units are treated as category A and 56% are treated as category B. In consequence, category B suffers more form errors made with codings units of category C than category A. In other words: The data for category B is more strongly biased by errors of category C than the data of category A.

## 2.6 Scale level
The measures described above provide detailed insights into the reliability of every single category which is a new feature for content analysis and very helpful for constructing a coding scheme or for evaluating data of empirical studies. In many applications however, the values have to be summarized to values representing the quality of the complete coding scheme. In the Iota Concept this is done by averaging the iota values for each category. The value is accessible by `$average_iota`.
```{r}
results$average_iota
```
In the current example average iota is about .49. At the moment only a rule of thumb for ordinal data exist. According to Berding et al. (2022) an average iota of at least  .474 is necessary for an acceptable level of reliability on the scale level. For a "good" reliability average iota should be at least .601.

# References
- Berding, F., Elisabeth r., Simone S., Jahncke, H., Slopinski, A., and Rebmann, K. (2022). 
Performance and Configuration of Artificial Intelligence in Educational Settings.
Introducing a New Reliability Concept Based on Content Analysis. 
Frontiers in Education. https://doi.org/10.3389/feduc.2022.818365
- Krippendorff, K. (2019). Content Analysis: An Introduction to Its Methodology 
- Schreier, M. (2012). Qualitative Content Analysis in Practice. SAGE. 

---
title: "2) Estimating Consequences for Subsequent Analyses"
author: "Florian Berding and Julia Pargmann"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{02_estimating_cons}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```
# 1 Introduction
In the first vignette [Get started](iotarelr.html), rules of thumb are used
to evaluate the degree of reliability. In this vignette, we would like to
demonstrate that a more situation-specific evaluation of reliability is possible. In
doing so, we continue the example of the first vignette, which you can find 
[here](iotarelr.html). The focus of this kind of analysis is the function 
`get_consequences()`, which can be used to evaluate pretests of coding schemes, as well as
for planning or evaluating existing studies.

The cut-off values for judging reliability are derived in a way that ensures
a high data quality in many practical situations. In the study of Berding and Pargmann (2022),
the most demanding situations are chosen. Thus, for some applications, these 
rules of thumb may be too strict. Additionally, it may be helpful to understand 
the consequences induced by specific reliability values.

To illustrate this kind of analysis, we need to extend our example by introducing
a new variable. Let us assume that all participants of the exam were interviewed 
in order to assess how confident
the students are about their abilities to successfully pass the exam. Let us further 
assume that the degree of confidence varies between "low", "medium" and "high".

In analogy to studies investigating the relationship between achievement and
self-concept (Huang 2011; Möller et al. 2020), we assume that participants with better exams have higher 
confidence in their abilities. Figure 1 illustrates this relationship.

![Figure 1: Assumed Relationship between Performance and Ability Self Concept](con_and_values_intro.png){width=100%}

To prove this relationship, we use the data from the exams and the rated interviews.
Whether or not we can draw the right conclusions from the data depends on the reliability of
the generated data, the sample size and the sample method. This can be explored 
in more detail with the function `get_consequences()`

# 2 Using *get_consequences()*

The function `get_consequences()` provides information about the impact of reliability
on significance testing and drawing conclusions. It requires at least
four arguments. With `measure_typ` you can decide which measure of reliability 
 you would like to use on the scale level. Currently, we recommend to use 
`measure_typ = "dynamic_iota_index"` because in our analysis, this measure showed the
highest value for *R²* in predicting different kinds of data quality.

With `measure_1_val` you can set the reliability of the independent variable. In our
example, this is the performance a participant shows in their exam. Here, the value for
*Dynamic Iota Index* is about .267 (see [Get started](iotarelr.html)). With 
`measure_2_val` you can set the reliability of the dependent variable. If this
value is not set explicitly, the analysis assumes the same reliability as for the
independent variable. In our example, the dependent variable refers to the data
of the interviews showing participants' confidence in their abilities. Let us 
assume that the corresponding value of *Dynamic Iota Index* is about .879.

The argument `data_type` sets the scale level. Currently, "nominal" and "ordinal"
are possible. In the case of nominal data, all results of the function refer to
significance tests with *Cramer's V*. In the case of ordinal data, all statistics
refer to significance tests with *Kendall's Tau*. In our example, both scales
form an ordinal scale. 

The argument `strength` is closely connected to the data type and refers to the
**true** strength of a relationship between the independent and dependent variable. The
argument can be set to "no", "weak", "medium" and "strong". These categories are
based on the work of Cohen (1988), who classified statistical measures according to
their relevance for real-world applications. Thus,"no" does not imply a value of 0 for 
*Kendall's Tau*, but a small value around 0. "Strong" does not imply a perfect
relationship, but rather refers to values above .5. Cohen's (1988) work does not explicitly deal
with *Kendall's Tau* but instead employs Pearson correlation. For *Cramer's V*, the situation
is more complicated, as the class of the effect size depends on the number of categories.
However, this is considered in the function. In our example, we assume a medium
relationship between performance and confidence.

The argument `sample_size` refers to the sample size of a planned 
or already realized study. In our example, three raters judged the written exams of 318
participants and analyzed their corresponding interviews.

Finally, `level` refers to the certainty level of the calculated prediction intervals. 
A prediction interval characterizes the probability that the true value is 
within a specific range around the prediction (Afifi et al. 2020, p. 119). In the 
current example we choose 95%.

```{r calculation}
library(iotarelr)
get_consequences(measure_typ = "dynamic_iota_index",
                 measure_1_val = .267,
                 measure_2_val = .879,
                 data_type = "ordinal",
                 strength = "medium",
                 sample_size=318,
                 level = 0.95)
```

The function calculates three important aspects when investigating relationships.

**Deviation:** The first row called "deviation" characterizes the expected deviation between the
estimated sample effect size and the true sample effect size. Since we are using
ordinal data, the effect size is  *Kendall's Tau*. The mean value implies that we
expect that *Kendall's Tau* differs from an error-free assessment by about .260 units.
This is quite high if we use Cohen's (1988) classification for Pearson correlation. 
Here, the meaning of a correlation changes every .20 units. The upper and lower values mean that with a certainty of 95%, the estimated value for *Kendall's Tau*
differs from an error-free estimation by .107 to .413 units.

The column "practically no effect" reports the probability that the effect size does
not deviate more than .1 units. With a reliability of .267 for the exams and of
.879 for the interviews, this chance is about 2%, which is very low. The last column,
"practically weak effect", reports the probability that *Kendall's Tau* deviates
less than .3 units from an error-free measurement. This probability is about 70%.
At first glance, this seems to be very high. However, this probability implies
that in nearly one third of all studies with the same research design, the values
deviate by more than .3 units.

**Classification rate:** The second row is closely connected to the deviation. It
describes the chance to correctly classify an effect size as practically not relevant,
weak, medium or strong, based on Cohen's (1988) classification. The mean value
implies that the chance to correctly classify the effect size is about .9%, which is very low.
The upper and lower values state that with a certainty of 95%, the chance 
to correctly classify the effect size is between .2 and 2.8%. It becomes 
clear that with the current reliability of .267 for the exams and .879 for 
the interviews, the risk of drawing the wrong conclusion about the strength of the
relationship is immense.

The column "practically no effect" reports the probability that the chance to
correctly classify the effect size is at least 95%, while the last column reports that 
the probability to correctly classify the effect size is at least 90%. In both
cases the probability is zero.

**Type I errors:** The last row refers to the risk of Type I errors. Type I errors
 mean in this context that the significance test implies the acceptance of the null
hypothesis, while an error-free measurement would imply the rejection of the null hypothesis.
In other words: The results of the significance test imply that there is no relationship, although
an error-free measurement would imply the acceptance of a relationship.

The mean value implies that we have to expect a chance of 21.5% of a Type I error.
That is, in 21.5% of cases, the results of the significance tests imply that
there is no relationship between participants' performance in the exam and the
confidence in their abilities, although an error-free measurement would imply
the existence of such a relationship. The lower and upper values imply that
with a certainty of 95%, the risk for Type I errors is between 6.4% and 47.9%.
Thus, there is a high risk for drawing the wrong conclusions.

The column "practically no effect" reports the probability that the risk of Type I
errors does not exceed 5% while the last column reports the probability that
the risk of Type I errors does not exceed 10%. In the current example, the probability
for no relevant effect is about 1.1 % and for a weak effect about 9.5%.

Summing up, the information provided by `get_consequences()` helps to make the 
suggested cut-off values more specific. The information can be used to judge
the degree of reliability in a more situation-specifical manner and can support both the planning
and the evaluation of studies. 

# 3 Limitations
It is important to note that this analysis has some limitations. First,
only two types of significance tests are supported (*Cramer's V* and *Kendall's Tau*).
Second, the analysis assumes that the independent and dependent variables both have
the same number of categories. Thus, please use the results as an orientation.

# References
- Afifi, A., May, S., Donatello, R. A. & Clark, V. A. (2020). Practical Multivariate Analysis (6th ed.). Chapman and Hall/
CRC Texts in Statistical Science Ser. CRC Press LLC. 
- Huang, C. (2011). Self-concept and academic achievement: a meta-analysis of 
longitudinal relations. Journal of school psychology, 49(5), 505–528. 
https://doi.org/10.1016/j.jsp.2011.07.001
- Cohen, J. (1988). Statistical Power Analysis for the Behavioral Sciences 
(2nd Ed.). Taylor & Francis.
- Möller, J., Zitzmann, S., Helm, F., Machts, N. & Wolff, F. (2020). 
A Meta-Analysis of Relations Between Achievement and Self-Concept. 
Review of Educational Research, 90(3), 376–419. 
https://doi.org/10.3102/0034654320919354

---
title: "5) Test New Raters"
author: "Florian Berding and Julia Pargmann"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{05_new_rater}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```
# 1 Introduction
Many scientific studies include the development of a new coding scheme for their
study which is typically the focus of introductory literature of content analysis 
(Krippendorff, 2019; Kuckartz, 2018; Mayring, 2015; Schreier, 2012). In these cases, 
the reliability can be estimated and analyzed as described in the first [vignette](iotarelr.html). 
In contrast to the literature for new coding schemes, less recommendations refer to the application of existing coding schemes, 
which are available from published studies or other sources. In these cases, it is 
not necessary to develop a new scheme, but rather to apply an existing one to new 
data, often along with new raters. The application of an existing coding scheme to 
new data has several reasons. 

- First, studies using the same coding scheme can be directly compared, contributing 
to knowledge accumulation in a specific topic or discipline. 
- Second, applying an existing coding schemes provides the opportunity to prove 
the results of prior studies by trying to reproduce them. 
- Third, using an existing scheme saves resources, since ideally, the improvement 
cycle for developing is not necessary. This saves capacities which then become available 
for other important study characteristics (greater sample sizes, refining specific categories, 
considering more categories, etc.).

Estimating the reliability of codings based on an existing coding scheme differs 
from estimating the reliability in the development a coding scheme. The phase of developing 
a coding scheme aims to provide a theoretically and empirically sound guide for 
data analysis. This includes that researchers and raters clear up their own
interpretation of categories and data and develop the same understanding 
of the categories based on theory and empirical evidence. 
Ideally, the final coding scheme is precise enough to document this shared 
understanding and to guide users to the same interpretation of data and the same 
assignments of data to categories. 

In contrast, the phase of applying an existing coding scheme by new raters does 
not aim to incorporate the interpretation of data and categories of the new raters. 
The aim is to train new raters to achieve the understanding of data and categories 
documented in the scheme. The existing coding scheme represents an already discussed and validated 
understanding, which new raters have to acquire in order to apply the coding scheme 
in the same way as in its development study or any other preliminary study.
As a consequence, reliability estimation has to consider the existence of a predefined understanding. 

# 2 Estimating the quality of ratings for a new rater
Within the framework of the *Iota Concept*, this is realized with the function 
`check_new_rater()`. In order to estimate how well a new rater has developed the 
understanding documented in a coding scheme, data and material from existing sources 
must be used. The new rater assigns the material to categories and the assignments 
are compared to the existing assignments of the material. 
Based on this data, the *Assignment Error Matrix* for the new rater can be calculated.

To illustrate this process, we continue the example from the first [vignette](iotarelr.html).
First, we simulate an "old" study by calculating Iota2 for the example. Second,
we calculate the most likely true category for each coding unit. These assignments
serve us as "true" categories. In practice, these steps do not have to be performed in every
case. If an old source provides the final assignments of coding units, these values
can be used as "true" values. 
```{r setup}
library(iotarelr)
res_iota2<-compute_iota2(data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")],
                         random_starts = 10,
                         trace = FALSE)
expected_categories<-est_expected_categories(
  data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")],
  aem=res_iota2$categorical_level$raw_estimates$assignment_error_matrix)
true_values<-expected_categories$expected_category
```
Third, a new rater has to rate the same material as the old source. That is, 
the new rater has to rate the same exams with the same coding scheme. Their results
are saved in the vector `iotarelr_new_rater`. Now, we can request the reliability
estimate for this rater by requesting the function `check_new_rater()`.
```{r}
res_new_rater<-check_new_rater(
  true_values = true_values,
  assigned_values = iotarelr_new_rater)
```
The resulting object is of the class `iotarelr_iota2`. Thus, we can use the function
`get_summary()` to look into the results.
```{r}
get_summary(res_new_rater)
```
Focusing on the *Assignment Error Matrix*, the values on the diagonal are very low.
This leads to low vales for the *Alpha Reliability*, which is about 38.04% for
average exams, about 41.67% for good exams and about 50.92% for poor exams. Thus,
the chance is very high for this new rater to assign the false categories. Please
note that the interpretation of these values is now slightly different from the 
development phase. Here, the high values indicate that the new rater has a high chance
to assign different categories to a coding unit as they were assigned in the old study.

This is also documented in the values for *Iota*, which range between .2078 for
average exams to .3692 for the poor exams. The *Dynamic Iota Index* is quite low
with .213 Finally, we can plot the results for the new rater.
```{r, fig.height = 3, fig.width = 7.2, fig.align = "center", fig.cap="Figure 1: Plot of Iota for a New Rater"}
plot_iota(res_new_rater)

```
The plot of *Iota* for the new rater emphasizes that he is able to recover only a small 
number of true categories for all kinds of exams. Additionally we can also plot
the corresponding coding stream of this new rater.

```{r, fig.height = 5, fig.width = 7.2, fig.align = "center", fig.cap="Figure 2: Stream Plot"}
plot_alluvial_new_rater<-plot_iota2_alluvial(res_new_rater)
plot(plot_alluvial_new_rater)
```
The boxes on the left side represent how the true categories are distributed in 
the population. Their height directly reflects the true categorical sizes. 
The boxes on the right side show the amounts in the labeled data.
The curves between the two pairs of boxes illustrate the coding process. 

**Average exams:** Focusing on the true average exams, the plot illustrates that approximately one third of truly average exams are labeled average. This is represented by the big red stream going from true average to labeled as average. The rest of the exams that are truly average is falsely labeled as either poor or good exams. This is shown by the curves going from true average to the respectively labeled category (good or poor). Most of the exams that really are average are not identified as that. 

In this alluvial plot, the box for true average exams is slightly smaller than that of the ones labeled as average. That means that the coding scheme tends to overestimate the number of average exams. These false interpretations mostly stem from the true poor category that are labeled as average, shown by the big blue curve going into 'labeled as average'. In contrast, only a small portion of truly good exams are incorrectly labeled as average, demonstrated by the rather small green curve going from true good to labeled as average.

**Good exams:** Looking at the good exams, the minority of truly good exams are labeled correctly. This is shown by the green curve going from left (true good exams) to right (labeled as good). Most of the other truly good exams are labeled as poor, which is quite a big difference. 

The size of the boxes of the "true good" exams and "the labeled as good" exams is very similar, however the latter is almost entirely made up of the wrong exams. This means that the category seems to have the right size, but truly is not represented correctly because a lot of truly poor and some truly average exams are falsely labeled as good. This can be seen by the bigger blue and smaller red curve going into the box on the right side. In this sense, the coding scheme estimates the number of good exams almost correctly with a slight tendency to overestimate. 

**Poor exams:** Regarding the poor exams, the labeled poor exams box is a little bit smaller than the one for the true poor exams. The composition of these boxes is very different, however. About half of the truly poor exams are labeled correctly, represented by the big blue bar going from left to right. The remainder of the truly poor exams is either labeled as good or average. Most falsely labeled but truly poor exams are labeled average (the bigger blue curve) and some are labeled good (smaller blue curve). Regarding the "labeled as poor" exams on the right, the box is made up of truly poor exams to approximately 50%, with the remainder coming from truly good (the green curve) or average (the red curve) exams. This means that only half of the exams that are truly poor are also recognized as poor. 

The box on the left (true poor) and on the right (labeled as poor) have a similar size, although the right one is slightly smaller. This means that the total number of poor exams is estimated quite correct with a slight tendency to underestimate.

# 3 Conclusion
The function `check_new_rater()` provides detailed insights into how a new rater
has acquired their understanding of an existing coding scheme. This information
can be used to individually guide the training process of new raters, since 
the *Assignment Error Matrix* and *Iota Reliability* provide hints where errors occur and
where everything is fine. This can increase the quality of codings and saves both costs
and time, making the replication and comparison of studies using content analysis
easier. 

# References
- Früh, W. (2017). Inhaltsanalyse: Theorie und Praxis (9., überarbeitete Auflage). UTB.
- Krippendorff, K. (2019). Content Analysis: An Introduction to Its Methodology 
(4th Ed.). SAGE. 
- Kuckartz, U. (2018). Qualitative Inhaltsanalyse: Methoden, Praxis, 
Computerunterstützung (4. Auflage). Grundlagentexte Methoden. Beltz.
- Mayring, P. (2015). Qualitative Inhaltsanalyse: Grundlagen und Techniken 
(12., überarbeitete Auflage). Beltz.
- Schreier, M. (2012). Qualitative Content Analysis in Practice. SAGE. 

---
title: "3) Different Guidance Functioning"
author: "Florian Berding and Julia Pargmann"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{03_dgf}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  markdown: 
    wrap: 72
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# 1 Introduction

In this vignette, we would like to show you how the tools of the *Iota
Concept* can be used to analyze the suitability of a coding scheme for
rating different kinds of materials. The analysis is very similar to the
analysis of different item functioning (DIF) from item-response theory.
The focal point here is the question whether or not a coding scheme has
the same degree of reliability for different groups of materials. This
issue is closely connected to the question if a coding scheme reproduces
bias for specific groups.

If a coding scheme is perceived as a measurement instrument, like a test
or questionnaire, it must be reliable and valid. Reliability and
validity demand the absence of different guidance functioning (DGF), since
the existence of DGF means that the data is influenced by other sources, as
for example the phenomenon of interest.

We use the term "different guidance functioning" in order to eliminate
this kind of analysis from DIF. Furthermore, we would like to express
that a coding scheme usually only
provides a guideline for raters in assessing a phenomenon.

The analysis of DIF is important for a broad range of applications. For
example,

-   ensuring that ratings of newspapers do not prefer or discourage
    specific political parties.
-   ensuring that books in different languages are rated without the
    influence of the different languages.
-   ensuring that social media contributions are rated similarly without
    bias for the platform-specific form of contribution.
-   ensuring that material created by different groups of learners is
    assessed without bias for their socioeconomic background.
-   ...

In the following, we would like to illustrate this kind of analysis by
continuing the example from the first [vignette](iotarelr.html). In
order to conduct an analysis of DGF, we need additional data that
contains information on how the different coding units are grouped. Let
us have a look at the sample data set.

# 2 Example Analysis
## 2.1 Basic Analysis

```{r}
library(iotarelr)
head(iotarelr_written_exams)
```

The data set contains the ratings of exams from three raters and an
additional column called "Sex". This column stores the information if
the participant of the exam identifies as male or a female. Since an
exam should only measure participants' performance and should be fair,
the *Assignment Error Matrix* in both groups should not differ. That is,
men and women should have the same chance for a good, an average or a
poor exam.

The corresponding analysis can be requested via the function
`check_dgf()`. This function is similar to `compute_iota2()` but has an
additional argument, `splitcr`. This argument uses the group information
for the analysis while `data` only uses the ratings. Please note that
calling this function may take some time.

```{r}
dgf_exam<-check_dgf(data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")],
                    splitcr = iotarelr_written_exams$Sex,
                    random_starts = 300)
```

The results are stored in the object `dgf_exam`. For every group
specified in the column "Sex", a model of Iota2 is fitted. The results
can be accessed in the same way as with objects created with
`compute_iota2()`. The only thing to do is to request the function
`get_summary()`with the results for the specific group. Let us start with
the females.

```{r}
get_summary(dgf_exam$group_female)
```

The summary for the females shows the basic information of the fitted
model. The best log-likelihood has been replicated. Thus, we can be
confident that our model represents the best model for the females.

Special attention should be paid to the *Assignment Error Matrix*, since
this matrix describes how the true categories are assigned. For the
females, an average exam is recognized as indeed an average exam in about
88.7% of cases. A good exam is recognized as  good exam in about 50% of
cases. The remaining 50% of cases are assigned to be an average exam or a
a poor exam with the same probability of 25%. A truly poor exam is
assigned to be a poor exam in about 50% of cases and an average exam in nearly all other cases. The error to assign a poor exam to be a good exam
is only about 5%. Let us now have a look at the results for the men.

```{r}
get_summary(dgf_exam$group_male)
```

Before we look on the results please note that in some situations it is very
difficult to obtain the best estimates. In this example we use 300 random starts
and the algorithm is still not able to replicate the solution with the best
log-likelihood. Thus, the results for the men can slightly vary for every run.

For the men, the result is very different. According to their
*Assignment Error Matrix*, an average exam is recognized as an average
exam only in about 50% of the cases. If it is not recognized as an average
exam, it has a higher chance to be considered a good exam than a
poor one. Concentrating on the good exams, the *Assignment Error Matrix*
shows the same probability for all categories. This implies that a good
exam is assigned randomly to any of the three categories. Only the poor
exams have a high chance to be assigned to the correct category with
more than 70%.

Comparing the *Assignment Error Matrices*, it becomes clear that the
coding scheme is more reliable for females than for men.

How the different degrees of reliability affect the labeled data can be
explored with the function `plot_iota()`. Here we can directly pass the
object `dgf_exam` to the function.

```{r, fig.height = 6, fig.width = 7.2, fig.align = "center", fig.cap = "Figure 1: Plot of Iota for different Groups"}
plot_iota(dgf_exam,
          ylab = "Groups")
```

Let us start with the average exams of the females. The red rectangle
for this category implies that the data labeled as "average" contains a
lot of exams which truly are either good or poor exams (*Iota Error
II*). The red rectangle is even bigger than the green (*Iota*) and
orange ones (*Iota Error I*), meaning that the number of averagely good
exams is overestimated for the females. Concentrating on the males, the
red rectangle (*Iota Error II*) is quite small but the orange one (*Iota
Error I*) is quite big. Thus, the number of averagely good exams is
underestimated.

Regarding the good exams of the females, the red rectangle is quite
small (*Iota Error II*). This means that there is only a small number of
average or poor exams that are labeled as good. However, the orange
rectangle is quite big (*Iota Error I*). It nearly has the same size as
the green rectangle (*Iota*). Thus, the data for the good exams misses
about half of the corresponding exams. This means that the number of
good exams is underestimated in the data set for the females.
Concentrating on the males, *Iota* is only about .18, which is low.
Thus, the good exams are not reliably represented in the individual
data. In contrast, about 36% of the labeled data are missing good exams
(*Iota Error I*). Instead, the data set is made up of about 46% of exams
from other categories (*Iota Error II*). That is, from truly average or
truly poor exams. These exams compensate the missing good exams. As a
consequence, in the *average of all male participants*, the number of
good exams is actually quite correct for the males. On the *level of
single males*, the data is not reliable.

Finally, both diagrams for the poor exams look similar for both men and
women. For the women, the number of missing poor exams is higher than
for the men (*Iota Error I*). In contrast, the data labeled as poor
exams contains more exams which truly belong to the category "good" or
"average" for the men as well as for the females (*Iota Error II*). Thus, in
*average over all female participants*, the number of poor exams is
quite correct as well. For the males, the number of poor exams is
overestimated. For men as well as for women, the data is not reliable on
the individual level.

## 2.2 Coding Stream Analysis
Since version 0.1.3, *iotarelr* provides an additional analysis of the 
coding stream that is able to make the interpretation of the data more
convenient. This kind of analysis can be request via the function `plot_iota2_alluvial()`. 
Please note that the estimates cannot be passed
directly to the function. Only the results for a single group are compatible.
Applied to the current example, Figure 2 shows the coding stream analysis for the
females. The interpretation of the plot is explained in the vignette 
[Get started](iotarelr.html).

```{r, fig.height = 5, fig.width = 7.2, fig.align = "center", fig.cap="Figure 2: Stream Plot for Females"}
plot_alluvial_females<-plot_iota2_alluvial(
  dgf_exam$group_female,
  label_titel = "Coding Stream from True to Assigned Categories (Females)")
plot(plot_alluvial_females)
```
Comparing the boxes for the true and the labeled data, it becomes clear that 
the number of average exams is overestimated while the number of good exams
is underestimated. The sizes of the boxes for the true and assigned poor exams
are similar. That is, on average, the number of poor exams is represented in the
labeled date quite correctly.

The stream plot illustrates that the exams labeled as good exams are made up by exams which are in truth good exams to a very high extent. This can be seen by the
big green curve and the blue and red curves, which in comparison are small. The exams labeled as poor exams are made up of poor and good exams to about 50%. The data labeled as average exams is made up by exams which are in truth good,
average and poor to about one third.

```{r, fig.height = 5, fig.width = 7.2, fig.align = "center", fig.cap="Figure 3: Stream Plot for Males"}
plot_alluvial_males<-plot_iota2_alluvial(
  dgf_exam$group_male,
  label_titel = "Coding Stream from True to Assigned Categories (Males)")
plot(plot_alluvial_males)
```

Turning to the males, the stream plot is very different. Comparing the boxes for
the true and assigned categories, it becomes clear that the number of average 
exams is underestimated while the poor exams are overestimated. Furthermore, 
the coding scheme leads to a small overestimation of the good exams. Thus,
the coding stream plot reflects the results from section 2.1, indicating that
the coding scheme tends to divide males' exams into good or poor exams with no
average in between, compared to an error-free measurement. 

# 3 Summary

To sum up, the coding scheme treats the exams of males and females
differently. The number of averagely good exams is overestimated for
females and underestimated for men. The number of good exams is
underestimated for females but quite accurate for men, although the
number has its source in the wrong exam categories. Finally, the number
of poor exams is quite correct for females, although it sources from
quite a lot of wrong exams, while the number of poor exams for the men
tends to be overestimated. Thus, in descriptive statistics, females
appear to have lower performance than they truly have. For men, the data
shows the tendency that they show either a good or a poor performance
but nothing in between.

This example shows how the guidance of the coding scheme can be analyzed
for different groups of materials. This information can help in
developing coding schemes that are free from bias, providing a more
reliable and valid source of data for subsequent analysis and
conclusions.

---
title: "4) Error Correction"
author: "Florian Berding and Julia Pargmann"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{04_error_correction}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```
# 1 Introduction
Literature regarding content analysis often presents the estimation of reliability  as
a part of the development phase of a coding scheme, for example to inform a revision (Krippendorff, 2019; 
Kuckartz, 2018; Mayring, 2015; Schreier, 2012). If the reliability is 
considered to be sufficient, the main study starts. Often, the reliability of the codings
of this main study is not further controlled, as it is assumed that the reliability estimates
of the development phase hold for the entire main study. Sometimes however, researchers
communicate and discuss their findings and assign a category to a coding unit by 
agreeing on the relevant category when the coding scheme is unclear.

With *Iota Concept*, the reliability of a coding scheme can be taken 
into account more explicitly during the main study. It provides the opportunity 
for error corrections, which is not possible with traditional measures 
such as *Percentage Agreement*, *Cohen's Kappa* or *Krippendorff's Alpha*.  

The error correction of the *Iota Concept* is based on two ideas. First, an
*Assignment Error Matrix* produces patterns of ratings for every coding unit.
These patterns give hints which true category may be the source for this 
kind of observation. This requires at least two raters. Second, involving additional
raters provides more information on a coding unit, which can improve the assignments.
This idea is similar to the application of multiple items in questionnaires or tests 
where each item can be considered as a test for the phenomena of interest. Additional
test are used to reduce errors.

In this vignette, we would like to continue our example of the exams from 
the first [vignette](iotarelr.html) and now want to show you how the error 
correction can be applied.

# 2 Using the error correction of the Iota Concept
Applying the error correction of the *Iota Concept* requires that all coding units
of the core study are rated by at least two raters. The error correction can
be requested with the function `est_expected_categories()`. This function 
calculates the probability that a coding unit belongs to a specific true
category under the condition of the observed pattern. To illustrate the error 
correction, a look into the data set is helpful.

```{r}
library(iotarelr)
head(iotarelr_written_exams)
```
The first 6 rows of the data set show that the three raters do not agree on all
coding units. While the raters agree on the first two exams, they disagree partially
on exams 3 to 6. In particular, two raters consider exam 3 to be average
while one rater considers this exams to be good. Thus, there seems to be some
kind of error and it is not clear which category should be assigned to exam 3.

To solve this problem, we must first estimate the *Assignment Error Matrix*. In 
the next step, we pass the estimated *Assignment Error Matrix* to the function
`est_expected_categories()` and use the ratings as our data source. The results
are saved in the object `expected_categories`.

```{r}
res_iota2<-compute_iota2(
  data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")],
  random_starts = 10,
  trace = FALSE)
expected_categories<-est_expected_categories(
  data=iotarelr_written_exams[c("Coder A","Coder B","Coder C")],
  aem=res_iota2$categorical_level$raw_estimates$assignment_error_matrix)
head(expected_categories)

```
The resulting object contains the ratings and additional columns. The columns
contain the probability that a coding unit belongs to a specific true category.
The most plausible category is always presented in the last column.

For the first row, the probability that this exam is truly an average one is about
48.4%. The chance that this exam represents truly a good exam is about 36.3% and
that is is truly a poor exam is about 15.2%. Thus, it is
most plausible to assign exam number 1 to the category "average".

For exam number two, the probability that this exam is truly an average one is about 
0.00%. The chance that this exam represents truly a good exam is about 27.6% and
that is is truly a poor exam is about 72.4%. Thus, it is
most plausible to assign exam number 2 to the category "poor". 
 
If the ratings were done by only one rater, these kind of errors would not become
visible. For example, if the exams were rated only by rater A, exam 2 would have been assigned as an average exam, although it is more plausible to assign it to the category "poor". 

# 3 Conclusions
Estimating the category that is most likely true has several advantages:

- It allows multiple testing of coding units by more than two raters. 
- The additional information can be used to calculate the certainty of an
assignment.
- The category that is most likely true is assigned to a coding unit.
- Errors made by only one rater can be avoided.
- Researchers do not need to discuss every coding unit, which 
binds financial and time resources for large sample sizes.

# References
- Früh, W. (2017). Inhaltsanalyse: Theorie und Praxis (9., überarbeitete Auflage). UTB.
- Krippendorff, K. (2019). Content Analysis: An Introduction to Its Methodology 
(4th Ed.). SAGE. 
- Kuckartz, U. (2018). Qualitative Inhaltsanalyse: Methoden, Praxis, 
Computerunterstützung (4. Auflage). Grundlagentexte Methoden. Beltz.
- Mayring, P. (2015). Qualitative Inhaltsanalyse: Grundlagen und Techniken 
(12., überarbeitete Auflage). Beltz.
- Schreier, M. (2012). Qualitative Content Analysis in Practice. SAGE. 

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{check_conformity_c}
\alias{check_conformity_c}
\title{Check assumptions of weak superiority}
\usage{
check_conformity_c(aem)
}
\arguments{
\item{aem}{matrix of probabilities}
}
\value{
Returns the number of violations of the assumption of weak superiority.
0 if the assumptions are fulfilled.
}
\description{
This function tests if the probabilities within the Assignment Error Matrix
are in line with the assumption of weak superiority.
}
\references{
Berding, Florian, and Pargmann, Julia (2022).Iota Reliability Concept
of the Second Generation.Measures for Content Analysis Done by
Humans or Artificial Intelligences. Berlin: Logos.
https://doi.org/10.30819/5581
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{grad_ll}
\alias{grad_ll}
\title{Gradient for Log Likelihood in Condition Stage}
\usage{
grad_ll(param_values, observations)
}
\arguments{
\item{param_values}{\code{NumericVector} containing the probabilities of
a multinominal distribution. The length of this factor is the number of
categories - 1 since it contains only the parameters to be estimated.}

\item{observations}{\code{NumericVector} containing the number of
observations for each category of the multinominal distribution. The length
of this vector equals the number of categories.}
}
\value{
Returns the gradient as a \code{NumericVector}.
}
\description{
Function written in \code{C++} estimating the gradient of the log likelihood
function for a given parameter set and given observations.
}
\references{
Berding, Florian, and Pargmann, Julia (2022).Iota Reliability Concept
of the Second Generation.Measures for Content Analysis Done by
Humans or Artificial Intelligences. Berlin: Logos.
https://doi.org/10.30819/5581
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/additional_functions.R
\name{get_consequences}
\alias{get_consequences}
\title{Get Consequences}
\usage{
get_consequences(
  measure_typ = "dynamic_iota_index",
  measure_1_val,
  measure_2_val = NULL,
  level = 0.95,
  strength = NULL,
  data_type,
  sample_size
)
}
\arguments{
\item{measure_typ}{Type of measure used for estimation. Set "iota_index" for
the original Iota Index, "static_iota_index" for the static transformation
of the Iota Index with d=4 or "dynamic_iota_index" for the dynamic transformation
of the Iota Index with d=2.}

\item{measure_1_val}{Reliability value for the independent variable.}

\item{measure_2_val}{Reliability value for the dependent variable. If not
set, the function uses the same value as for the independent variable.}

\item{level}{Level of certainty for calculating the prediction intervals.}

\item{strength}{True strength of the relationship between the independent and
dependent variable. Possible values are "no", "weak", "medium" and "strong". If
no value is supplied, a strong relationship is assumed for deviation and a weak
relationship for all others. They represent the most demanding situations for the
reliability.}

\item{data_type}{Type of data. Possible values are "nominal" or "ordinal".}

\item{sample_size}{Size of the sample in the study.}
}
\value{
Returns a \code{data.frame} which contains the prediction intervals
for the deviation between  true and estimated sample association/correlation,
risk of Type I errors and chance to correctly classify the effect size.
Additionally, the probability is estimated so that the statistics of the sample
deviate from an error free sample with no or only a weak effect .
}
\description{
Function estimating the consequences of reliability for subsequent analysis.
}
\note{
The classification of effect sizes uses the work of Cohen (1988),
who differentiates effect sizes by their relevance for practice.

For nominal data, all statistics refer to Cramer's V. For ordinal data, all
statistics refer to Kendall's Tau.

The models for calculating the consequences are taken from Berding and
Pargmann (2022).
}
\references{
Cohen, J. (1988). Statistical Power Analysis for the Behavioral
Sciences (2nd Ed.). Taylor & Francis.

Berding, Florian, and Pargmann, Julia (2022).Iota Reliability Concept
of the Second Generation.Measures for Content Analysis Done by
Humans or Artificial Intelligences. Berlin:Logos.
https://doi.org/10.30819/5581
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/additional_functions.R
\name{check_new_rater}
\alias{check_new_rater}
\title{Check new rater}
\usage{
check_new_rater(
  true_values,
  assigned_values,
  con_step_size = 1e-04,
  con_random_starts = 5,
  con_max_iterations = 5000,
  con_rel_convergence = 1e-12,
  con_trace = FALSE,
  fast = TRUE,
  free_aem = FALSE
)
}
\arguments{
\item{true_values}{\code{Vector} containing the true categories of the coding
units. Vector must have the same length as \code{assigned_values}.}

\item{assigned_values}{\code{Vector} containing the assigned
categories of the coding units. Missing values are currently not supported and
have to be omitted from the vector. Vector must have the same length as
\code{true_values}.}

\item{con_step_size}{\code{Double} for specifying the size for increasing or
decreasing the probabilities during the conditioning stage of estimation.
This value should not be less than 1e-3.}

\item{con_random_starts}{\code{Integer} for the number of random starts
within the condition stage.}

\item{con_max_iterations}{\code{Integer} for the maximum number of iterations
during the conditioning stage.}

\item{con_rel_convergence}{\code{Double} for determining the convergence
criterion during the conditioning stage. The algorithm stops if the relative change
is smaller than this criterion.}

\item{con_trace}{\code{TRUE} for printing progress information on the console
during estimations in the conditioning stage. \code{FALSE} if you do not want to have
this information printed.}

\item{fast}{\code{Bool} If \code{TRUE} a fast estimation is applied during the
condition stage. This option ignores all parameters beginning with "con_".
If \code{FALSE} the estimation described in Berding and
Pargmann (2022) is used. Default is \code{TRUE}.}

\item{free_aem}{\code{Bool} If \code{TRUE} the Assignment Error Matrix is
estimated in a way ensuring conformity with the assumption of weak superiority.
if \code{FALSE} the Assignment Error Matrix is freely estimated. \code{TRUE}
is default.}
}
\value{
Returns a \code{list} with the following three components:
The first component \code{estimates_categorical_level} comprises all
elements that describe the ratings on a categorical level. The elements are
sub-divided into raw estimates and chance-corrected estimates.

\code{raw_estimates}
\describe{
\item{\code{alpha_reliability:}}{A vector containing the Alpha
Reliabilities for each category. These values represent probabilities.}
\item{\code{beta_reliability:}}{A vector containing the Beta Reliabilities for each
category. These values represent probabilities.}
\item{\code{assignment_error_matrix:}}{An Assignment Error Matrix containing the conditional
probabilities for assigning a unit of category i to categories 1 to n.}
\item{\code{iota:}}{A vector containing the Iota values for each category.}
}
\code{elements_chance_corrected}
\describe{
\item{\code{alpha_reliability:}}{A vector containing the chance-corrected Alpha Reliabilities for each category.}
\item{\code{beta_reliability:}}{A vector containing the chance-corrected Beta Reliabilities for each category.}
}

The second component \code{estimates_scale_level} contains elements to
describe the quality of the ratings on a scale level. It contains the
following elements:
\describe{
\item{\code{iota_index:}}{The Iota Index representing the reliability on a scale level.}
\item{\code{iota_index_d4:}}{The Static Iota Index, which is a transformation of the original Iota Index,
in order to consider the uncertainty of estimation.}
\item{\code{iota_index_dyn2:}}{The Dynamic Iota Index, which is a transformation of the original Iota Index,
in order to consider the uncertainty of estimation.}
}
The third component \code{information} contains important information
regarding the parameter estimation. It comprises the following elements:
\describe{
\item{\code{log_likelihood:}}{Log-likelihood of the best solution.}
\item{\code{convergence:}}{If estimation converged 0, otherwise 1.}
\item{\code{est_true_cat_sizes:}}{Estimated categorical sizes. This is the estimated amount of the categories.}
\item{\code{conformity:}}{\code{0} if the solution is in line with assumptions of weak superiority.
 A number greater 0 indicates the number of violations of the assumption
 of weak superiority.}
 \item{\code{random_starts:}}{Numer of random starts for the EM algorithm.}
\item{\code{boundaries:}}{\code{False} if the best solution does not contain boundary values.
\code{True} if the best solution does contain boundary values}
\item{\code{p_boundaries:}}{Percentage of solutions with boundary values during estimation.}
\item{\code{call:}}{Name of the function that created the object.}
\item{\code{n_rater:}}{Number of raters.}
\item{\code{n_cunits:}}{Number of coding units.}
}
}
\description{
Function for estimating the reliability of codings for a new rater based on
Iota 2
}
\note{
The returned object contains further slots since the returned object is
of class \code{iotarelr_iota2}. These slots are empty because they are not part of the
estimation within this function.

Please do not use the measures on the scale level if the Assignment Error Matrix
was freely estimated since this kind of matrix is not conceptualized for comparing
the coding process with random guessing.
}
\references{
Florian Berding and Julia Pargmann (2022). Iota Reliability Concept
of the Second Generation. Measures for Content Analysis Done by
Humans or Artificial Intelligences. Berlin:Logos.
https://doi.org/10.30819/5581
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{get_random_start_values_p}
\alias{get_random_start_values_p}
\title{Generating randomly chosen probabilities for Assignment Error Matrix}
\usage{
get_random_start_values_p(n_categories)
}
\arguments{
\item{n_categories}{Integer for the number of categories in the data. Must be
at least 2.}
}
\value{
Returns a matrix for Assignment Error Matrix (AEM) with randomly
generated probabilities. The generated probabilities are in line with
the assumption of weak superiority.
}
\description{
Function written in \code{C++} for generating a set of randomly chosen
probabilities for the Assignment Error Matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/iota_version_2.R
\name{get_patterns}
\alias{get_patterns}
\title{Get patterns}
\usage{
get_patterns(data, categorical_levels)
}
\arguments{
\item{data}{\code{Matrix} or \code{data.frame} containing the ratings. The
cases are in the rows and the raters are in the columns. Characters in the cells
are supported. At least two raters are necessary.}

\item{categorical_levels}{\code{Vector} containing all possible categories of
the content analysis.}
}
\value{
Function returns a \code{list} with the following components:
\item{n}{Integer representing the number of different patterns in the data.}
\item{shape}{\code{Matrix} containing all unique patterns in in the data.
Cells of the matrix are characters.}
\item{frq}{\code{Vector} containing the frequencies of the patterns.}
\item{count}{\code{Matrix} containing the relative frequencies of the
categories within each pattern. The number of rows equals the number of
patterns. The number of columns equals the number of categories.}
}
\description{
Auxiliary function written in \code{R} for providing the necessary information
about the patterns generated by raters. This function produces the
input for the EM-algorithm.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{get_random_start_values_class_sizes}
\alias{get_random_start_values_class_sizes}
\title{Generating randomly chosen probabilities for categorical sizes}
\usage{
get_random_start_values_class_sizes(n_categories)
}
\arguments{
\item{n_categories}{Integer for the number of categories in the data. Must be
at least 2.}
}
\value{
Returns a vector of randomly chosen categorical sizes.
}
\description{
Function written in \code{C++} for generating a set of randomly chosen
probabilities describing the size of the different classes. The
probabilities describe the relative frequencies of the categories in the data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/writtenexams.R
\docType{data}
\name{iotarelr_written_exams}
\alias{iotarelr_written_exams}
\title{Example Data Set}
\format{
A data frame with 318 rows and 4 variables:
\describe{
  \item{Coder A}{Ratings of coder A.}
  \item{Coder B}{Ratings of coder B.}
  \item{Coder C}{Ratings of coder C.}
  \item{Sex}{Referring to the biological aspects of an individual.}
}
}
\usage{
iotarelr_written_exams
}
\description{
A data set containing the ratings of three coders for written exams.
It also contains the gender of the people who took the exam. The data
is not real and is only created for illustrating purposes.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/additional_functions.R
\name{est_expected_categories}
\alias{est_expected_categories}
\title{Estimate Expected Categories}
\usage{
est_expected_categories(data, aem)
}
\arguments{
\item{data}{\code{Matrix} which contains the codings for every coding unit. The
coding units must be in the rows and the raters must be in the columns. At
least two raters are necessary.}

\item{aem}{Assignment Error Matrix based on the second generation of the Iota Concept (Iota2).}
}
\value{
Returns a \code{matrix} with the original data, the conditioned
probability of each true category, and the expected category for every coding unit.
}
\description{
Function for estimating the expected category of coding units.
}
\references{
Florian Berding and Julia Pargmann (2022).Iota Reliability Concept
of the Second Generation. Measures for Content Analysis Done by
Humans or Artificial Intelligences. Berlin:Logos.
https://doi.org/10.30819/5581
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{est_con_multinominal_c}
\alias{est_con_multinominal_c}
\title{Estimating log likelihood in Condition Stage}
\usage{
est_con_multinominal_c(
  observations,
  anchor,
  max_iter = 500000L,
  step_size = 1e-04,
  cr_rel_change = 1e-12,
  n_random_starts = 10L,
  fast = TRUE,
  trace = FALSE
)
}
\arguments{
\item{observations}{\code{NumericVector} containing the frequency of the
categories.}

\item{anchor}{\code{Integer} ranging between 1 and the number of categories.
Anchor defines the reference category. That is the category with the highest
probability according to the assumption of weak superiority.}

\item{max_iter}{\code{Integer} specifying the maximal number of iterations
for each random start.}

\item{step_size}{\code{Double} for specifying the size for increasing or
decreasing the probabilities during the estimation. This value should not
be less than 1e-3.}

\item{cr_rel_change}{\code{Double} for defining when the estimation should
stop. That is, if the change in log-likelihood is smaller as this value the
estimation stops.}

\item{n_random_starts}{\code{Integer} for the number of random start.}

\item{fast}{\code{Bool} If \code{TRUE} a fast estimation is applied. This
option ignored all other parameters. If
\code{FALSE} the estimation described in Berding and Pargmann (2022) is used.
Default is \code{TRUE}.}

\item{trace}{\code{Bool} \code{TRUE} if information about the progress of
estimation should be printed to the console. \code{FALSE} if not desired.}
}
\value{
Returns the log likelihood as a single numeric value.
}
\description{
Function written in \code{C++} estimating the log likelihood of a given
parameter set during the condition stage.
}
\references{
Berding, Florian, and Pargmann, Julia (2022).Iota Reliability Concept
of the Second Generation.Measures for Content Analysis Done by
Humans or Artificial Intelligences. Berlin: Logos.
https://doi.org/10.30819/5581
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot_iota2_alluvial}
\alias{plot_iota2_alluvial}
\title{Plot of the Coding Stream}
\usage{
plot_iota2_alluvial(
  object,
  label_titel = "Coding Stream from True to Assigned Categories",
  label_prefix_true = "true",
  label_prefix_assigned = "labeled as",
  label_legend_title = "True Categories",
  label_true_category = "True Category",
  label_assigned_category = "Assigned Category",
  label_y_axis = "Relative Frequencies",
  label_categories_size = 3,
  key_size = 0.5,
  text_size = 10,
  legend_position = "right",
  legend_direction = "vertical"
)
}
\arguments{
\item{object}{Estimates of Iota 2 created with \code{compute_iota2()},
\code{check_new_rater()} or with \code{check_dgf()}. Please note that the object
created by \code{check_dgf()} cannot be passed directly. Only the elements of
the corresponding list are compatible.}

\item{label_titel}{\code{Character} containing the title of the plot.}

\item{label_prefix_true}{\code{Character} representing the prefix for tagging
the true categories. Character is applied to every category.}

\item{label_prefix_assigned}{\code{Character} representing the prefix for tagging
the assigned categories. Character is applied to every category.}

\item{label_legend_title}{\code{Character} containing the title of the legend.}

\item{label_true_category}{\code{Character} describing the stratum of true
categories.}

\item{label_assigned_category}{\code{Character} describing the stratum of
assigned categories.}

\item{label_y_axis}{\code{Character}. Label of the y-axis.}

\item{label_categories_size}{\code{double} determining the size of the
label for each true and assigned category within the plot.}

\item{key_size}{\code{double} determining the size of the legend.}

\item{text_size}{\code{double} determining the size of the text within the legend.}

\item{legend_position}{\code{string} Position of the legend. Possible values
are "bottom","right","left", "top" and "none".}

\item{legend_direction}{\code{string} Layout of the items in the legend. Possible
values are "horizontal" and "vertical".}
}
\value{
Returns an object of class \code{gg} and \code{ggplot} which can be
shown with \code{plot()}.
}
\description{
Function for creating an alluvial plot that can be plotted via 'ggplot2'.
}
\note{
An example for interpreting the plot can be found in the vignette
\href{../doc/iotarelr.html}{Get started} or via
\code{vignette("iotarelr", package = "iotarelr")}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/iota_version_1.R
\name{compute_iota1}
\alias{compute_iota1}
\title{Computes Iota and its elements in version 1}
\usage{
compute_iota1(data)
}
\arguments{
\item{data}{Data for which the elements should be estimated. Data must be
an object of type \code{data.frame} or \code{matrix} with  cases in the rows and
raters in the columns.}
}
\value{
A list with the following components
\item{alpha}{A vector containing the chance-corrected Alpha Reliabilities for
every category.}
\item{beta}{A vector containing the chance-corrected Beta Reliabilities for
every category.}
\item{iota}{A vector containing the Iota values for every category.}
\item{assignment_error_matrix}{A matrix with the conditional probabilities
for every category. The rows refer to the true categories and the columns
refer to the assigned categories. The elements on the diagonal represent the
alpha errors of that category. The other elements in each row represent the
conditioned probabilities that a coding unit is wrongly assigned to another
category.}
\item{average_iota}{A numeric value ranging between 0 and 1, representing the
Average Iota values on a categorical level. It describes the reliability of
the whole scale.}
}
\description{
Computes all elements of the Iota Reliability Concept
}
\references{
-   Berding, Florian, Elisabeth Riebenbauer, Simone Stuetz, Heike
Jahncke, Andreas Slopinski, and Karin Rebmann. 2022. Performance and
Configuration of Artificial Intelligence in Educational
Settings.Introducing a New Reliability Concept Based on Content
Analysis. Frontiers in Education.
https://doi.org/10.3389/feduc.2022.818365
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot_iota}
\alias{plot_iota}
\title{Plot Iota2}
\usage{
plot_iota(
  object,
  xlab = "Amount on all cases",
  ylab = "Categories",
  liota = "Assignment of the true category (Iota)",
  lcase2 = "Assignment to the false category",
  lcase3 = "Assignment from the false true category",
  lscale_quality = "Scale Quality",
  lscale_cat = c("insufficent", "minimum", "satisfactory", "good", "excellent"),
  number_size = 6,
  key_size = 0.5,
  text_size = 10,
  legend_position = "bottom",
  legend_direction = "vertical",
  scale = "none"
)
}
\arguments{
\item{object}{Estimates of Iota 2 created with \code{compute_iota2()},
\code{check_dgf()} or \code{check_new_rater()}.}

\item{xlab}{\code{Character} passed to xlab() from scale_fill_manual(). Label
of the x-axis.}

\item{ylab}{\code{Character} passed to ylab() from scale_fill_manual(). Label
of the y-axis.}

\item{liota}{\code{Character} passed to labels() from scale_fill_manual().
Label for Iota.Amount of cases that are assigned to the correct category.}

\item{lcase2}{\code{Character} passed to labels() from scale_fill_manual().
Label for the amount of cases that are assigned to a false category.}

\item{lcase3}{\code{Character} passed to labels() from scale_fill_manual().
Label for the amount of cases that are assigned from a false category.}

\item{lscale_quality}{\code{character} passed to scale_fill_manual() determining
the title for the quality of a scale. Only used in conjunction with
\code{scale}.}

\item{lscale_cat}{Vector of strings with length 5. This vector contains the
labels for each category of quality for the scale.}

\item{number_size}{\code{Double} passed to geom_text() determining the size
of the numbers within the plot.}

\item{key_size}{\code{Double} passed to theme() determining the size of the
legend keys.}

\item{text_size}{\code{Double} passed to theme() determining the size of the
text within the legend.}

\item{legend_position}{\code{string} Position of the legend. Possible values
are "bottom","right","left", "top" and "none".}

\item{legend_direction}{\code{string} Layout of the items in the legend. Possible
values are "horizontal" and "vertical".}

\item{scale}{\code{String} for requesting an additional plot of reliability
on the scale level. If \code{scale="dynamic_iota_index"} Dynamic Iota Index
is used.  If \code{scale="static_iota_index"} Static Iota Index
is used. If \code{scale="none"} no additional plot is created.}
}
\value{
Function returns an object of class \code{gg, ggplot} illustrating how
the data of the different categories influence each other.
}
\description{
Function for creating a plot object that can be plotted via 'ggplot2'.
}
\references{
Florian Berding and Julia Pargmann (2022).Iota Reliability Concept
of the Second Generation. Measures for Content Analysis Done by
Humans or Artificial Intelligences. Berlin: Logos.
https://doi.org/10.30819/5581
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/additional_functions.R
\name{get_summary}
\alias{get_summary}
\title{Get Summary}
\usage{
get_summary(object)
}
\arguments{
\item{object}{An object of class \code{iotarelr_iota2} created by
\code{\link{compute_iota2}}, \code{\link{check_new_rater}}, or
\code{\link{check_dgf}}.}
}
\value{
Prints central statistics of the estimated model.
}
\description{
Function for creating a short summary of the estimated Iota components.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{log_likelihood_multi_c}
\alias{log_likelihood_multi_c}
\title{Estimating log-likelihood in Condition Stage}
\usage{
log_likelihood_multi_c(probabilities, observations)
}
\arguments{
\item{probabilities}{\code{NumericVector} containing the probabilities of
a multinominal distribution. In the context of Iota Reliability this refers
to a specific row of the Assignment Error Matrix.}

\item{observations}{\code{NumericVector} containing the number of
observations for each category of the multinominal distribution.}
}
\value{
Returns the log likelihood as a single numeric value.
}
\description{
Function written in \code{C++} estimating the log likelihood of a given
parameter set during the condition stage.
}
\references{
Berding, Florian, and Pargmann, Julia (2022).Iota Reliability Concept
of the Second Generation.Measures for Content Analysis Done by
Humans or Artificial Intelligences. Berlin: Logos.
https://doi.org/10.30819/5581
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/iota_version_2.R
\name{compute_iota2}
\alias{compute_iota2}
\title{Computes Iota and its elements in version 2}
\usage{
compute_iota2(
  data,
  random_starts = 10,
  max_iterations = 5000,
  cr_rel_change = 1e-12,
  con_step_size = 1e-04,
  con_rel_convergence = 1e-12,
  con_max_iterations = 5000,
  con_random_starts = 5,
  b_min = 0.01,
  fast = TRUE,
  trace = TRUE,
  con_trace = FALSE
)
}
\arguments{
\item{data}{Data for which the elements should be estimated. Data must be
an object of type \code{data.frame} or \code{matrix} with cases in the rows and
raters in the columns.}

\item{random_starts}{An integer for the number of random starts for the
EM algorithm.}

\item{max_iterations}{An integer for the maximum number of iterations within
the EM algorithm.}

\item{cr_rel_change}{Positive numeric value for defining the convergence of the
EM algorithm.}

\item{con_step_size}{\code{Double} for specifying the size for increasing or
decreasing the probabilities during the conditioning stage of estimation.
This value should not be less than 1e-3.}

\item{con_rel_convergence}{\code{Double} for determining the convergence
criterion during the conditioning stage. The algorithm stops if the relative change
is smaller than this criterion.}

\item{con_max_iterations}{\code{Integer} for the maximum number of iterations
during the conditioning stage.}

\item{con_random_starts}{\code{Integer} for the number of random starts
within the conditioning stage.}

\item{b_min}{Value ranging between 0 and 1, determining the minimal size of
the categories for checking if boundary values occurred. The algorithm tries
to select solutions that are not considered to be boundary values.}

\item{fast}{\code{Bool} If \code{TRUE} a fast estimation is applied during the
condition stage. This option ignores all parameters beginning with "con_".
If \code{FALSE} the estimation described in Berding and
Pargmann (2022) is used. Default is \code{TRUE}.}

\item{trace}{\code{TRUE} for printing progress information on the console.
\code{FALSE} if this information is not to be printed.}

\item{con_trace}{\code{TRUE} for printing progress information on the console
during estimations in the conditioning stage. \code{FALSE} if this information
is not to be printed.}
}
\value{
Returns a \code{list} with the following three components:
The first component \code{estimates_categorical_level} comprises all
elements that describe the ratings on a categorical level. The elements are
sub-divided into raw estimates and chance-corrected estimates.
\describe{
\item{\code{raw_estimates}}{
\describe{
\item{\code{alpha_reliability:}}{A vector containing the Alpha
Reliabilities for each category. These values represent probabilities.}
\item{\code{beta_reliability:}}{A vector containing the Beta Reliabilities for each
category. These values represent probabilities.}
\item{\code{assignment_error_matrix:}}{Assignment Error Matrix containing the conditional
probabilities for assigning a unit of category i to categories 1 to n.}
 \item{\code{iota:}}{A vector containing the Iota values for each category.}
 \item{\code{iota_error_1:}}{A vector containing the Iota Error Type I values for each category.}
 \item{\code{iota_error_2:}}{A vector containing the Iota Error Type II values for each category.}
}}

\item{\code{elements_chance_corrected}}{
\describe{
\item{\code{alpha_reliability:}}{A vector containing the chance-corrected Alpha Reliabilities for each category.}
\item{\code{beta_reliability:}}{A vector containing the chance-corrected Beta Reliabilities for each category.}
}}

The second component \code{estimates_scale_level} contains elements for
describing the quality of the ratings on a scale level. It comprises the
following elements:
\describe{
\item{\code{iota_index:}}{The Iota Index, representing the reliability on a scale level.}
\item{\code{iota_index_d4:}}{The Static Iota Index, which is a transformation of the original Iota Index,
in order to consider the uncertainty of estimation.}
\item{\code{iota_index_dyn2:}}{The Dynamic Iota Index, which is a transformation of the original Iota Index,
in order to consider the uncertainty of estimation.}
}

The third component \code{information} contains important information
regarding the parameter estimation. It comprises the following elements:
\describe{
\item{\code{log_likelihood:}}{Log-likelihood of the best solution.}
\item{\code{convergence:}}{If estimation converged 0, otherwise 1.}
\item{\code{est_true_cat_sizes:}}{Estimated categorical sizes. This is the estimated amount of the categories.}
\item{\code{conformity:}}{\code{0} if the solution is in line with assumptions of weak superiority.
 A number greater 0 indicates the number of violations of the assumption
 of weak superiority.}
 \item{\code{random_starts:}}{Numer of random starts for the EM algorithm.}
\item{\code{boundaries:}}{\code{False} if the best solution does not contain boundary values.
\code{True} if the best solution does contain boundary values}
\item{\code{p_boundaries:}}{Percentage of solutions with boundary values during the estimation.}
\item{\code{call:}}{Name of the function that created the object.}
\item{\code{n_rater:}}{Number of raters.}
\item{\code{n_cunits:}}{Number of coding units.}
}}
}
\description{
Fits a model of Iota2 to the data
}
\references{
Florian Berding and Julia Pargmann (2022).Iota Reliability Concept
of the Second Generation. Measures for Content Analysis Done by
Humans or Artificial Intelligences. Berlin: Logos.
https://doi.org/10.30819/5581
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_new_rater.R
\docType{data}
\name{iotarelr_new_rater}
\alias{iotarelr_new_rater}
\title{Sample Vector}
\format{
A vector with the length of 318.
}
\usage{
iotarelr_new_rater
}
\description{
A vector containing the ratings of a new rater.
The data is not real and is only created for illustration purposes.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{fct_log_likelihood_c}
\alias{fct_log_likelihood_c}
\title{Estimating log-likelihood}
\usage{
fct_log_likelihood_c(
  categorial_sizes,
  aem,
  obs_pattern_shape,
  obs_pattern_frq,
  categorical_levels
)
}
\arguments{
\item{categorial_sizes}{\code{Vector} containing the sizes of the different
categories. That is amount of a category on all cases.}

\item{aem}{\code{Matrix} in aem form. This matrix reports the true category
in the rows and the assigned categories in the columns. The cells represent
the probabilities that a coding unit of category i is assigned to category j.}

\item{obs_pattern_shape}{\code{Matrix} containing the unique patterns found
in the data. Ideally this matrix is generated by the function
\code{get_patterns()}.}

\item{obs_pattern_frq}{\code{Vector} containing the frequencies of the
patterns. Ideally it is generated by the the function
\code{get_patterns()}.}

\item{categorical_levels}{\code{Vector} containing all possible categories of
the content analysis.}
}
\value{
Returns the log likelihood as a single numeric value.
}
\description{
Function written in \code{C++} estimating the log likelihood of a given
parameter set.
}
\references{
Berding, Florian, and Pargmann, Julia (2022).Iota Reliability Concept
of the Second Generation.Measures for Content Analysis Done by
Humans or Artificial Intelligences. Berlin: Logos.
https://doi.org/10.30819/5581
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{EM_algo_c}
\alias{EM_algo_c}
\title{Parameter estimation via EM Algorithm with Condition Stage}
\usage{
EM_algo_c(
  obs_pattern_shape,
  obs_pattern_frq,
  obs_internal_count,
  categorical_levels,
  random_starts,
  max_iterations,
  rel_convergence,
  con_step_size,
  con_random_starts,
  con_max_iterations,
  con_rel_convergence,
  fast,
  trace,
  con_trace
)
}
\arguments{
\item{obs_pattern_shape}{\code{Matrix} containing the unique patterns found
in the data. Ideally this matrix is generated by the function
\code{get_patterns()}.}

\item{obs_pattern_frq}{\code{Vector} containing the frequencies of the
patterns. Ideally it is generated by the the function
\code{get_patterns()}.}

\item{obs_internal_count}{\code{Matrix} containing the relative frequencies
of each category within each pattern. Ideally this matrix is generated by
the function \code{get_patterns()}.}

\item{categorical_levels}{\code{Vector} containing all possible categories of
the content analysis.}

\item{random_starts}{\code{Integer} for determining how often the algorithm
should restart with randomly chosen values for the Assignment Error Matrix
and the categorical sizes.}

\item{max_iterations}{\code{Integer} for determining the maximum number of iterations
for each random start.}

\item{rel_convergence}{\code{Double} for determining the convergence criterion. The
algorithm stops if the relative change is smaller than this criterion.}

\item{con_step_size}{\code{Double} for specifying the size for increasing or
decreasing the probabilities during the condition stage of estimation.
This value should not be less than 1e-3.}

\item{con_random_starts}{\code{Integer} for the number of random starts
within the condition stage.}

\item{con_max_iterations}{\code{Integer} for the maximum number of iterations
during the condition stage.}

\item{con_rel_convergence}{\code{Double} for determining the convergence
criterion during condition stage. The algorithm stops if the relative change
is smaller than this criterion.}

\item{fast}{\code{Bool} If \code{TRUE} a fast estimation is applied during the
condition stage. This option ignores all parameters beginning with "con_".
If \code{FALSE} the estimation described in Berding and
Pargmann (2022) is used. Default is \code{TRUE}.}

\item{trace}{\code{TRUE} for printing progress information on the console.
\code{FALSE} if this information should not be printed.}

\item{con_trace}{\code{TRUE} for printing progress information on the console
during estimations in the condition stage. \code{FALSE} if this information
should not be printed.}
}
\value{
Function returns a \code{list} with the estimated parameter sets for
every random start. Every parameter set contains the following components:
\item{log_likelihood}{Log likelihood of the estimated solution.}
\item{aem}{Estimated Assignment Error Matrix (aem). The rows represent the
true categories while the columns stand for the assigned categories. The cells
describe the probability that a coding unit of category i is assigned to
category j.}
\item{categorial_sizes}{\code{Vector} of estimated sizes for each
category.}
\item{convergence}{If the algorithm converged within the iteration limit
\code{TRUE}. \code{FALSE} in every other case.}
\item{iteration}{Number of iterations when the algorithm was terminated.}
}
\description{
Function written in \code{C++} for estimating the parameters of the model
via Expectation Maximization (EM Algorithm).
}
\references{
Berding, Florian, and Pargmann, Julia (2022).Iota Reliability Concept
of the Second Generation.Measures for Content Analysis Done by
Humans or Artificial Intelligences. Berlin: Logos.
https://doi.org/10.30819/5581
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/additional_functions.R
\name{check_dgf}
\alias{check_dgf}
\title{Check for Different Guidance Functioning (DGF)}
\usage{
check_dgf(
  data,
  splitcr,
  random_starts = 300,
  max_iterations = 5000,
  cr_rel_change = 1e-12,
  con_step_size = 1e-04,
  con_random_starts = 10,
  con_max_iterations = 5000,
  con_rel_convergence = 1e-12,
  b_min = 0.01,
  trace = FALSE,
  con_trace = FALSE,
  fast = TRUE
)
}
\arguments{
\item{data}{Data for which the elements should be estimated. Data must be
an object of type \code{data.frame} or \code{matrix} with cases in the rows and
raters in the columns. Please note that no additional variables are allowed
in this object.}

\item{splitcr}{\code{Vector} containing the assignments of coding units to
groups. The vector must have the same length as the number of rows of object
\code{data}.}

\item{random_starts}{An integer for the number of random starts for the
EM algorithm.}

\item{max_iterations}{An integer for the maximum number of iterations within
the EM algorithm.}

\item{cr_rel_change}{Positive numeric value for defining the convergence of the
EM algorithm.}

\item{con_step_size}{\code{Double} for specifying the size for increasing or
decreasing the probabilities during the conditioning stage of estimation.
This value should not be less than 1e-3.}

\item{con_random_starts}{\code{Integer} for the number of random starts
within the condition stage.}

\item{con_max_iterations}{\code{Integer} for the maximum number of iterations
during the condition stage.}

\item{con_rel_convergence}{\code{Double} for determining the convergence
criterion during condition stage. The algorithm stops if the relative change
is smaller than this criterion.}

\item{b_min}{Value ranging between 0 and 1 determining the minimal size of
the categories for checking if boundary values occurred. The algorithm tries
to select solutions that are not considered to be boundary values.}

\item{trace}{\code{TRUE} for printing progress information on the console.
\code{FALSE} if this information is not to be printed.}

\item{con_trace}{\code{TRUE} for printing progress information on the console
during estimations in the condition stage. \code{FALSE} if this information
is not to be printed.}

\item{fast}{\code{Bool} If \code{TRUE} a fast estimation is applied during the
condition stage. This option ignores all parameters beginning with "con_".
If \code{FALSE} the estimation described in Berding and
Pargmann (2022) is used. Default is \code{TRUE}.}
}
\value{
Returns an object of class \code{iotarelr_iota2_dif}. For each group,
the results of the estimation are saved separately. The structure within each
group is similar to the results from \code{compute_iota2()}. Please check
that documentation.
}
\description{
Function for checking if the coding scheme is the same for different sub-groups.
}
\references{
Florian Berding and Julia Pargmann (2022).Iota Reliability Concept
of the Second Generation. Measures for Content Analysis Done by
Humans or Artificial Intelligences. Berlin:Logos.
https://doi.org/10.30819/5581
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/iota_version_2.R
\name{get_iota2_measures}
\alias{get_iota2_measures}
\title{Get Iota2 Measures}
\usage{
get_iota2_measures(aem, categorical_sizes, categorical_levels)
}
\arguments{
\item{aem}{Assignment Error Matrix.}

\item{categorical_sizes}{Probabilities for the different categories to occur.}

\item{categorical_levels}{\code{Vector} containing all possible categories of
the content analysis.}
}
\value{
Returns a \code{list} of all measures belonging to the Iota Concept
of the second generation.
The first component \code{estimates_categorical_level} comprises all
elements that describe the ratings on a categorical level. The elements are
sub-divided into raw estimates and chance-corrected estimates.
\describe{
\item{\code{raw_estimates}}{
\describe{
\item{\code{iota:}}{A vector containing the Iota values for each category.}
 \item{\code{iota_error_1:}}{A vector containing the Iota Error Type I values for each category.}
 \item{\code{iota_error_2:}}{A vector containing the Iota Error Type II values for each category.}
\item{\code{alpha_reliability:}}{A vector containing the Alpha
Reliabilities for each category. These values represent probabilities.}
\item{\code{beta_reliability:}}{A vector containing the Beta Reliabilities for each
category. These values represent probabilities.}
\item{\code{assignment_error_matrix:}}{Assignment Error Matrix containing the conditional
probabilities for assigning a unit of category i to categories 1 to n.}
}}
\item{\code{elements_chance_corrected}}{
\describe{
\item{\code{alpha_reliability:}}{A vector containing the chance-corrected Alpha Reliabilities for each category.}
\item{\code{beta_reliability:}}{A vector containing the chance-corrected Beta Reliabilities for each category.}
}}
}
The second component \code{estimates_scale_level} contains elements for
describing the quality of the ratings on a scale level. It comprises the
following elements:
\describe{
\item{\code{iota_index:}}{The Iota Index, representing the reliability on a scale level.}
\item{\code{iota_index_d4:}}{The Static Iota Index, which is a transformation of the original Iota Index,
in order to consider the uncertainty of estimation.}
\item{\code{iota_index_dyn2:}}{The Dynamic Iota Index, which is a transformation of the original Iota Index,
in order to consider the uncertainty of estimation.}
}
}
\description{
Function for calculating the elements of the Iota Concept 2
}
\references{
Florian Berding and Julia Pargmann (2022).Iota Reliability Concept
of the Second Generation. Measures for Content Analysis Done by
Humans or Artificial Intelligences. Berlin: Logos.
https://doi.org/10.30819/5581
}
